"""
Title     : Json Schemas
Objective : Define Json Schemas for scraping configuration
Created by: Janid
Created on: 5/13/20
"""

# Standard library imports

# Related third party imports
from pydantic import BaseModel, Field, AnyUrl


# Import local applications


# noinspection PyMissingOrEmptyDocstring
class MinMaxModel(BaseModel):
    min: int = Field(...,
                     title='Min'
                     )
    max: int = Field(...,
                     title='Max'
                     )

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            'min': 'min',
            'max': 'max'
        }


# noinspection PyMissingOrEmptyDocstring
class DriverModel(BaseModel):
    timeout: int = Field(...,
                         title='timeout',
                         description='Timeout in seconds to wait for a WebElement to load',
                         )
    scroll_increment: int = Field(...,
                                  title='Scroll increment',
                                  description='Pixels to scroll for automated scrolling',
                                  )
    click_pause: MinMaxModel = Field(...,
                                     title='Click pause',
                                     description='Pause between clicks'
                                     )
    scroll_pause: MinMaxModel = Field(...,
                                      title='Scroll pause',
                                      )
    home_pause: MinMaxModel = Field(...,
                                    title='Pause at the home page',
                                    )

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            'timeout': 'timeout',
            'scroll_increment': 'scroll_increment',
            'click_pause': 'click_pause',
            'scroll_pause': 'scroll_pause',
            'home_pause': 'home_pause',
        }


# noinspection PyMissingOrEmptyDocstring
class LoginModel(BaseModel):
    login_check_flag: bool = Field(...,
                                 title='if true check login',
                                 )

    login_check_type: str = Field(...,
                                 title='set to "attribute" to check an attribute value. "content" to check content',
                                 min_length=3
                                 )

    login_page: AnyUrl = Field(...,
                               title='Login Page',
                               description='The web address for login',
                               examples='https://www.linkedin.com/login'
                               )
    user: str = Field(...,
                      title='User name',
                      description='User name for login',
                      min_length=3
                      )
    css_user: str = Field(...,
                          title='User CSS Selector',
                          min_length=3
                          )
    password: str = Field(...,
                          title='Password',
                          description='Password for login',
                          min_length=3
                          )
    css_password: str = Field(...,
                              title='Password CSS Selector',
                              min_length=3
                              )
    login_check: str = Field(...,
                             title='Login Check',
                             description='A string used as condition to check that the login has been successful',
                             example='Abdellah Janid',
                             min_length=3
                             )
    css_login_check: str = Field(...,
                                 title='Login Check CSS Selector',
                                 min_length=3
                                 )
    css_login_tag: str = Field(...,
                                 title='html element tag to check',
                                 min_length=3
                                 )
    css_login_attribute: str = Field(...,
                                 title='html element attribute to check',
                                 min_length=3
                                 )
    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            'login_check_flag': 'login_check_flag',
            'login_check_type': 'login_check_type',
            'login_page': 'login_page',
            'user': 'user',
            'css_user': 'css_user',
            'password': 'password',
            'css_password': 'css_password',
            'login_check': 'login_check',
            'css_login_check': 'css_login_check',
            'css_login_tag': 'css_login_tag',
            'css_login_attribute': 'css_login_attribute'
        }


# noinspection PyMissingOrEmptyDocstring
class LinksListModel(BaseModel):
    schools: str = Field(...,
                           title='Schools list name',
                           examples='schools_links_list.csv',
                           min_length=5
                           )
    companies: str = Field(...,
                           title='Companies list name',
                           examples='companies_links_list.csv',
                           min_length=5
                           )

    employees: str = Field(...,
                           title='Employees list name',
                           example='employees_links_list.csv',
                           min_length=5
                           )
    group_members: str = Field(...,
                               title='Group members list name',
                               examples='group_members_links_list.csv',
                               min_length=5
                               )
    my_connections: str = Field(...,
                                title='My connections list name',
                                examples='my_connections_links_list.csv',
                                min_length=5
                                )
    my_followers: str = Field(...,
                              title='My followers list name',
                              example='my_followers_links_list.csv',
                              min_length=5
                              )
    people: str = Field(...,
                        title='People list name',
                        example='people_links_list.csv',
                        min_length=5
                        )
    jobs_list: str = Field(...,
                      title='jobs list list name',
                      examples='jobs_list.csv',
                      min_length=5
                      )
    jobs: str = Field(...,
                      title='jobs links list name',
                      examples='jobs_links_list.csv',
                      min_length=5
                      )

    mutual_connections: str = Field(...,
                      title='mutual_conections links list name',
                      examples='mutual_conections_links_list.csv',
                      min_length=5
                      )
    connections_in_org: str = Field(...,
                      title='connections_in_org links list name',
                      examples='connections_in_org_links_list.csv',
                      min_length=5
                      )
    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            'schools': 'schools',
            'companies': 'companies',
            'employees': 'employees',
            'group_members': 'group_members',
            'my_connections': 'my_connections',
            'my_followers': 'my_followers',
            'people': 'people',
            'jobs_list': 'jobs_list',
            'jobs': 'jobs',
            'mutual_connections': 'mutual_connections',
            'connections_in_org': 'connections_in_org'
        }


class LinkedinConfigSchema(BaseModel):
    """Linkedin configuration Schema"""
    miner: str = Field(...,
                       title='Name',
                       description='The name of the person\'s profile used for scraping',
                       examples='Joe',
                       min_length=2
                       )
    hold_to_queued: bool = Field(...,
                                 title='hold to queued',
                                 description='If True set random number between 15 and 30 to queued',
                                 )
    watchdog: bool = Field(...,
                           title='Watchdog',
                           description='Meant to ensure data validity, '
                                       'If True the scraper will enable extra data validation tools',
                           )
    failed_limit: int = Field(...,
                           title='Watchdog',
                           description='Meant to ensure data validity, '
                                       'If True the scraper will enable extra data validation tools',
                           )
    driver: DriverModel = Field(...)
    login_settings: LoginModel = Field(...)
    links_list: LinksListModel = Field(...)
    get_endorsers_flag: bool = Field(...,
                                     title='Get endorsers',
                                     description='Get skills endorsers on Linkedin, better to keep False, '
                                                 'It activates Linkedin anti bot mechanism',
                                     )
    interest_popup: bool = Field(...,
                                 title='Get interests popup',
                                 )
    popup_limit: int = Field(...,
                             title='Popup limit',
                             description='Limit any popup list on Linkedin to an indicative maximum given number of '
                                         'items to get',
                             )

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Linkedin configuration Schema'
        fields = {
            'miner': 'miner',
            'hold_to_queued': 'hold_to_queued',
            'watchdog': 'watchdog',
            'failed_limit': 'failed_limit',
            'driver': 'driver',
            'login_settings': 'login_settings',
            'links_list': 'links_list',
            "get_endorsers_flag": "get_endorsers_flag",
            "interest_popup": "interest_popup",
            "popup_limit": "popup_limit",

        }
        allow_population_by_field_name = True
