"""
Title     : General schemas used by all other scrapers
Objective :
Created by: Abdellah Janid
Created on: 5/25/20
"""
# Standard library imports
from typing import Optional, Any

# Related third party imports
from pydantic import BaseModel, Field, AnyUrl


# Import local applications


#############################################
# Init
#############################################
# noinspection PyMissingOrEmptyDocstring
class InitModel(BaseModel):
    # variable : type  = Field/schema
    url: AnyUrl = Field(...,
                        title='url being scraped',
                        )

    id: str = Field(...,
                    title='Unique id',
                    description='hex A unique id that could help connecting info to other sources',
                    examples='a82d2eb9461578cf73388347488344e6'
                    )

    miner: str = Field(...,
                       title='Name',
                       description='The name of the person\'s profile used for scraping',
                       examples='janid',
                       min_length=2
                       )
    timestamp_start: str = Field(...,
                                 title="Timestamp start",
                                 description='timestamp when the profile has been scraped in the format "%d-%m-%Y ('
                                             '%H:%M:%S)" same used to name html, json profile, csv start, and inside '
                                             'json init',
                                 examples='24_03_20_(09_43_20)'
                                 )
    timestamp_end: str = Field(...,
                               title="Timestamp end",
                               description='timestamp when the profile has finished being scraped in the format '
                                           '"%d-%m-%Y (%H:%M:%S)" same in csv timestamp_end',
                               examples='24_03_20_(09_43_20)'
                               )

    file_name_source: Optional[str] = Field(...,
                                            title='File name source',
                                            description='The file name from which the link to scrape has been acquired',
                                            examples='a82d2eb9461578cf73388347488344e6_05_11_20_(16_01_03).json'
                                            )

    source_description: Optional[str] = Field(...,
                                              title='Name',
                                              description="Description source is experimental, is meant to trace the "
                                                          "context and origin of the information source",
                                              examples='For a person it can be:'
                                                       '[from]employees:<link>,'
                                                       '[from]companies:https://www.linkedin.com/company/2155/,'
                                                       '[@]event:https://cphfintechweek.com/program/,'
                                                       '[from]person:http://www.linkedin.com/in/janid/,'
                                                       '[when]datetime:15_09_20_(11:05),'
                                                       '[what]tag:#thesis#buisness_intelligence#,'
                                                       'or:'
                                                       '[from]jobs:<link>,'
                                                       '[from]jobs_list:<link>'
                                                       'or:'
                                                       '[from]employees:<link>,'
                                                       '[from]schools:https://www.linkedin.com/school/copenhagen-business-school/about/,'
                                                       '[@]event:https://cphfintechweek.com/program/,'
                                                       '[from]person:http://www.linkedin.com/in/janid/,'
                                                       '[when]datetime:15_09_20_(11:05),'
                                                       '[what]tag:#thesis#buisness_intelligence#,',

                                              min_length=2
                                              )
    tag: Optional[str] = Field(...,
                               title='Tag',
                               description='Hash tag of related topics',
                               examples='#thesis#bmi'
                               )
    info: Optional[str] = Field(...,
                                title='Info',
                                description='Extra info related the link',
                                examples='The employees in the website are different from the employees in linkedin aryze'
                                )
    version: str = Field(...,
                         title="Version",
                         description='Version of scraper package'
                         )

    browser_version: Optional[str] = Field(None,
                                           title="Browser Version",
                                           )
    driver_version: Optional[str] = Field(None,
                                          title="Webdriver Version",
                                          )

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            'url': 'url',
            'id': 'id',
            'miner': 'miner',
            'timestamp_start': 'timestamp_start',
            'timestamp_end': 'timestamp_end',
            'file_name_source': 'file_name_source',
            'source_description': 'source_description',
            'info': 'info',
            'tag': 'tag',
            'version': 'version',
            'browser_version': 'browser_version',
            'driver_version': 'driver_version'

        }
        allow_population_by_field_name = True


class InitSchema(BaseModel):
    """
    Scraping session information
    """
    init: InitModel = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Init'
        fields = {
            'init': 'init'
        }
        allow_population_by_field_name = True


# noinspection PyMissingOrEmptyDocstring
class ExtraModel(BaseModel):
    name: Optional[str] = Field(...)
    len: Optional[int] = Field(...)
    tag: Optional[Any] = Field(...)
    added: Optional[Any] = Field(...)
    other: Optional[Any] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "name": "name",
            "len": "len",
            "tag": "tag",
            "added": "added",
            "other": "other",
        }
        allow_population_by_field_name = True
