"""
Title     : Companies
Objective : Provide tools to scrape data from a list of companies
Created by: Abdellah Janid
Created on: 6/2/20
"""
# Standard library imports
import logging
import random
import re
import urllib.request
from pathlib import Path
from urllib.parse import urljoin, urlparse
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By

from scraper import scraping_tools as mt
# Import local applications
from scraper.linkedin.companies import items
from scraper.session import ScraperSession

# Related third party imports

# Create a module logger
logger_companies_scraper = logging.getLogger('companies_scraper')
# Set level
logger_companies_scraper.setLevel(logging.DEBUG)


class Companies(ScraperSession):
    """Scrape the list of employees of a given company"""

    def get_top_card_info(self):
        """
        Get the employees of a given company
        Returns
        -------

        """
        self.top_card_info = {}
        top_card = self.scroll_select(By.CSS_SELECTOR, 'section.org-top-card',
                                      return_type="bs_el",
                                      html_name="org_top_card",
                                      html_path=self.html_path_profile,
                                      screenshot_name="org_top_card",
                                      screenshot_path=self.screenshots_path_profile,
                                      )
        self.top_card_info = items.get_top_card(top_card)

        self.data.update({"top_card_info": self.top_card_info})
        logger_companies_scraper.debug("top_card_info, done")
        return self.top_card_info

    def get_logo(self):
        """ Get the logo"""
        logo_element = self.select_element(By.CSS_SELECTOR,
                                           'img.org-top-card-primary-content__logo',
                                           return_type="bs_el",
                                           html=False,
                                           screenshot_name=self.file_name + "_logo",
                                           screenshot_path=self.screenshots_path_profile,
                                           )

        if logo_element:
            # select the attribute src in the img tag
            urllib.request.urlretrieve(logo_element.img["src"],
                                       Path(self.media_path,
                                            self.file_name_ts + ".png"))
        logger_companies_scraper.debug("getting logo, done")

    def get_grouped_locations(self):
        """
        Get grouped locations form metadata
        """
        meta_data_grouped_locations = mt.desired_dict("groupedLocations", self.meta_data)
        elements = meta_data_grouped_locations["data"]["elements"]

        # get groupedLocations
        for i in elements:
            if "groupedLocations" in i.keys():
                self.groupedLocations = i["groupedLocations"]

        # Clean the dictionary by removing the key $type
        mt.scrub(self.groupedLocations, bad_key="$type")
        self.data.update({"groupedLocations": self.groupedLocations})
        logger_companies_scraper.debug("groupedLocations, done")

    def get_extra_info(self):
        """
        Get extra data from the meta data
        """
        meta = mt.desired_dict("fundingData", self.meta_data)
        included = meta["included"]
        keys = ["description", "staffCount", "staffCountRange", "phone", "tagline",
                "associatedHashtags", "companyPageUrl", "specialities", "fundingData", "foundedOn", "companyType"]
        for k in keys:
            # noinspection PyBroadException
            try:
                if k in ["description"]:
                    info = list(mt.find_keys(included, k))
                else:
                    info = list(mt.find_keys(included, k))[0]
                mt.scrub(info, "$type")
                self.data.update({k: info})
            except:
                logger_companies_scraper.debug("key not found " + k)
        logger_companies_scraper.debug("Getting extra info, done")

    def get_similar_companies(self):
        """
        Get similar companies in home
        Returns
        -------

        """
        self.similar_companies = []
        similar_section = self.scroll_select(By.XPATH, '//span[text()="See all similar pages"]',
                                                return_type="sel_el",
                                                html_name="similar_section",
                                                html_path=self.html_path_profile,
                                                screenshot_name="similar_section",
                                                screenshot_path=self.screenshots_path_profile,
                                                )
        if similar_section:
            self.click_on(
                method=By.XPATH,
                selector='//span[text()="See all similar pages"]',
                html_name="silimar_pages",
                html_path=self.html_path_profile,
                screenshot_name="silimar_pages",
                screenshot_path=self.screenshots_path_profile
            )

            similar_companies = self.select_elements(By.CSS_SELECTOR, 'div.reusable-org-list-with-modal__card'
                                                        , return_type="bs_el")
            self.similar_companies = list(map(items.get_similar_companies_info, similar_companies))

            #Close popup
            self.click_on(
                method=By.CSS_SELECTOR,
                selector='button.artdeco-modal__dismiss',
                html_name="silimar_pages",
                html_path=self.html_path_profile,
                screenshot_name="silimar_pages",
                screenshot_path=self.screenshots_path_profile
            )
        self.data.update({"similar_companies": self.similar_companies})
        logger_companies_scraper.debug("similar_companies, done")
        return self.similar_companies

    def scrape(self):
        """
        Returns
        -------
            data : dict
                see profile data structure for more details

        """
        # Define the status
        self.scraping_status = "running"

        o = urlparse(self.url)
        start, end = 0, 3
        self.url = o.path
        self.url = "https://www.linkedin.com" + '/'.join(self.url.split('/')[start:end]) + '/' + 'about/'


        logger_companies_scraper.debug('Scraping url : ' + self.url)
        # Load the profile
        self.load_page(By.CSS_SELECTOR, ".scaffold-layout-container",
                       By.CSS_SELECTOR, ".artdeco-empty-state__headline",
                       "/company/")

        # Get info from the top banner
        self.get_top_card_info()
        self.get_logo()
        self.get_meta_data("start")
        self.get_grouped_locations()
        self.get_extra_info()
        self.get_similar_companies()
        # Add extra dictionary info
        self.extra = {
            "name": None,
            "len": None,
            "tag": None,
            "added": None,
            "other": None
        }
        self.data.update({"extra": self.extra})
        # -----------------------------------------------------------------------
        # Save html of tabs for future inspections
        # -----------------------------------------------------------------------
        tabs = self.select_elements(By.CSS_SELECTOR, 'nav.org-page-navigation ul > li')
        random.shuffle(tabs)
        for tab in tabs:
            if tab.text == "More":
                continue
            logger_companies_scraper.debug("getting tab : " + tab.text)
            tab.click()
            page = self.select_element(method=By.CSS_SELECTOR, selector='a.active', web_element=tab)

            if page:
                mt.save_html(self.driver.page_source, self.html_path_profile, "full_page_" + tab.text)
                self.get_meta_data(tab.text.lower())

            mt.random_rest(self.config["driver"]["click_pause"]["min"],
                           self.config["driver"]["click_pause"]["max"])

        self.scraping_status = "completed"
