"""
Title     : Company employees
Objective : Provide tools to scrape the list of employees of a given company
Created by: Abdellah Janid
Created on: 6/2/20
"""
# Standard library imports
import logging
from pathlib import Path

import pandas as pd
# Related third party imports
from selenium.webdriver.common.by import By

from scraper.linkedin.connections_in_org import items
# Import local applications
from scraper.session import ScraperSession

# Create a module logger
logger_connections_in_org_scraper = logging.getLogger('connections_in_org_scraper')
# Set level
logger_connections_in_org_scraper.setLevel(logging.DEBUG)


class ConnectionsInOrg(ScraperSession):
    """Scrape the list of mutual connections link of given person"""

    def get_connections_in_org(self):
        """
        Get connections in an organization
        Returns
        -------

        """
        self.connections_in_org = []
        self.get_data_from_pagination(
            section="connections_in_org",
            method=By.CSS_SELECTOR,
            selector=".search-results-container",
            list_method=By.CSS_SELECTOR,
            list_selector="li.reusable-search__result-container",
            get_info_function=items.get_mutual_connections_info,
            page_nr_method=By.CSS_SELECTOR,
            page_nr_selector='.artdeco-pagination__indicator--number.active',
            page_nr_attribute='data-test-pagination-page-btn',
            tot_pages_method=By.CSS_SELECTOR,
            tot_pages_selector='li.artdeco-pagination__indicator--number',
            next_method=By.CSS_SELECTOR,
            next_selector='button[aria-label="Next"]:not(.artdeco-button--disabled)',
            info=self.connections_in_org
        )
        self.extra = {
            "name": "connections_in_org",
            "len": len(self.connections_in_org),
            "tag": None,
            "added": None,
            "other": None
        }
        self.data.update({"extra": self.extra})
        self.data.update({"connections_in_org": self.connections_in_org})

    def select_second_degree(self):
        logger_connections_in_org_scraper.debug("select second degree")
        # Open filter popup
        self.click_on(By.CSS_SELECTOR,
                      'button[aria-controls="connections-facet-values"]',
                      html_name=self.file_name + "_connections_filter",
                      html_path=self.html_path_profile,
                      screenshot_name=self.file_name + "_connections_filter",
                      screenshot_path=self.screenshots_path_profile
                      )

        # Select second degree
        self.click_on(By.CSS_SELECTOR,
                      'label[for="network-S"]',
                      html_name=self.file_name + "_second_degree",
                      html_path=self.html_path_profile,
                      screenshot_name=self.file_name + "_second_degree",
                      screenshot_path=self.screenshots_path_profile,
                      )
        # Apply
        self.click_on(By.CSS_SELECTOR,
                      '.search-s-facet__values--network button.facet-collection-list__apply-button',
                      html_name=self.file_name + "_apply_filter",
                      html_path=self.html_path_profile,
                      screenshot_name=self.file_name + "_apply_filter",
                      screenshot_path=self.screenshots_path_profile,
                      )

    def scrape(self):
        """
        Returns
        -------
            data : dict
                see profile data structure for more details

        """
        # Define the status
        self.scraping_status = "running"

        # Load the profile
        self.load_page(By.CSS_SELECTOR, ".search-results-container", By.CSS_SELECTOR, ".search-no-results__container", "/search/")

        # If command is 2 select second degree
        # if self.csv_command:
        #     if int(self.csv_command) == 2:
        #         self.select_second_degree()

        self.get_connections_in_org()
        if self.connections_in_org:
            # Saving csv
            logger_connections_in_org_scraper.debug(
                "saving mutual_connections csv:{}".format(len(self.connections_in_org)))
            df = pd.DataFrame(self.data["connections_in_org"])
            df_shuffled = df.sample(frac=1)
            df_shuffled.to_csv(Path(self.profiles_path, self.file_name + "_" + self.init["timestamp_start"] + ".csv"),
                               index=False, sep=";")
            self.extra_info = len(self.connections_in_org)
            # Check that at least one url is there
            urls = [p["url"] for p in self.connections_in_org]

            if all(u is None for u in urls):
                logger_connections_in_org_scraper.info("Failed, all urls are None")
                raise ValueError("Failed, all urls are None")

            logger_connections_in_org_scraper.info("done , " +
                                                   " , " +
                                                   self.data["init"]["id"] +
                                                   " , " +
                                                   self.data["init"]["url"])
        else:
            logger_connections_in_org_scraper.info("Failed getting mutual_connections")
            raise ValueError("Failed getting mutual_connections")

        self.scraping_status = "completed"
