"""
Title     : Company employees
Objective : Provide tools to scrape the list of employees of a given company
Created by: Abdellah Janid
Created on: 6/2/20
"""
# Standard library imports
import logging
from pathlib import Path

import pandas as pd
# Related third party imports
from selenium.webdriver.common.by import By

from scraper.linkedin.employees import items
# Import local applications
from scraper.session import ScraperSession

# Create a module logger
logger_employees_scraper = logging.getLogger('employees_scraper')
# Set level
logger_employees_scraper.setLevel(logging.DEBUG)


class Employees(ScraperSession):
    """Scrape the list of employees of a given company"""

    def get_employees(self):
        """
        Get the employees of a given company
        Returns
        -------

        """
        self.employees = []
        self.get_data_from_pagination(
            section="employees",
            method=By.CSS_SELECTOR,
            selector=".search-results-container",
            list_method=By.CSS_SELECTOR,
            list_selector="li.reusable-search__result-container",
            get_info_function=items.get_employees,
            page_nr_method=By.CSS_SELECTOR,
            page_nr_selector='.artdeco-pagination__indicator--number.active',
            page_nr_attribute='data-test-pagination-page-btn',
            tot_pages_method=By.CSS_SELECTOR,
            tot_pages_selector='li.artdeco-pagination__indicator--number',
            next_method=By.CSS_SELECTOR,
            next_selector='button[aria-label="Next"]:not(.artdeco-button--disabled)',
            info=self.employees
        )
        self.extra = {
            "name": "employees",
            "len": len(self.employees),
            "tag": None,
            "added": None,
            "other": None
        }
        self.data.update({"extra": self.extra})
        self.data.update({"employees": self.employees})

    def scrape(self):
        """
        Returns
        -------
            data : dict
                see profile data structure for more details

        """
        # Define the status
        self.scraping_status = "running"

        # Load the profile
        self.load_page(By.CSS_SELECTOR, ".search-results-container", By.CSS_SELECTOR, ".search-no-results__container",
                       "/search/")
        self.get_employees()
        if self.employees:
            # Saving csv
            logger_employees_scraper.debug("saving employees csv:{}".format(len(self.employees)))
            df = pd.DataFrame(self.data["employees"])
            df_shuffled = df.sample(frac=1)
            df_shuffled.to_csv(Path(self.profiles_path, self.file_name + "_" + self.init["timestamp_start"] + ".csv"),
                               index=False, sep=";")
            self.extra_info = len(self.employees)

            # Check that at least one url is there
            urls = [p["url"] for p in self.employees]
            if all(u == None for u in urls):
                logger_employees_scraper.info("Failed getting employees, all urls are None")
                raise ValueError("Failed getting employees, all urls are None")

            logger_employees_scraper.info("done , " +
                                          " , " +
                                          self.data["init"]["id"] +
                                          " , " +
                                          self.data["init"]["url"])
        else:
            logger_employees_scraper.info("Failed getting employees")
            raise ValueError("Failed getting employees")

        self.scraping_status = "completed"
