"""
Title     : TODO
Objective : Get jobs_list info
Created by: Abdellah Janid
Created on: 5/26/20
"""
# Standard library imports


# Related third party imports
from bs4 import BeautifulSoup

# Import local applications
from scraper import scraping_tools as mt


def get_top_card(soup: BeautifulSoup):
    """
    Get the top card of the job description
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        title       : str
            Job title
        company      : str, default None
            Company name
        location        : str, default None
            location of the job offer
        posting_date        : str, default None
            posting date
        nr_applicants     : str, default None
            number of applicants
        message      : str, default None
            extra info on the job
        url_company      : str, default None
            url id the company
    """
    info = mt.get_info(soup, {
        "title": "h1",
        "company": ".jobs-unified-top-card__subtitle-primary-grouping span",
        "location": ".jobs-unified-top-card__subtitle-primary-grouping span:nth-of-type(2)",
        "posting_date": ".jobs-unified-top-card__posted-date",
        "nr_applicants": ".jobs-unified-top-card__applicant-count",
        "message": "span.artdeco-inline-feedback__message"

    })

    info["url_company"] = mt.get_attribute_value(soup, ".jobs-unified-top-card__subtitle-primary-grouping span a", "href")
    if info['url_company'] is not None:
        info['url_company'] = mt.validate_link(info['url_company'])
    return info


def get_jobs_poster(soup: BeautifulSoup):
    """
    Get job poster info
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        full_name       : str
            Full name of a person
        headline        : str, default None
            headline title.
        location        : str, default None
            The location of the person.
        distance     : str, default None
            distance to the job poster
        url     : str, default None
            linkedin profile
    """
    info = mt.get_info(soup, {
        "full_name": ".jobs-poster__name",
        "headline": ".jobs-poster__headline",
        "distance": ".jobs-poster__name .distance"
    })

    info["url"] = mt.get_attribute_value(soup, "a.ember-view", "href")
    if info['url'] is not None:
        info['url'] = "https://www.linkedin.com" + info['url']

    if info['full_name'] is not None:
        info['full_name'] = info["full_name"].split("\n")[0].strip()
    return info


def get_top_skills(soup: BeautifulSoup):
    """
    Get top skills
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        skill       : str
            skill name
        satisfied      : bool, default None
            True if the skill is satisfied, False if not
    """
    info = mt.get_info(soup, {
        "skill": "p"
    })
    if soup.select_one(".jobs-premium-applicant-insights__skill--matched") is not None:
        info["satisfied"] = True
    else:
        info["satisfied"] = False
    return info


def get_seniority_level(soup: BeautifulSoup):
    """
    Get seniority level
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        level       : str
            Seniority level of the applicants
        max      : bool, default None
            True if the skill satisfied, False if not
    """
    info = mt.get_info(soup, {
        "level": "p"
    })
    info["max"] = mt.get_attribute_value(soup, "progress", "max")
    return info


def get_education(soup: BeautifulSoup):
    """
    Get education
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        num       : str
            number of people with a given qualification
        qualification      : bool, default None
            qualification

    """
    info = mt.get_info(soup, {
        "num": ".t-32",
        "qualification": ".jobs-premium-applicant-insights__list-description"
    })
    return info


def get_location(soup: BeautifulSoup):
    """
    Get location
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        number       : str
            Seniority level of the applicants
        location      : bool, default None
            True if the skill satisfied, False if not
    """
    info = mt.get_info(soup, {
        "number": "span.jobs-premium-applicant-insights__top-locations-number",
        "location": ".jobs-premium-applicant-insights__top-locations-title"
    })
    if info["number"] is not None:
        info["number"] = info["number"].replace("applicants", "").strip()
    return info


def get_similar_jobs(soup: BeautifulSoup):
    """
    Get similar jobs
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        title       : str
            Job title
        company      : str, default None
            Company name
        location        : str, default None
            location of the job offer
        posting_date        : str, default None
            posting date
        nr_applicants     : str, default None
            number of applicants
        message      : str, default None
            extra info on the job
        url_company      : str, default None
            url id the company
    """
    info = mt.get_info(soup, {
        "title": ".job-card__title",
        "company": "h4.t-14",
        "location": "h5.job-card__location",
        "posting_date": ".job-card__time-badge",
        "flavors": ".job-card__job-flavors-container"
    })
    if info['location'] is not None:
        info['location'] = info['location'].replace("Job Location","").strip()

    info["url"] = mt.get_attribute_value(soup, "a.job-card__link-wrapper", "href")
    if info['url'] is not None:
        info['url'] = "https://www.linkedin.com" + info['url']

    if info['flavors'] is not None:
        flavor = mt.get_attribute_value(soup, "div.job-flavors__logo-container img", "title")
        if flavor is not None:
            info['flavors'] = flavor + " " + info['flavors']
    return info
