"""
Title     : Jobs
Objective : Scrape data from linkedin's link of jobs_list search
Created by: Janid
Created on: 5/25/20
"""

# Standard library imports
import logging

# Related third party imports
from selenium.webdriver.common.by import By

# Import local applications
from scraper import scraping_tools as mt
from scraper.linkedin.jobs import items
from scraper.session import ScraperSession

# Create a module logger
logger_jobs_scraper = logging.getLogger('jobs')
# Set level
logger_jobs_scraper.setLevel(logging.DEBUG)


class Jobs(ScraperSession):
    """Contains the tools to scrape data on a jobs from a link on linkedin"""

    def get_top_card_info(self):
        """
        Get the top card
        """
        logger_jobs_scraper.debug("session, started, top_card_info")
        self.top_card_info = {}
        # Select top-card: pv-top-card
        top_card = self.scroll_select(By.CSS_SELECTOR, 'div.jobs-unified-top-card',
                                      return_type="bs_el",
                                      html_name="pv_top_card",
                                      html_path=self.html_path_profile,
                                      screenshot_name="pv_top_card",
                                      screenshot_path=self.screenshots_path_profile,
                                      )
        # get top card info
        self.top_card_info = items.get_top_card(top_card)
        self.data.update({"top_card_info": self.top_card_info})

        logger_jobs_scraper.debug("session, finished, top_card_info")

    def get_jobs_poster_info(self):
        """
        Get the job poster info
        """
        logger_jobs_scraper.debug("session, started, jobs_poster")
        self.jobs_poster = {}
        # Select top-card: pv-top-card
        jobs_poster = self.scroll_select(By.CSS_SELECTOR, 'div.jobs-description__details .jobs-poster',
                                         return_type="bs_el",
                                         html_name="jobs_poster",
                                         html_path=self.html_path_profile,
                                         screenshot_name="jobs_poster",
                                         screenshot_path=self.screenshots_path_profile,
                                         )
        if jobs_poster:
            self.jobs_poster = items.get_jobs_poster(jobs_poster)
        self.data.update({"jobs_poster": self.jobs_poster})

        logger_jobs_scraper.debug("session, finished, jobs_poster")

    def get_description_details_info(self):
        """
        Get description details
        """
        logger_jobs_scraper.debug("session, started, description_details")
        self.description_details = {}
        description_details_el = self.scroll_select(By.CSS_SELECTOR, 'div.jobs-unified-top-card',
                                                       return_type="bs_el",
                                                       html_name="description_details",
                                                       html_path=self.html_path_profile,
                                                       screenshot_name="description_details",
                                                       screenshot_path=self.screenshots_path_profile,
                                                       )

        description_titles = [d["type"] for d in
                              description_details_el.select(".jobs-unified-top-card__job-insight li-icon")]
        insights = description_details_el.select(".jobs-unified-top-card__job-insight")
        for i, e in enumerate(insights):
            info = e.select_one("span").get_text().strip()
            if "briefcase-icon" in str(e):
                if "·" in info:
                    self.description_details["employment_type"] = info.split("·")[0].strip()
                    self.description_details["seniority_level"] = info.split("·")[1].strip()
                else:
                    self.description_details["employment_type"] = info.strip()
            if "company-icon" in str(e):
                if "·" in info:
                    self.description_details["staffCount"] = info.split("·")[0].strip()
                    self.description_details["industry"] = info.split("·")[1].strip()
                else:
                    self.description_details["company_icon"] = info.strip()
            if "people-icon" in str(e):
                self.description_details["people_icon"] = info.split("·")[0].strip()
            if "radar-screen-icon" in str(e):
                self.description_details["actively_recruiting"] = True

        self.data.update({"description_details": self.description_details})

        logger_jobs_scraper.debug("session, finished, description_details")

    # Commented because it is old, but I will keep it in case it comes back
    # def get_description_details_info(self):
    #     """
    #     Get description details
    #     """
    #     logger_jobs_scraper.debug("session, started, description_details")
    #     self.description_details = {}
    #     # Select top-card: pv-top-card
    #     description_details_el = self.scroll_select(By.CSS_SELECTOR, 'div.jobs-description__details',
    #                                                 return_type="bs_el",
    #                                                 html_name="description_details",
    #                                                 html_path=self.html_path_profile,
    #                                                 screenshot_name="description_details",
    #                                                 screenshot_path=self.screenshots_path_profile,
    #                                                 )
    #
    #     description_titles = [d.get_text().strip() for d in description_details_el.select("h3")]
    #     self.description_details.update({"description_titles": description_titles})
    #
    #     groups = description_details_el.select(".jobs-box__group")
    #
    #     # get top card info
    #     for g in groups:
    #         info = g.select("p")
    #         if info:
    #             self.description_details[mt.get_text(g, "h3").lower().replace(" ", "_")] = [mt.get_text(g, "p")]
    #         else:
    #             self.description_details[mt.get_text(g, "h3").lower().replace(" ", "_")] = [d.get_text().strip() for d
    #                                                                                         in g.select("li")]
    #
    #     self.data.update({"description_details": self.description_details})
    #
    #     logger_jobs_scraper.debug("session, finished, description_details")
    def get_description_info(self):
        """
        Get job description info
        """
        logger_jobs_scraper.debug("session, started, description")
        description = self.scroll_select(By.CSS_SELECTOR, 'div.jobs-description-content',
                                         return_type="bs_el",
                                         html_name="description",
                                         html_path=self.html_path_profile,
                                         screenshot_name="description",
                                         screenshot_path=self.screenshots_path_profile,
                                         )
        # get description
        description = mt.get_text(description, ".jobs-description-content__text")
        self.data.update({"description": description})

        logger_jobs_scraper.debug("session, finished, description")

    def get_applicant_rank_info(self):
        """
        Get applicant rank info from insights
        """
        logger_jobs_scraper.debug("session, started, applicant-rank")
        applicant_rank = {}
        applicant_rank_el = self.scroll_select(By.CSS_SELECTOR, '.jobs-details-premium-insight.applicant-rank',
                                               return_type="bs_el",
                                               html_name="applicant_rank",
                                               html_path=self.html_path_profile,
                                               screenshot_name="applicant_rank",
                                               screenshot_path=self.screenshots_path_profile,
                                               )

        if applicant_rank_el:
            if "Top applicants" in applicant_rank_el.h3.get_text():
                applicant_rank["rank_value"] = mt.get_text(applicant_rank_el, ".t-normal-70")
                # groups = applicant_rank_el.select("li")
                # for g in groups:
                #     applicant_rank[mt.get_text(g, "p").lower().replace(" ", "_")] = mt.get_text(g, "span")

            if "Applicants for this job" in applicant_rank_el.h3.get_text():
                groups = applicant_rank_el.select("li")
                for g in groups:
                    applicant_rank[
                        mt.get_text(g, ".jobs-premium-applicant-insights__list-description").lower().replace(" ",
                                                                                                             "_")] = mt.get_text(
                        g, ".jobs-premium-applicant-insights__list-num")

        self.data.update({"applicant_rank": applicant_rank})
        logger_jobs_scraper.debug("session, finished, applicant-rank ")

    def get_top_skills_info(self):
        """
        Get top skills
        """
        top_skills = []
        logger_jobs_scraper.debug("session, started, top skills")
        top_skills_el = self.scroll_select(By.CSS_SELECTOR, '.jobs-details-premium-insight.top-skills',
                                           return_type="bs_el",
                                           html_name="top_skills",
                                           html_path=self.html_path_profile,
                                           screenshot_name="top_skills",
                                           screenshot_path=self.screenshots_path_profile,
                                           )

        if top_skills_el:
            top_skills_els = top_skills_el.select(".jobs-premium-applicant-insights__list-skill-item")
            top_skills = list(map(items.get_top_skills, top_skills_els))

        self.data.update({"top_skills": top_skills})

        logger_jobs_scraper.debug("session, finished, top skills")

    def get_seniority_level_info(self):
        """
        Get Seniority level
        """
        logger_jobs_scraper.debug("session, started, seniority level")
        self.seniority_level = []
        seniority_level_el = self.scroll_select(By.CSS_SELECTOR, '.jobs-details-premium-insight.applicant-experience',
                                                return_type="bs_el",
                                                html_name="seniority_level",
                                                html_path=self.html_path_profile,
                                                screenshot_name="seniority_level",
                                                screenshot_path=self.screenshots_path_profile,
                                                )

        if seniority_level_el:
            groups = seniority_level_el.select("li")
            self.seniority_level = list(map(items.get_seniority_level, groups))
        self.data.update({"seniority_level": self.seniority_level})
        logger_jobs_scraper.debug("session, finished, seniority_level")

    def get_education_info(self):
        """
        Get Education info
        """
        logger_jobs_scraper.debug("session, started, education")
        self.education = []
        education_el = self.scroll_select(By.CSS_SELECTOR, '.jobs-details-premium-insight.applicant-education',
                                          return_type="bs_el",
                                          html_name="education",
                                          html_path=self.html_path_profile,
                                          screenshot_name="education",
                                          screenshot_path=self.screenshots_path_profile,
                                          )

        if education_el:
            groups = education_el.select("li.jobs-premium-applicant-insights__list-item")
            self.education = list(map(items.get_education, groups))
        self.data.update({"education": self.education})
        logger_jobs_scraper.debug("session, finished, education")

    def get_location_info(self):
        """
        Get location
        """
        logger_jobs_scraper.debug("session, started, location")
        self.location = []
        location_el = self.scroll_select(By.CSS_SELECTOR, '.jobs-details-premium-insight.top-locations',
                                         return_type="bs_el",
                                         html_name="location",
                                         html_path=self.html_path_profile,
                                         screenshot_name="location",
                                         screenshot_path=self.screenshots_path_profile,
                                         )

        if location_el:
            groups = location_el.select("li.jobs-premium-applicant-insights__top-locations-item")
            self.location = list(map(items.get_location, groups))
        self.data.update({"location": self.location})
        logger_jobs_scraper.debug("session, finished, location")

    def get_similar_jobs_info(self):
        """
        Get similar jobs
        """
        logger_jobs_scraper.debug("session, started, similar jobs")
        self.similar_jobs = []
        similar_jobs_el = self.scroll_select(By.CSS_SELECTOR, 'div.jobs-similar-jobs',
                                             return_type="bs_el",
                                             html_name="similar_jobs",
                                             html_path=self.html_path_profile,
                                             screenshot_name="similar_jobs",
                                             screenshot_path=self.screenshots_path_profile,
                                             )

        if similar_jobs_el:
            groups = similar_jobs_el.select("li.jobs-similar-jobs__list-item")
            self.similar_jobs = list(map(items.get_similar_jobs, groups))
        self.data.update({"similar_jobs": self.similar_jobs})
        logger_jobs_scraper.debug("session, finished, similar_jobs")

    def get_more_similar_jobs(self):
        """
        Get similar jobs
        """
        more_similar = None
        logger_jobs_scraper.debug("session, started, link to more similar jobs")
        more_similar_jobs_el = self.scroll_select(By.CSS_SELECTOR, 'a.jobs-similar-jobs__see-more-btn',
                                                  return_type="bs_el",
                                                  html_name="more_similar_jobs",
                                                  html_path=self.html_path_profile,
                                                  screenshot_name="more_similar_jobs",
                                                  screenshot_path=self.screenshots_path_profile,
                                                  )

        if more_similar_jobs_el:
            more_similar = "https://www.linkedin.com" + more_similar_jobs_el.a["href"]

        self.data.update({"more_similar_jobs": more_similar})
        logger_jobs_scraper.debug("session, finished, more similar_jobs")

    def get_extra_info(self):
        """
        Get data from meta data
        """
        # Init jobs_list container
        logger_jobs_scraper.debug("session, started, meta data")

        meta = mt.desired_dict("dashEntityUrn", self.meta_data)

        keys = ["logo", "attributes", "$type", "$recipeTypes", "baseType", "listedAt"]
        meta["data"]["companyDetails"] = meta["included"]
        meta["data"]["description"] = meta["data"]["description"]["text"]
        meta["data"].pop("entityUrn")
        meta.pop("meta")
        mt.scrub(meta, "included")
        for k in keys:
            try:
                mt.scrub(meta, k)
            except:
                logger_jobs_scraper.debug("key not found " + k)

        self.data.update({"meta_data": meta["data"]})
        logger_jobs_scraper.debug("Getting meta data , done")

    def scrape(self):
        """
        Returns
        -------
            data : dict
                see profile data structure for more details

        """
        # Define the status
        self.scraping_status = "running"
        # Load the profile
        self.load_page(By.CSS_SELECTOR, ".job-view-layout", By.CSS_SELECTOR, ".not-found", "/jobs/view/")
        self.get_meta_data("start")
        self.extra = {
            "name": "jobs",
            "len": None,
            "tag": None,
            "added": None,
            "other": None
        }
        self.data.update({"extra": self.extra})
        # Get top card
        self.get_top_card_info()
        # Get jobs poster info
        self.get_jobs_poster_info()
        # Get description details
        # self.get_description_details_info()
        self.get_description_details_info()
        self.get_description_info()
        self.get_applicant_rank_info()
        self.get_top_skills_info()
        self.get_seniority_level_info()
        self.get_education_info()
        self.get_location_info()
        self.get_similar_jobs_info()
        self.get_more_similar_jobs()
        # Get meta data
        self.get_meta_data()
        self.get_extra_info()
        self.save_html_and_png_full_page()

        self.scraping_status = "completed"
