"""
Title     : Company employees
Objective : Provide tools to scrape the list of employees of a given company
Created by: Abdellah Janid
Created on: 6/2/20
"""
# Standard library imports
import logging
from pathlib import Path

import pandas as pd
# Related third party imports
from selenium.webdriver.common.by import By

from scraper.linkedin.mutual_connections import items
# Import local applications
from scraper.session import ScraperSession

# Create a module logger
logger_mutual_connections_scraper = logging.getLogger('mutual_connections_scraper')
# Set level
logger_mutual_connections_scraper.setLevel(logging.DEBUG)


class MutualConnections(ScraperSession):
    """Scrape the list of mutual connections link of given person"""

    def get_mutual_connections(self):
        """
        Get mutual connections of a given person
        Returns
        -------

        """
        self.mutual_connections = []
        self.get_data_from_pagination(
            section="mutual_connections",
            method=By.CSS_SELECTOR,
            selector=".search-results-container",
            list_method=By.CSS_SELECTOR,
            list_selector="li.reusable-search__result-container",
            get_info_function=items.get_mutual_connections_info,
            page_nr_method=By.CSS_SELECTOR,
            page_nr_selector='.artdeco-pagination__indicator--number.active',
            page_nr_attribute='data-test-pagination-page-btn',
            tot_pages_method=By.CSS_SELECTOR,
            tot_pages_selector='li.artdeco-pagination__indicator--number',
            next_method=By.CSS_SELECTOR,
            next_selector='button[aria-label="Next"]:not(.artdeco-button--disabled)',
            info=self.mutual_connections
        )

        # Get the name of the person with mutual connections
        element = self.driver.find_element(By.CSS_SELECTOR,
                                           "nav li:nth-of-type(4) li.search-reusables__collection-values-item input")
        name = element.get_attribute("name")

        self.extra = {
            "name": name,
            "len": len(self.mutual_connections),
            "tag": None,
            "added": None,
            "other": None
        }
        self.data.update({"extra": self.extra})
        self.data.update({"mutual_connections": self.mutual_connections})

    def select_second_degree(self):
        logger_mutual_connections_scraper.debug("select second degree")
        # Open filter popup
        self.click_on(By.CSS_SELECTOR,
                      'button[aria-controls="connections-facet-values"]',
                      html_name=self.file_name + "_connections_filter",
                      html_path=self.html_path_profile,
                      screenshot_name=self.file_name + "_connections_filter",
                      screenshot_path=self.screenshots_path_profile
                      )

        # Select second degree
        self.click_on(By.CSS_SELECTOR,
                      'label[for="network-S"]',
                      html_name=self.file_name + "_second_degree",
                      html_path=self.html_path_profile,
                      screenshot_name=self.file_name + "_second_degree",
                      screenshot_path=self.screenshots_path_profile,
                      )
        # Apply
        self.click_on(By.CSS_SELECTOR,
                      '.search-s-facet__values--network button.facet-collection-list__apply-button',
                      html_name=self.file_name + "_apply_filter",
                      html_path=self.html_path_profile,
                      screenshot_name=self.file_name + "_apply_filter",
                      screenshot_path=self.screenshots_path_profile,
                      )

    def scrape(self):
        """
        Returns
        -------
            data : dict
                see profile data structure for more details

        """
        # Define the status
        self.scraping_status = "running"

        # Load the profile
        self.load_page(By.CSS_SELECTOR, ".search-results-container", By.CSS_SELECTOR, ".search-no-results__container", "/search/")

        # If command is 2 select second degree
        if self.csv_command:
            if int(self.csv_command) == 2:
                self.select_second_degree()

        self.get_mutual_connections()
        if self.mutual_connections:
            # Saving csv
            logger_mutual_connections_scraper.debug(
                "saving mutual_connections csv:{}".format(len(self.mutual_connections)))
            df = pd.DataFrame(self.data["mutual_connections"])
            df_shuffled = df.sample(frac=1)
            df_shuffled.to_csv(Path(self.profiles_path, self.file_name + "_" + self.init["timestamp_start"] + ".csv"),
                               index=False, sep=";")
            self.extra_info = len(self.mutual_connections)
            logger_mutual_connections_scraper.info("done , " +
                                                   " , " +
                                                   self.data["init"]["id"] +
                                                   " , " +
                                                   self.data["init"]["url"])
        else:
            logger_mutual_connections_scraper.info("Failed getting mutual_connections")
            raise ValueError("Failed getting mutual_connections")

        self.scraping_status = "completed"
