"""
Title     : TODO
Objective : TODO
Created by: Abdellah Janid
Created on: 5/26/20
"""
# Standard library imports
import re

# Related third party imports
from bs4 import BeautifulSoup

# Import local applications
from scraper import scraping_tools as mt


def get_top_card(soup: BeautifulSoup):
    """
    Get top card info
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        full_name       : str
            The full name of the person.
        dist_value      : str, default None
            The distance to the person.
        headline        : str, default None
            headline title.
        location        : str, default None
            The location of the person.
        connections     : str, default None
            The connections the person has on linkedin.
    """
    info = mt.get_info(soup, {
        "full_name": "h1.text-heading-xlarge",
        "dist_value": ".dist-value",
        "headline": ".text-body-medium",
        "tags": ".pv-text-details__left-panel .text-body-small.break-words span",
        "location": ".pb2 .text-body-small",
        "connections": ".pv-top-card--list-bullet span"
    })

    return info

def get_followers_count(soup):
    """
    Used to get info from tabs like companies

    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
        a dictionaries with the following {keys: values}
            followers   : int, default None


    """
    info = mt.get_info(soup, {
        'followers': '.pv-recent-activity-section__follower-count'
    })

    if info['followers']:
        info['followers'] = int(
            info['followers'].replace("followers", "").replace("members", "").replace(",", "").strip())

    return info


def get_extra_top_card(soup):
    """
    Used to get info from tabs like companies

    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
        a dictionaries with the following {keys: values}
            followers   : int, default None


    """
    info = mt.get_info(soup, {
        mt.get_text(soup, "h3").replace(" ", "_").lower(): 'p'
    })

    for k in list(info.keys()):
        if k == "start_date":
            continue
        info[k] = info[k].split("·")
        info[k] = [e.strip() for e in info[k]]
    return info


def get_extra_top_card_jobs(soup: BeautifulSoup):
    """
    Returns:
        dict
    """
    info = mt.get_info(soup, {
        'title': '.job-card-list__title',
        'company': '.job-card-container__company-name',
        'location': '.job-card-container__metadata-item'
    })

    info["url_job"] = mt.get_attribute_value(soup, "a.job-card-list__title", "href")
    if info['url_job'] is not None:
        info['url_job'] = "https://www.linkedin.com" + info['url_job']
    info["posting_date"] = mt.get_attribute_value(soup, "li.job-card-container__listed-time time", "datetime")
    return [info]


def get_contact_info(soup: BeautifulSoup):
    """
    Get contact info
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        address   : str, default None
            Address of the person
        email     : str, default None
            Email pf the person
        birthday  : str, default None
            Birthday of the person in the format "%B %d"
        phone     : str, default None
            Phone number
        connected': str, default None
            Date in which the scraper has connected with the person in the format "%B %d, %Y"
        twitter'  : str, default None
            Twitter account of the person
    """
    info = mt.get_info(soup, {
        "address": ".ci-address .pv-contact-info__ci-container",
        "email": ".ci-email .pv-contact-info__ci-container",
        "birthday": ".ci-birthday .pv-contact-info__ci-container",
        "phone": '.ci-phone .pv-contact-info__ci-container',
        "connected": ".ci-connected .pv-contact-info__ci-container",
        "twitter": ".ci-twitter .pv-contact-info__ci-container"
    })

    return info


# test if summary is None
def get_summary(soup: BeautifulSoup):
    """
    Get attendees from the attendees page
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        summary   : str, default None
            Address of the person

    """
    info = mt.get_info(soup, {
        "summary": '.pv-about-section'
    })
    # noinspection PyBroadException
    try:
        info["summary"] = info["summary"].replace('... see more', '').strip()
    except:
        pass

    return info


# Test no mutual connections
def get_highlights(soup: BeautifulSoup):
    """
    Get attendees from the attendees page
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        summary   : str, default None
            Address of the person

    """
    info = mt.get_info(soup, {
        "mutual_connections": 'a.app-aware-link strong'
    })
    # noinspection PyBroadException
    try:
        # TODO there was a bug, if a connection is one, mutual connections should be used. recover info from html
        # TODO, classify None data, create a statistical test and check html to recover lost data
        # https://www.linkedin.com/in/sofie-strecker-a4686350/
        info["mutual_connections"] = info["mutual_connections"].replace('mutual connections:', '').replace(
            'mutual connection:', '').strip()
        info["mutual_connections"] = int(info["mutual_connections"])
    except:
        info["mutual_connections"] = None
    # noinspection PyBroadException
    try:
        info["url_connections"] = mt.get_attribute_value(soup, "a.app-aware-link", "href")
    except:
        info["url_connections"] = None
        pass

    return info


def get_experience_info(soup: BeautifulSoup):
    """
    Get experience info
    Returns
    -------
    info : dict
        Returns a list of dictionaries with the following {keys: values}

        title           : str, default None  [Must have]
            role in the job position.

        employment_type : str, default None.
            Full-time, Part-time, Self-employed, Freelance, Contract, Internship, Apprenticeship

        date_range      : str, default None
            date rage on the job position.

        location        : str, default None
            location of of the job position.

        description     : str, default None
            job description.

        organization    : str, default None
            organization name.

        li_org_url      : str, default None
            linkedin url of the organization.
    """
    position_elements = mt.select_all(soup, '.pv-entity__role-details-container')

    # Handle case where user has multiple  roles at the same company
    if position_elements:
        organization = mt.get_text(soup, '.pv-entity__company-summary-info > h3 > span:nth-of-type(2)')

        positions = list(map(lambda position: mt.get_info(position, {
            'title': '.pv-entity__summary-info-v2 > h3 > span:nth-of-type(2)',
            'employment_type': '.pv-entity__summary-info-v2 > h4[class="t-14 t-black t-normal"]',
            'date_range': '.pv-entity__date-range span:nth-of-type(2)',
            'location': '.pv-entity__location > span:nth-of-type(2)',
            'description': '.pv-entity__description'

        }), position_elements))
        for pos in positions:
            pos['organization'] = organization
            pos['url'] = mt.get_attribute_value(soup,
                                                'a[data-control-name="background_details_company"]', "href")
            if pos['url'] is not None:
                pos['url'] = "https://www.linkedin.com" + pos['url']
        return positions

    else:
        job_info = mt.get_info(soup, {
            'title': '.pv-entity__summary-info h3:nth-of-type(1)',
            'employment_type': '.pv-entity__secondary-title > span',
            'date_range': '.pv-entity__date-range span:nth-of-type(2)',
            'location': '.pv-entity__location span:nth-of-type(2)',
            'description': '.pv-entity__description',
            'organization': '.pv-entity__secondary-title'

        })

        # DONE
        # FIX for having employment_type nested
        # e.g -> "organization": "Capgemini\n        Full-time",
        # info[key] = info[key].split("\n")[-1].strip()
        job_info['organization'] = job_info["organization"].split("\n")[0].strip()
        job_info['url'] = mt.get_attribute_value(soup, 'a[data-control-name="background_details_company"]', "href")
        if job_info['url'] is not None:
            job_info['url'] = "https://www.linkedin.com" + job_info['url']
        return [job_info]


def get_education_info(soup: BeautifulSoup):
    """
    Returns:
        dict of school name, degree, grades, field_of_study, date_range, &
        extra-curricular activities
    """
    info = mt.get_info(soup, {
        'school': '.pv-entity__school-name',
        'degree': '.pv-entity__degree-name span:nth-of-type(2)',
        'field_of_study': '.pv-entity__fos span:nth-of-type(2)',
        'date_range': '.pv-entity__dates span:nth-of-type(2)',
        'grades': '.pv-entity__grade span:nth-of-type(2)',
        'activities': '.activities-societies',
        'description': '.pv-entity__description',
    })

    info['url'] = mt.get_attribute_value(soup,
                                         'a[data-control-name="background_details_school"]', "href")
    if info['url'] is not None:
        info['url'] = "https://www.linkedin.com" + info['url']
    return [info]


def get_certifications_info(soup: BeautifulSoup):
    """
    Returns
    -------
    licenses_and_certifications : dict
        Returns a list of dictionaries with the following {keys: values}

        name                 : str, default None [ Must have]

        issuing_organization : str, default None [ Must have]

        issue_date           : str, default None
            issue date in the format "Issued 

        expiration_date      : str, default None
            returns:
                'No Expiration Date', if the licenses doesn't expire
                Expiring date with strftime for month and year 'Expires %b %Y' e.g 'Expires Dec 2021'
                Expired date with strftime for month and year 'Expired %b %Y' e.g 'Expired Mar 2016'


        credential_id        : str, default None

        credential_url       : str, default None

        li_org_url           : str, default None

    """

    info = mt.get_info(soup, {
        'name': '.pv-certifications__summary-info > h3',
        'issuing_organization': '.pv-certifications__summary-info > p > span:nth-of-type(2)',
        'issue_date': '.pv-certifications__summary-info > p:nth-of-type(2)> span:nth-of-type(2)',
        'expiration_date': '.pv-certifications__summary-info > p:nth-of-type(2)> span>span',

        'credential_id': '.pv-certifications__summary-info > p:nth-of-type(3)> span:nth-of-type(2)'
    })

    info['credential_url'] = mt.get_attribute_value(soup, '.pv-certifications-entity__credential-link', "href")

    if info['issue_date'] and info['expiration_date']:
        info['issue_date'] = info['issue_date'].replace(info['expiration_date'], "").strip()  # Quick fix
        info['expiration_date'] = re.sub("\n* +", " ", info['expiration_date'])

    info['url'] = mt.get_attribute_value(soup, 'a[data-control-name="background_details_certification"]', "href")
    if info['url'] is not None:
        info['url'] = "https://www.linkedin.com" + info['url']
    return [info]


def get_volunteering_info(soup):
    """
    Returns
    -------
    info : dict
        Returns a list of dictionaries with the following {keys: values}
        organization    : str, default None [ Must have]
        role            : str, default None
        cause           : str, default None
        date_range      : str, default None
        description     : str, default None
        url          : str, default None

    """
    info = mt.get_info(soup, {
        'organization': '.pv-entity__secondary-title',
        'role': '.pv-entity__summary-info h3:nth-of-type(1)',
        'cause': '.pv-entity__cause span:nth-of-type(2)',
        'date_range': '.pv-entity__date-range span:nth-of-type(2)',
        'description': '.pv-entity__description'
    })

    info['url'] = mt.get_attribute_value(soup, 'a[data-control-name="background_details_company"]', "href")
    if info['url'] is not None:
        info['url'] = "https://www.linkedin.com" + info['url']
    return [info]


def get_skills_info(skill):
    """
    Returns
    -------
    info : dict
        name           : str, default None
            name of the skill
        endorsements     : int, default None
            the number of endorsements received between 0 and 99
    """
    info = mt.get_info(skill, {
        'name': '.pv-skill-category-entity__name-text',
        'endorsements': '.pv-skill-category-entity__endorsement-count'
    })

    if skill.find("div", {"class": "pv-skill-entity__verified-icon"}) is not None:
        info["verified"] = True
    else:
        info["verified"] = False

    info["endorsements"] = int(info['endorsements'].replace('+', '')) if info['endorsements'] else 0
    info["endorsers"] = None
    return [info]


def get_people_popup_info(soup: BeautifulSoup):
    """
    Returns
    -------
    info : dict
        a list of dictionaries with the following {keys: values}
        full_name   : str, default None
        headline    : str, default None
        li_per_url  : str, default None


    """
    info = mt.get_info(soup, {
        'full_name': '.pv-endorsement-entity__name--has-hover',
        'headline': '.pv-endorsement-entity__headline'
    })

    info['url'] = mt.get_attribute_value(soup, '.pv-endorsement-entity__link', "href")
    if info['url'] is not None:
        info['url'] = "https://www.linkedin.com" + info['url']
    return [info]


def get_recommendations_info(soup: BeautifulSoup):
    """
    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
    recommendation : dict
        full_name           : str, default None
        headline            : str, default None
        recommendation      : str, default None
        relationship_info   : str, default None
        date                : str, default None
            Date format "%B %d, %Y", e.g "October 18, 2018"
        relationship        : str, default None
        li_per_url          : str, default None

    """
    info = mt.get_info(soup, {
        'full_name': 'h3',
        'headline': '.pv-recommendation-entity__headline',
        'recommendation': '.pv-recommendation-entity__text',
        'relationship_info': 'div > p:nth-of-type(2)',

    })


    info['date'] = ",".join(info["relationship_info"].split(",")[:2])
    info['relationship'] = info['relationship_info'].replace(info['date'] + ", ", "")
    info['url'] = mt.get_attribute_value(soup, 'a[data-control-name="recommendation_details_profile"]', "href")
    if info['url'] is not None:
        info['url'] = "https://www.linkedin.com" + info['url']

    info['recommendation'] = mt.strip_string_from_end(info['recommendation'], 'See less')
    info['recommendation'] = mt.strip_string_from_end(info['recommendation'], '... See more')

    return [info]


def get_publications_info(soup):
    """
    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
    publications : a dict with the following keys

        title           : str, default None
        date            : str, default None
            Date format "%b %d, %Y", e.g "Jan 1, 2017"
        publisher       : str, default None
        description     : str, default None
        pub_url         : str, default None
            publication url

        # TODO LATER an analysis of the publications
        collaborators   : a list of dictionaries with the following keys, default empty list
            full_name   : str, default None
            li_per_url  : str, default None
            headline    : str, default None



    """
    info = mt.get_info(soup, {
        'title': '.pv-accomplishment-entity__title',
        'date': '.pv-accomplishment-entity__date',
        'publisher': '.pv-accomplishment-entity__publisher',
        'description': '.pv-accomplishment-entity__description',
        'collaborators_selector': '.pv-accomplishment-entity__group'

    })

    for key in info:
        if key == "collaborators_selector":
            continue
        if info[key]:
            info[key] = info[key].split("\n")[1].strip()

    info['url'] = mt.get_attribute_value(soup, 'a.link-without-visited-state', "href")

    return [info]


def get_patents_info(soup):
    """
    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
    patents : a dict with the following keys

        title             : str, default None
        issuer_and_nr     : str, default None
            e.g "us 8024695"
            # TODO LATER Some people instead of the number they have pending,https://www.linkedin.com/in/richardbreiter/

        status_and_date   : A string representing the date
            e.g "Issued Sep 20, 2011"
            status can be "Issued" or ""
            Date format "%b %d, %Y", e.g "Jan 1, 2017"


        description : str, default None
        url         : str, default None

        inventors      : a list of dictionaries with the following keys
            fullname   : str, default None
            headline   : str, default None
            li_per_url : str, default None

    """
    info = mt.get_info(soup, {
        'title': '.pv-accomplishment-entity__title',
        'issuer_and_nr': '.pv-accomplishment-entity__issuer',
        'status_and_date': '.pv-accomplishment-entity__date',
        'description': '.pv-accomplishment-entity__description',
        'collaborators_selector': '.pv-accomplishment-entity__group'
    })

    for key in info:
        if info[key]:
            info[key] = info[key].split("\n")[-1].strip()

    info['url'] = mt.get_attribute_value(soup, 'a.link-without-visited-state', "href")
    return [info]


def get_courses_info(soup):
    """
    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
    courses : a dict with the following keys
        name            : str, default None
        number          : str, default None

    """
    info = mt.get_info(soup, {
        'name': '.pv-accomplishment-entity__title',
        'number': '.pv-accomplishment-entity__course-number'

    })
    for key in info:
        if info[key]:
            info[key] = info[key].split("\n")[-1].strip()
    return [info]


def get_projects_info(soup):
    """
    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
    projects : a dict with the following keys

        name            : str, default None
        date            : str, default None
            Date format "%b %Y", e.g "Oct 2008"
        description     : str, default None

        url         : str, default None
            publication url

        collaborators   : a list of dictionaries with the following keys, default empty list
            full_name   : str, default None
            headline    : str, default None
            li_per_url  : str, default None


    """
    info = mt.get_info(soup, {
        'name': '.pv-accomplishment-entity__title',
        'date': '.pv-accomplishment-entity__date',
        'description': '.pv-accomplishment-entity__description',
        'collaborators_selector': '.pv-accomplishment-entity__group'

    })

    for key in info:
        if info[key]:
            info[key] = info[key].split("\n")[-1].strip()

    info['url'] = mt.get_attribute_value(soup, 'a.link-without-visited-state', "href")

    # Get collaborators element

    # If found

    # Open popup
    # Scroll to the end
    # Get the list

    return [info]


def get_honors_info(soup):
    """
    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
    honors : a dict with the following keys
        title           : str, default None
        issuer          : str, default None
        date            : str, default None
            Date format "%b %Y", e.g "Jan 2007" or "%Y", e.g "2006"
            #TODO LATER after few scraping review the documendation that the format is correct of dates
        description     : str, default None
    """
    info = mt.get_info(soup, {
        'title': '.pv-accomplishment-entity__title',
        'issuer': '.pv-accomplishment-entity__issuer',
        'date': '.pv-accomplishment-entity__date',
        'description': '.pv-accomplishment-entity__description'

    })

    for key in info:
        if info[key]:
            info[key] = info[key].split("\n")[-1].strip()
    return [info]


def get_languages_info(soup):
    """
    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
    languages : a dict with the following keys
        language          : str, default None
        proficiency       : str, default None
            returns:
                Native or bilingual proficiency
                Full professional proficiency
                Professional working proficiency
                Limited working proficiency
                Elementary proficiency

    """
    info = mt.get_info(soup, {
        'language': '.pv-accomplishment-entity__title',  # MUST HAVE CAN NOT BE NONE
        'proficiency': '.pv-accomplishment-entity__proficiency'

    })
    for key in info:
        if info[key]:
            info[key] = info[key].split("\n")[-1].strip()
    return [info]


def get_organizations_info(soup):
    """
    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
    organizations : a dict with the following keys
        name           : str, default None
        position       : str, default None
        date_range           : str, default None
            Date format "%b %Y – %b %Y", e.g "Jan 2007 – Jan 2008" or "%b %Y – Present", e.g "Jan 2006 – Present"
        description    : str, default None
    """
    info = mt.get_info(soup, {
        'name': '.pv-accomplishment-entity__title',
        'position': '.pv-accomplishment-entity__position',
        'date': '.pv-accomplishment-entity__date',
        'description': '.pv-accomplishment-entity__description'  # TODO correct typo in others

    })

    for key in info:
        if info[key]:
            info[key] = info[key].split("\n")[-1].strip()

    return [info]


def get_test_scores_info(soup):
    """
    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
    test_scores : a dict with the following keys
        name           : str, default None
        date           : str, default None
            Date format "%b %Y", e.g "Oct 2008"
        score          : str, default None
        description    : str, default None
    """

    info = mt.get_info(soup, {
        'name': '.pv-accomplishment-entity__title',
        'date': '.pv-accomplishment-entity__date',
        'score': '.pv-accomplishment-entity__score',
        'description': '.pv-accomplishment-entity__description'

    })

    for key in info:
        if info[key]:
            info[key] = info[key].split("\n")[-1].strip()

    return [info]


def get_project_collaborators_pop_info(soup):
    """
    Returns:
        a list of dictionaries with the following {keys: values}
            full_name   : str, default None
            headline    : str, default None

    """
    info = mt.get_info(soup, {
        'full_name': '.profile-item__name--has-hover',
        'headline': 'p:nth-of-type(2)'
    })

    info['url'] = mt.get_attribute_value(soup, '.entity-link-wrapper', "href")
    if info['url'] is not None:
        info['url'] = "https://www.linkedin.com" + info['url']

    return info


def get_interest_info(soup):
    """
    Used to get info from tabs like companies

    Parameters
    ----------
        soup : a beautiful soup object

    Returns
    -------
        a dictionaries with the following {keys: values}
            name        : str, default None
            headline    : str, default None
            followers   : int, default None
            url         : str, default None


    """
    info = mt.get_info(soup, {
        'name': '.pv-entity__summary-title-text',
        'headline': '.pv-entity__occupation',
        'followers': '.pv-entity__follower-count'
    })

    if info['followers']:
        info['followers'] = int(
            info['followers'].replace("followers", "").replace("members", "").replace(",", "").strip())

    info['url'] = mt.get_attribute_value(soup, '.pv-interest-entity-link', "href")
    if info['url'] is not None:
        info['url'] = "https://www.linkedin.com" + info['url']

    return [info]


def get_people_browser_info(soup):
    """
    Returns:
        a list of dictionaries with the following {keys: values}
            full_name     : str, default None
            headline      : str, default None
            dist_value    : int, default None
            li_per_url    : str, default None


    """
    info = mt.get_info(soup, {
        'full_name': 'span.name',
        'headline': 'p.pv-browsemap-section__member-headline',
        'dist_value': 'span.dist-value'

    })

    info['url'] = mt.get_attribute_value(soup, 'a.pv-browsemap-section__member', "href")
    if info['url'] is not None:
        info['url'] = "https://www.linkedin.com" + info['url']
    return [info]
