"""
Title     : People
Objective : Scrape data from linkedin's person profile profile
Created by: Janid
Created on: 5/25/20
"""

# Standard library imports
import logging
import random
import re
import urllib.request
from pathlib import Path

# Related third party imports
from selenium.webdriver.common.by import By

# Import local applications
from scraper import scraping_tools as mt
from scraper.linkedin.people import items
from scraper.session import ScraperSession

# Create a module logger
logger_people_scraper = logging.getLogger('people_scraper')
# Set level
logger_people_scraper.setLevel(logging.DEBUG)


class People(ScraperSession):
    """Contains the tools to scrape data on a given person on linkedin"""

    # NOTE: better to dismiss manually the first time. after that is won't show
    # TODO: To avoid clicking every time find something to avoid. eg lower timeout and on purpose close
    # Dismiss the cookies if it is present
    # el = self.select_element(By.CSS_SELECTOR, "button.artdeco-global-alert__dismiss")
    # if el is not None:
    #     el.click()

    def get_pv_top_card_info(self):
        """Gets info in the top card of the profile
        Returns
        -------
        self.top_card_info : dict
            Returns a dictionary with the following {keys: values}.
            full_name       : str
                The full name of the person.
            dist_value      : str, default None
                The distance to the person.
            headline        : str, default None
                headline title.
            location        : str, default None
                The location of the person.
            connections     : str, default None
                The connections the person has on linkedin.
            dist_value_int  : int, default None
                The distance to the person.
            connections_int : int, default None
                The connections the person has on linkedin.

            membership_info : dict
                a dictionary with keys ["premium", "influencer", "jobSeeker"] that take:
                premium : bool
                influencer : bool
                jobSeeker : bool

            image : bool
                True if the profile has a picture and False if not.
                Profile picture is saved with the id name in the image path.

            followersCount : int, default None
                The number of followers, it could be possible to infer the number of connections from it.

        """
        logger_people_scraper.debug("session, started, top_card_info")
        self.top_card_info = {}
        # Select top-card: pv-top-card
        top_card = self.scroll_select(By.CSS_SELECTOR, '.pv-top-card',
                                      return_type="bs_el",
                                      html_name="pv_top_card",
                                      html_path=self.html_path_profile,
                                      screenshot_name="pv_top_card",
                                      screenshot_path=self.screenshots_path_profile,
                                      )

        # get top card info
        self.top_card_info = items.get_top_card(top_card)

        # Get the int of the dist-value
        # noinspection PyBroadException
        try:
            self.top_card_info["dist_value_int"] = int(re.findall(r'\d+', self.top_card_info["dist_value"])[0])
        except:
            self.top_card_info["dist_value_int"] = None

        # Get the int of the connections
        # noinspection PyBroadException
        try:
            self.top_card_info["connections_int"] = int(re.findall(r'\d+', self.top_card_info["connections"])[0])
        except:
            self.top_card_info["connections_int"] = None

        # Get membership info
        # ----------------------------------------------------------------------
        self.get_membership_info()

        # Get extra top card
        # ----------------------------------------------------------------------
        extra_el = self.select_element(By.CSS_SELECTOR, 'section.pv-open-to-carousel-card', return_type="sel_el")
        extra_top_card = None
        # if extra_el:
        #     # If hiring:
        #     self.click_on(By.CSS_SELECTOR,
        #                     'section.pv-open-to-carousel-card a',
        #                     html_name=self.file_name + "_opportunities_card",
        #                     html_path=self.html_path_profile,
        #                     screenshot_name=self.file_name + "_opportunities_card",
        #                     screenshot_path=self.screenshots_path_profile,
        #                     )
        #     # extra info from top card as jobSeeker or offering services or hiring
        #     extra_top_card = {}
        #     el = self.select_element(By.CSS_SELECTOR, 'h2[id="po-route-modal-header-details"', return_type="bs_el")
        #     if el:
        #         extra_top_card.update({"details": el.get_text().strip()})
        #         if extra_top_card["details"] == "Job preferences":
        #             self.top_card_info["membership_info"]["jobSeeker"] = True
        #
        #
        #         els = self.select_elements(By.CSS_SELECTOR, '.artdeco-modal__content li', return_type="bs_el")
        #
        #         for e in els:
        #             extra_top_card.update(items.get_extra_top_card(e))
        #     else:
        #         el = self.select_element(By.CSS_SELECTOR, 'h2[id="po-route-modal-header-view"', return_type="bs_el")
        #         extra_top_card.update({"details": el.get_text().strip()})
        #         section_bs = self.select_elements(By.CSS_SELECTOR, ".po-view__job-card", return_type="bs_el")
        #         if section_bs:
        #             extra_top_card.update({"details": el.get_text().strip()})
        #             jobs = list(map(items.get_extra_top_card_jobs, section_bs))
        #             jobs = mt.flatten_list(jobs)
        #             extra_top_card.update({"jobs_list": jobs})
        #
        #     self.get_meta_data("extra_top_card")
        #     self.click_on(By.CSS_SELECTOR,
        #                      'button.artdeco-modal__dismiss',
        #                      html_name=self.file_name + "_open_to_work_dismiss",
        #                      html_path=self.html_path_profile,
        #                      screenshot_name=self.file_name + "_open_to_work_dismiss",
        #                      screenshot_path=self.screenshots_path_profile,
        #                      )

        self.top_card_info.update({"extra_top_card": extra_top_card})

        # Get image
        # ----------------------------------------------------------------------
        self.get_image()

        # Get the number of followers
        # ----------------------------------------------------------------------
        self.get_followers()
        # ----------------------------------------------------------------------
        self.highlights = items.get_highlights(top_card)
        self.data.update({"highlights": self.highlights})
        # ----------------------------------------------------------------------
        self.data.update({"top_card_info": self.top_card_info})

        logger_people_scraper.debug("session, finished, top_card_info")
        return self.top_card_info

    def get_membership_info(self):
        """Gets info in the top card of the profile
        Returns
        -------
        membership_info : dict
            premium       : bool
                True, if the person has a premium profile otherwise False
            influencer    : bool
                True, if the person is an influencer otherwise False
            jobSeeker     : bool, default None
                True, if the person is a Job seeker
        """

        membership_info = {'premium': None, 'influencer': None, 'jobSeeker': None}
        el = self.select_element(By.CSS_SELECTOR, '.pv-member-badge li-icon', return_type="bs_el")
        if el:
            icon_type = mt.get_attribute_value(el, "li-icon", "type")
            if icon_type == "linkedin-bug":
                membership_info.update({"premium": "True"})
            if icon_type == "linkedin-influencer-color-icon":
                membership_info.update({"influencer": "True"})

        self.top_card_info.update({"membership_info": membership_info})

    def get_followers(self):
        """Gets info in the top card of the profile
        Returns
        -------
        followersCount : int
        """
        followersCount = {'followersCount': None}

        el = self.select_element(By.CSS_SELECTOR, '.pv-recent-activity-section-v2 span.align-self-center',
                                 return_type="bs_el")
        if el:
            followersCount["followersCount"] = int(el.text.replace(",", "").replace("followers", "").strip())
        elif self.url == "https://www.linkedin.com/in/janid/":
            el = self.select_element(By.CSS_SELECTOR, 'a[data-control-name="profile_view_link_to_followers"]',
                                     return_type="bs_el")
            if el:
                followersCount["followersCount"] = int(el.text.replace(",", "").replace("followers", "").strip())

        self.top_card_info.update(followersCount)



    def get_image(self):
        """get image"""
        # Scroll to the top, element has to be visible to be clicked
        self.driver.execute_script("window.scrollTo(0, 0);")

        my_image = False
        # noinspection PyBroadException
        try:
            my_image = self.driver.find_element(By.CSS_SELECTOR, '.profile-photo-edit__edit-btn')
            logger_people_scraper.debug("Scraping my profile")
            self.top_card_info.update({"image": True})
        except:
            pass

        if not my_image:
            # Click on the picture
            self.click_on(By.CSS_SELECTOR,
                          'button.pv-top-card-profile-picture',
                          html_name=self.file_name + "_photo_click",
                          html_path=self.html_path_profile,
                          screenshot_name=self.file_name + "_photo_click",
                          screenshot_path=self.screenshots_path_profile,
                          )

            # ---------------------

            # For photo without watermark
            css_dismiss = ".pv-member-photo-modal"
            image_element = self.select_element(By.CSS_SELECTOR,
                                                '.pv-member-photo-modal__profile-image',
                                                return_type="bs_el",
                                                html_name=self.file_name + "_photo",
                                                html_path=self.html_path_profile,
                                                screenshot_name=self.file_name + "_photo",
                                                screenshot_path=self.screenshots_path_profile,
                                                )
            # For photo with watermark
            if not image_element:
                image_element = self.select_element(By.CSS_SELECTOR,
                                                    '.pv-member-photo-modal__content',
                                                    return_type="bs_el",
                                                    html_name=self.file_name + "_photo",
                                                    html_path=self.html_path_profile,
                                                    screenshot_name=self.file_name + "_photo",
                                                    screenshot_path=self.screenshots_path_profile,
                                                    )
                css_dismiss = '.pv-nonself-member-photo-modal'

            if image_element:
                # select the attribute src in the img tag
                urllib.request.urlretrieve(image_element.img["src"],
                                           Path(self.media_path, self.file_name_ts + ".png"))

                # Close the picture
                # ---------------------
                # TODO add mechanism against failed closing picture
                self.click_on(By.CSS_SELECTOR,
                              css_dismiss + ' button[aria-label="Dismiss"]',
                              html_name=self.file_name + "_photo_dismiss",
                              html_path=self.html_path_profile,
                              screenshot_name=self.file_name + "_photo_dismiss",
                              screenshot_path=self.screenshots_path_profile,
                              )

                self.top_card_info.update({"image": True})
            else:
                # log no image
                self.top_card_info.update({"image": False})

    def get_contact_info(self):
        """
        Gets info in the top card of the profile

        Returns
        -------
        contact_info : dict
            Returns a dictionary with the following {keys: values}
            address   : str, default None
                Address of the person.
            email     : str, default None
                Email pf the person.
            birthday  : str, default None
                Birthday of the person in the format "%B %d".
            phone     : str, default None
                Phone number.
            connected': str, default None
                Date in which the scraper has connected with the person in the format "%B %d, %Y".
            twitter'  : str, default None
                Twitter account of the person.

            websites' : list, default []
                List of strings of websites

            im        : list, default []
                List of instant messenger

            contact_info_headers : list, default [<name>'s profile]
                List of contact info headers (can be used to see what type of headers exist)

            first_name: str

            last_name : str

        """
        logger_people_scraper.debug("session, started, contact_info")
        self.contact_info = {}

        self.click_on(
            method=By.LINK_TEXT,
            selector="Contact info",
            html_name="contact_info_click",
            html_path=self.html_path_profile,
            screenshot_name="contact_info_click",
            screenshot_path=self.screenshots_path_profile
        )
        contact_info_card = self.select_element(
            method=By.CSS_SELECTOR,
            selector='.pv-contact-info',
            return_type="bs_el",
            html_name="contact_info",
            html_path=self.html_path_profile,
            screenshot_name="contact_info",
            screenshot_path=self.screenshots_path_profile
        )
        self.get_meta_data("contact_info")
        # dismiss
        self.click_on(
            method=By.CSS_SELECTOR,
            selector=".artdeco-modal__dismiss",
            html_name="contact_info_click_dismiss",
            html_path=self.html_path_profile,
            screenshot_name="contact_info_click_dismiss",
            screenshot_path=self.screenshots_path_profile
        )
        self.contact_info = items.get_contact_info(contact_info_card)
        # websites
        # test websites empty
        # test websites = 1
        # test websites > 1
        websites = []
        if contact_info_card:
            websites = contact_info_card.select('.ci-websites li a')
            websites = list(map(lambda x: x['href'], websites))
        self.contact_info.update({"websites": websites})

        # Instant messenger
        # test Instant empty
        # test Instant = 1
        # test Instant > 1
        instant_messengers = []
        if contact_info_card:
            ims = contact_info_card.select('.ci-ims li')
            if ims:
                instant_messengers = [re.sub(r"\s+", "", i.text.strip()) for i in ims]
        self.contact_info.update({"im": instant_messengers})

        # contact info headers auto
        contact_info_card_auto = contact_info_card.select('section h3')
        contact_info_headers = [i.text.strip().lower() for i in contact_info_card_auto]
        self.contact_info.update({"contact_info_headers": contact_info_headers})

        # First name
        if "’ profile" in contact_info_headers[0]:
            first_name = re.sub("’ profile", "", contact_info_headers[0])
        else:
            first_name = re.sub("’s profile", "", contact_info_headers[0])
        self.contact_info.update({"first_name": first_name})

        # Last name:
        # noinspection PyBroadException
        last_name = None
        # noinspection PyBroadException
        try:
            last_name = self.data["top_card_info"]["full_name"].lower().replace(first_name.lower(), "").strip()
        except:
            pass
        self.contact_info.update({"last_name": last_name})
        # test that the data is updated
        self.data.update({"contact_info": self.contact_info})
        logger_people_scraper.debug("session, done, contact_info")
        # test that the return
        return self.contact_info

    # test has no summary : https://www.linkedin.com/in/adamsefler/
    def get_summary(self):
        """
        Updates person_data with the summary and returns the summary

        Returns
        -------
        summary : str
            summary in the about section
        """
        self.summary = None
        logger_people_scraper.debug("session, started, summary")
        summary = self.scroll_select(
            By.CSS_SELECTOR,
            '.pv-profile-section.pv-about-section'
        )

        if summary and (summary.get_attribute("style") != 'display: none;'):
            self.expand_buttons(By.CSS_SELECTOR, '.pv-about-section button[aria-expanded="false"]',
                                By.CSS_SELECTOR,
                                '.pv-about-section')
            summary = self.select_element(
                By.CSS_SELECTOR,
                '.pv-profile-section.pv-about-section',
                return_type="bs_el",
                html_name="summary",
                html_path=self.html_path_profile,
                screenshot_name="summary",
                screenshot_path=self.screenshots_path_profile,
            )

            self.summary = items.get_summary(summary)["summary"]
            # A security check to remove the 'About' and white spaces
            if self.summary[:6].strip() == 'About':
                self.summary = self.summary[6:].strip()

        else:
            logger_people_scraper.warning("session, summary, not found")

        self.data.update({"summary": self.summary})

        logger_people_scraper.debug("session, done, summary")
        return self.summary

    def get_highlights(self):
        """
        Updates person_data with the highlights

        Returns
        -------
        highlights : dict
            Returns a dictionary with the following {keys: values}

            mutual_connections   : int, default None
                Number of mutual connections
            url_connections   : str, default None
                Link to mutual connections
        """

        logger_people_scraper.debug("session, started, highlights")
        # Init highlights dictionary
        self.highlights = {}
        if self.url != "https://www.linkedin.com/in/janid/":

            highlights = self.scroll_select(
                By.CSS_SELECTOR,
                ".pv-top-card a"
            )

            if highlights and (highlights.get_attribute("style") != 'display: none;'):
                # Scroll to Highlights section
                highlights = self.scroll_select(By.CSS_SELECTOR, ".pv-highlights-section",
                                                return_type="bs_el",
                                                html_name="highlights",
                                                html_path=self.html_path_profile,
                                                screenshot_name="highlights",
                                                screenshot_path=self.screenshots_path_profile,
                                                )
                if highlights:
                    self.highlights = items.get_highlights(highlights)
            else:
                logger_people_scraper.debug("section, not found, highlights")

        self.data.update({"highlights": self.highlights})

        logger_people_scraper.debug("session, done, highlights")
        return self.highlights

    def get_featured(self):
        """
        For now it just saves the html

        Returns TODO
        -------
        """
        logger_people_scraper.debug("session, started, featured")

        # test on jack
        # test om carolina she doe not have featured
        # el = self.scroll_select(By.CSS_SELECTOR, '.pv-profile-section.pab-featured-section')
        featured = self.scroll_select(
            By.CSS_SELECTOR,
            '.pv-profile-section.pab-featured-section'
        )

        if featured and (featured.get_attribute("style") != 'display: none;'):
            self.scroll_select(By.CSS_SELECTOR, '.pv-profile-section.pab-featured-section',
                               return_type="bs_el",
                               html_name="featured",
                               html_path=self.html_path_profile,
                               screenshot_name="featured",
                               screenshot_path=self.screenshots_path_profile,
                               )
            # if el and (el.get_attribute("style") != 'display: none;'):
            #     el = self.wait_one_element(By.CSS_SELECTOR, '.pv-profile-section.pab-featured-section',
            #                                html_folder=self.file_name)

        logger_people_scraper.debug("session, done, featured")

    def get_activity(self):
        """
        For now it just saves the html

        Returns TODO
        -------
        """
        logger_people_scraper.debug("session, started, activity")
        activity = self.scroll_select(
            By.CSS_SELECTOR,
            '.pv-profile-section.pv-recent-activity-section-v2'
        )

        if activity and (activity.get_attribute("style") != 'display: none;'):
            self.scroll_select(By.CSS_SELECTOR, '.pv-profile-section.pv-recent-activity-section-v2',
                               return_type="bs_el",
                               html_name="activity",
                               html_path=self.html_path_profile,
                               screenshot_name="activity",
                               screenshot_path=self.screenshots_path_profile,
                               )

        # el = self.scroll_to_element(By.CSS_SELECTOR, '.pv-profile-section.pv-recent-activity-section-v2')
        #
        # if el and (el.get_attribute("style") != 'display: none;'):
        #     el = self.wait_one_element(By.CSS_SELECTOR, '.pv-profile-section.pv-recent-activity-section-v2',
        #                                  html_folder=self.file_name)

        logger_people_scraper.debug("session, done, activity")

    def get_data_in_section(self,
                            section,
                            method, selector,
                            list_method, list_selector,
                            get_info_function,
                            inline_method=None, inline_selector=None,
                            show_more_method=None, show_more_selector=None,
                            accomp_el_method=None,
                            accomp_el_selector=None,
                            might_have_collaborators=None
                            ):
        """
        get info from a section
        method and its selector, are meant to extract the main container, in this container there is a list.
        list_method and list_selector, are meant for the list of element from which extract the information.
        inline_method and inline_selector, are meant to expand the button to show more in the list.
        show_more_method and show_more_selector are meant to expand the see more in each element in the list.
        accomp_el_method and its selector, are meant to click on the button to show the section under accomplishments.

        in t
        Parameters
        ----------
        section: str
            section name
        method : str
            By selector type selenium.webdriver.common.by.By
        selector : str
            selector to look for
        list_method : str
            The list to extract to which the function to get info will me mapped
        list_selector : str
            selector to look for
        get_info_function : Any
            function to map to the list
        inline_method : str
            By selector type selenium.webdriver.common.by.By
        inline_selector : str
            selector to look for
        show_more_method : str
            By selector type selenium.webdriver.common.by.By
        show_more_selector : str
            selector to look for
        accomp_el_method : str
            is meant to click on the button to show the section under accomplishments
        accomp_el_selector : str
            is meant to click on the button to show the section under accomplishments
        might_have_collaborators: bool
            a section in accomplishment might have collaborators


        Returns
        -------
        info : list of dict
        """
        logger_people_scraper.debug("session, started, " + section)
        # Init Experience list
        self.info = []

        # Scroll to section
        element = self.scroll_to_element(method, selector)

        # if the element is found
        if element and (element.get_attribute("style") != 'display: none;'):
            if accomp_el_selector:
                # Expand section
                logger_people_scraper.debug("clicking on the section: " + section)
                self.click_on(accomp_el_method, accomp_el_selector)

            # Expand the see mores in the inline
            if inline_method:
                logger_people_scraper.debug("expanding buttons on the section: " + section)
                self.expand_buttons(inline_method, inline_selector, method, selector)

            # Expand the see more inside a text
            if show_more_method:
                self.expand_buttons(show_more_method, show_more_selector, list_method, list_selector, element)

            # save the html
            self.select_element(method, selector,
                                return_type="bs_el",
                                html_name=section,
                                html_path=self.html_path_profile,
                                screenshot_name=section,
                                screenshot_path=self.screenshots_path_profile,
                                )

            section_bs = self.select_elements(list_method, list_selector, return_type="bs_el")

            if section_bs:
                self.info = list(map(get_info_function, section_bs))
                self.info = mt.flatten_list(self.info)
            else:
                logger_people_scraper.debug("section, not found, " + section)
            if might_have_collaborators:

                sec_key = {
                    "publications": "publication_contributors",
                    "projects": "project_contributors",
                    "patents": "patent_contributors"

                }

                section_sel = self.select_elements(list_method, list_selector)
                for p, el in zip(self.info, section_sel):
                    if p["collaborators_selector"]:
                        p["collaborators"] = self.get_popup(function=items.get_project_collaborators_pop_info,
                                                            sel_element=el,
                                                            open_popup_method=By.CSS_SELECTOR,
                                                            open_popup_selector='a[data-control-name=' + sec_key[
                                                                section] + ']',
                                                            list_method=By.CSS_SELECTOR,
                                                            list_selector='.pv-profile-detail__modal '
                                                                          '.entity-list-item',
                                                            close_popup_method=By.CSS_SELECTOR,
                                                            close_popup_selector='.pv-profile-detail__modal '
                                                                                 'button[aria-label="Dismiss"] '
                                                            )
                    else:
                        p["collaborators"] = []

        else:
            logger_people_scraper.debug("section, not found, " + section)

        logger_people_scraper.debug("session, done, " + section)
        return self.info

    def get_experience(self):
        """
        Updates person_data with the experience section

        Returns
        -------
        experience : list of dict
            Returns a list of dictionaries with the following {keys: values}

            title           : str, default None  [Must have]
                role in the job position.

            employment_type : str, default None.
                Full-time, Part-time, Self-employed, Freelance, Contract, Internship, Apprenticeship

            date_range      : str, default None
                date rage on the job position.

            location        : str, default None
                location of of the job position.

            description     : str, default None
                job description.

            organization    : str, default None
                organization name.

            li_org_url      : str, default None
                linkedin url of the organization.
        """
        # logger_people_scraper.debug("session, started, experience")
        # Init Experience list
        self.experience = []
        self.experience = self.get_data_in_section(
            section="experience",
            method=By.CSS_SELECTOR,
            selector=".experience-section",
            list_method=By.CSS_SELECTOR,
            list_selector="#experience-section ul .pv-position-entity",
            inline_method=By.CSS_SELECTOR,
            inline_selector='.experience-section  button[aria-expanded="false"].pv-profile-section__see-more-inline',
            show_more_method=By.CSS_SELECTOR,
            show_more_selector='button[aria-expanded="false"].inline-show-more-text__button',
            get_info_function=items.get_experience_info)
        self.data.update({"experience": self.experience})

        # logger_people_scraper.debug("session, done, experience")
        return self.experience

    def get_education(self):
        """
        Updates person_data with the education

        Returns
        -------
        education : list of dict
            Returns a list of dictionaries with the following {keys: values}

            school          : str, default None [Must have]

            degree          : str, default None

            field_of_study  : str, default None

            date_range      : str, default None

            grades          : str, default None

            activities      : str, default None

            description     : str, default None

            li_org_url   : str, default None
        """
        # logger_people_scraper.debug("session, started, education")
        # Init education list
        self.education = []
        self.education = self.get_data_in_section(
            section="education",
            method=By.CSS_SELECTOR,
            selector=".education-section",
            list_method=By.CSS_SELECTOR,
            list_selector='#education-section ul .pv-education-entity',
            inline_method=By.CSS_SELECTOR,
            inline_selector='.education-section  button[aria-expanded="false"].pv-profile-section__see-more-inline',
            show_more_method=By.CSS_SELECTOR,
            show_more_selector='button[aria-expanded="false"].inline-show-more-text__button',
            get_info_function=items.get_education_info)
        self.data.update({"education": self.education})
        return self.education

    def get_licenses_and_certifications(self):
        """
        Updates person_data with the licenses and certifications

        Returns
        -------
        licenses_and_certifications : list of dict
            Returns a list of dictionaries with the following {keys: values}

            name                 : str, default None [Must have]

            issuing_organization : str, default None [Must have]

            issue_date           : str, default None
                issue date in the format "Issued

            expiration_date      : str, default None
                returns:
                    'No Expiration Date', if the licenses doesn't expire
                    Expiring date with strftime for month and year 'Expires %b %Y' e.g 'Expires Dec 2021'
                    Expired date with strftime for month and year 'Expired %b %Y' e.g 'Expired Mar 2016'


            credential_id        : str, default None

            credential_url       : str, default None

            li_org_url           : str, default None


        """
        # Init list
        self.lic_and_cert = []
        self.lic_and_cert = self.get_data_in_section(
            section="certifications",
            method=By.CSS_SELECTOR,
            selector=".pv-profile-section--certifications-section",
            list_method=By.CSS_SELECTOR,
            list_selector='.pv-certification-entity',
            inline_method=By.CSS_SELECTOR,
            inline_selector='.pv-profile-section--certifications-section button['
                            'aria-expanded="false"].pv-profile-section__see-more-inline ',
            show_more_method=By.CSS_SELECTOR,
            show_more_selector='button[aria-expanded="false"].inline-show-more-text__button',
            get_info_function=items.get_certifications_info
        )

        self.data.update({"lic_and_cert": self.lic_and_cert})
        return self.lic_and_cert

    def get_volunteering(self):
        """
        Updates person_data with the volunteering experiences

        Returns
        -------
        volunteering : list of dict
            Returns a list of dictionaries with the following {keys: values}

            organization    : str, default None [ Must have]

            role            : str, default None

            cause           : str, default None

            date_range      : str, default None

            description     : str, default None

            li_url          : str, default None

        """
        logger_people_scraper.debug("session, started, volunteering")
        # Init list
        self.volunteering = []
        self.volunteering = self.get_data_in_section(
            section="volunteering",
            method=By.CSS_SELECTOR,
            selector=".volunteering-section",
            list_method=By.CSS_SELECTOR,
            list_selector='.pv-profile-section.volunteering-section '
                          '.pv-volunteering-entity',
            inline_method=By.CSS_SELECTOR,
            inline_selector='.volunteering-section button[aria-expanded="false"].pv-profile-section__see-more-inline',
            show_more_method=By.CSS_SELECTOR,
            show_more_selector='.volunteering-section button[aria-expanded="false"].inline-show-more-text__button',
            get_info_function=items.get_volunteering_info)
        # Scroll to education section
        # element = self.scroll_to_element(By.CSS_SELECTOR, ".volunteering-section")
        #
        # # if the element is found
        # if element:
        #     # Expand the see more in the section
        #     self.expand_buttons(
        #         By.CSS_SELECTOR,
        #         '.volunteering-section button[aria-expanded="false"].pv-profile-section__see-more-inline'
        #     )
        #     # Expand the see mores in the inline section
        #     self.expand_buttons(By.CSS_SELECTOR,
        #                         '.volunteering-section button[aria-expanded="false"].inline-show-more-text__button')
        #
        #     # save its html
        #     self.select_element(By.CSS_SELECTOR, ".volunteering-section",
        #                         return_type="bs_el",
        #                         html_name="volunteering-section",
        #                         html_path=self.html_path_profile,
        #                         screenshot_name="volunteering-section",
        #                         screenshot_path=self.screenshots_path_profile,
        #                         )
        #     # get the list
        #     volunteering_section = self.select_elements(By.CSS_SELECTOR,
        #                                                 '.pv-profile-section.volunteering-section '
        #                                                 '.pv-volunteering-entity',
        #                                                 return_type="bs_el"
        #                                                 )
        #     # Extract info using get_school_info
        #     self.volunteering = list(map(items.get_volunteering_info, volunteering_section))

        self.data.update({"volunteering": self.volunteering})

        return self.volunteering

    def get_skills(self):
        """
            Returns
            -------
            skills : list of dict, default empty list
                Returns a list of dictionaries with the following {keys: values}

                skill           : str, default None
                    name of the skill

                endorsements     : int, default None
                    the number of endorsements received between 0 and 99

                verified     : srt, default None
                    Passed: LinkedIn Assessments or none

                if in the configuration get_endorsers_flag is true gets also the endorsers
                    endorsers       : list of dict, default empty list
                        a list of dictionaries with the following {keys: values}
                            full_name   : str, default None
                            url         : str, default None

        """

        # Init list
        self.skills = []
        self.skills = self.get_data_in_section(
            section="skills",
            method=By.CSS_SELECTOR,
            selector=".pv-skill-categories-section",
            list_method=By.CSS_SELECTOR,
            list_selector='.pv-skill-category-entity',
            get_info_function=items.get_skills_info,
            inline_method=By.CSS_SELECTOR,
            inline_selector='.pv-skill-categories-section button['
                            'aria-expanded="false"].pv-skills-section__additional-skills',
        )

        # Sort skills by endorsement
        self.skills = sorted(self.skills, key=lambda i: i['endorsements'], reverse=True)

        skills_sel_el = self.select_element(By.CSS_SELECTOR, ".pv-skill-categories-section")
        # TODO find someone with more than 99 skills
        # If the flag is true get the list of endorsers for each skill
        # Test using carolina's profile
        if self.config['get_endorsers_flag']:
            for skill in self.skills:

                # check if the skill has no endorsers, if so go to the next skill
                # test
                if skill['endorsements'] == 0:
                    logger_people_scraper.debug("skill has no endorsers , " + skill['name'])
                    skill.update({"endorsers": []})
                    continue

                logger_people_scraper.debug("Getting list of endorsers for skill , " + skill['name'])

                # Go the skill
                # -------------------------------
                self.scroll_to_element(By.LINK_TEXT, skill['name'], skills_sel_el)

                # Get endorsers info
                # -------------------------------
                endorsers_list = self.get_popup(items.get_people_popup_info,
                                                skills_sel_el,
                                                By.LINK_TEXT, skill['name'],
                                                By.CSS_SELECTOR, '.pv-profile-detail__modal .pv3',
                                                By.CSS_SELECTOR,
                                                '.pv-profile-detail__modal button[aria-label="Dismiss"]'
                                                )

                skill.update({"endorsers": endorsers_list})

            # check that the list is equal to the number of endorsements
            for skill in self.skills:
                if len(skill["endorsers"]) != int(skill["endorsements"]):
                    logger_people_scraper.error(
                        "Number of endorsement different from number endorsers " + skill['name'])

        self.data.update({"skills": self.skills})
        logger_people_scraper.debug("session, done, skills")
        return self.skills

    def get_popup(self,
                  function,
                  sel_element,
                  open_popup_method, open_popup_selector,
                  list_method, list_selector,
                  close_popup_method, close_popup_selector,
                  limit_list=None,
                  dismiss=True
                  ):
        """
        Get popup info

        Parameters
        ----------
        function        : Any
            function to be applied to map to the beautifulsoup list
        sel_element   : WebElement 
            selenium element used to open the popup, in the given section
        open_popup_method : str
            By selector type selenium.webdriver.common.by.By
        open_popup_selector : str
            selector to look for
        list_method  : str
            mechanism used to locate element
        list_selector : str
            value of the popup mechanism to locate
        close_popup_method : str
            By selector type selenium.webdriver.common.by.By
        close_popup_selector : str

        limit_list : int
            minimum limit to stop loading the list
        dismiss : bool
            if to automatically close the popup

        Returns
        -------
        popup_list : list of dict
            a list of returns from the applied function
        """

        logger_people_scraper.debug("session, started, get_popup")
        # Open the popup
        # -------------------------------
        self.click_on(method=open_popup_method, selector=open_popup_selector, web_element=sel_element)

        # scroll to the end to load page
        # -------------------------------
        while True:
            # noinspection PyBroadException
            try:
                # Get the list of endorsers
                popup_list = self.select_elements(method=list_method, selector=list_selector)
                # Store the last self
                old = popup_list[-1]
                # Put into view the last self
                # noinspection PyStatementEffect
                popup_list[-1].location_once_scrolled_into_view
                # Wait for the list to load

                mt.random_rest(self.config["driver"]["scroll_pause"]["min"],
                               self.config["driver"]["scroll_pause"]["max"])

                # Get the list of endorsers
                popup_list = self.select_elements(method=list_method, selector=list_selector)
                # Store the last self of the new list
                new = popup_list[-1]

                # If the old is the same, exit the loop
                if old == new:
                    break

                if limit_list and (len(popup_list) > limit_list):
                    break

            except:
                logger_people_scraper.error("failed expanding list ," + list_method + " , " + list_selector)
                break

        # get the list of bs elements
        popup_list = self.select_elements(method=list_method, selector=list_selector, return_type="bs_el")

        # Get data
        popup_list = list(map(function, popup_list))

        # Close the list
        if dismiss:
            self.click_on(method=close_popup_method, selector=close_popup_selector)

        logger_people_scraper.debug("session, done, get_popup")
        return popup_list

    # TEST profile examples:
    # Given 0, Received 0 # Carolina, Valerio, otto
    # Given 0, Received 1 # https://www.linkedin.com/in/marc-tromp/
    # Given 1, Received 0 # # TODO check that the recommendations are correct in the data downloaded
    # Given 1, Received 1 # Jack
    def get_recommendations(self):
        """
        Updates person_data with the recommendations

        Returns
        -------
        recommendations : dict
            a dict with two list,  default dictionary with empty two lists, received = [] and given = []
            received: list of dict
            list of dictionaries with the following keys
                full_name           : str, default None
                headline            : str, default None
                recommendation      : str, default None
                relationship_info   : str, default None
                date                : str, default None
                    Date format "%B %d, %Y", e.g "October 18, 2018"
                relationship        : str, default None
                li_per_url          : str, default None

            given: a list of dictionaries with the following keys
                full_name           : str, default None
                headline            : str, default None
                recommendation      : str, default None
                relationship_info   : str, default None
                date                : str, default None
                    Date format "%B %d, %Y", e.g "October 18, 2018"
                relationship        : str, default None
                li_per_url          : str, default None

        """
        logger_people_scraper.debug("session, started, recommendations")
        # Init recommendations dictionary
        self.recommendations = {
            "received": [],
            "given": []
        }

        # Scroll to recommendations section
        element = self.scroll_to_element(By.CSS_SELECTOR, ".pv-recommendations-section")

        # if the element is found, and the attribute style is different, signs that there are recommendations
        if element and (element.get_attribute("style") != 'display: none;'):
            # TODO add check if the recommendation is different than 0, open tab.
            # TEST on https://www.linkedin.com/in/innovativebusinessdevelopment/
            # Test on https://www.linkedin.com/in/madsbayer/
            # https://www.linkedin.com/in/jacknikogosian/

            selector_soup = self.select_element(By.CSS_SELECTOR,
                                                '.pv-recommendations-section button[aria-selected="true"]',
                                                return_type="bs_el")
            default_aria = selector_soup.get_text().strip()
            default_aria_key = default_aria.strip().split(" ")[0].lower()

            self.recommendations[default_aria_key] = self.get_recommendations_from_tab(default_aria_key)

            other_aria = self.select_element(By.CSS_SELECTOR,
                                             '.pv-recommendations-section button[aria-selected="false"]',
                                             return_type="bs_el")
            other_aria = other_aria.get_text().strip()
            other_aria_key = other_aria.split(" ")[0].lower()

            if other_aria.split(" ")[-1] != '(0)':
                self.click_on(By.CSS_SELECTOR, '.pv-recommendations-section button[aria-selected="false"]')
                self.recommendations[other_aria_key] = self.get_recommendations_from_tab(other_aria_key)

        self.data.update({"recommendations": self.recommendations})

        logger_people_scraper.debug("session, done, recommendations")
        return self.recommendations

    def get_recommendations_from_tab(self, section):
        """
        Get recommendations for a given aria
        Parameters
        ----------
        section: str
            section to get data from
        Returns
        -------
        info : list
             a list of the recommendations

        """

        info = self.get_data_in_section(
            section=section,
            method=By.CSS_SELECTOR,
            selector=".pv-recommendations-section .artdeco-tabpanel.active",
            list_method=By.CSS_SELECTOR,
            list_selector='.pv-recommendations-section .artdeco-tabpanel.active .pv-recommendation-entity',
            inline_method=By.CSS_SELECTOR,
            inline_selector='section.pv-recommendations-section .artdeco-tabpanel.active button['
                               'aria-expanded="false"][aria-controls="recommendation-list"]',

            show_more_method=By.CSS_SELECTOR,
            show_more_selector='.pv-recommendations-section .artdeco-tabpanel.active a['
                            'aria-expanded="false"].lt-line-clamp__more',

            get_info_function=items.get_recommendations_info)
        return info

    def get_accomplishments(self):
        """
        Updates person_data with the accomplishments

        Returns
        -------
        accomplishments : dict


        """
        logger_people_scraper.debug("session, started, accomplishments")
        # Init accomplishments dictionary
        self.accomplishments = {
            "publications": [],  #
            "patents": [],  #
            "courses": [],  #
            "projects": [],  #
            "honors": [],  #
            "languages": [],  #
            "organizations": [],  #
            "test-scores": []  #
        }

        keys = list(self.accomplishments.keys())

        # define functions for each section
        function = {
            "publications": items.get_publications_info,
            "patents": items.get_patents_info,
            "courses": items.get_courses_info,
            "projects": items.get_projects_info,
            "honors": items.get_honors_info,
            "languages": items.get_languages_info,
            "organizations": items.get_organizations_info,
            "test-scores": items.get_test_scores_info
        }

        sections_with_collaborations = ["publications", "projects", "patents"]

        # Scroll to accomplishments section
        element = self.scroll_to_element(By.CSS_SELECTOR, ".pv-accomplishments-section")

        if element:

            for key in keys:
                might_have_collaborators = None
                if key in sections_with_collaborations:
                    might_have_collaborators = True
                self.accomplishments[key] = self.get_data_in_section(
                    section=key,
                    method=By.CSS_SELECTOR,
                    selector="section." + key,
                    list_method=By.CSS_SELECTOR,
                    list_selector="section." + key + ' ul > li.pv-accomplishment-entity',
                    inline_method=By.CSS_SELECTOR,
                    inline_selector='section.' + key + ' button[aria-expanded="false"].pv-profile-section__see-more'
                                                       '-inline',
                    get_info_function=function[key],
                    accomp_el_method=By.CSS_SELECTOR,
                    accomp_el_selector='.pv-accomplishments-section button[aria-controls="' + key + '-expandable'
                                                                                                    '-content"]',
                    might_have_collaborators=might_have_collaborators
                )

        self.accomplishments["test_scores"] = self.accomplishments.pop("test-scores")

        self.data.update({"accomplishments": self.accomplishments})

        logger_people_scraper.debug("session, done, accomplishments")
        return self.accomplishments

    def get_interests(self):
        """
        Updates person_data with the interests



        # Types of interests:
        https://www.linkedin.com/help/linkedin/answer/34993/-interests-section-overview?lang=en&src=aff-lilpar&veh=
        aff_src.aff-lilpar_c.partners_pkw.10078_plc.Skimbit%20Ltd._pcrid.449670_learning&trk=aff_src.aff-lilpar_c.
        partners_pkw.10078_plc.Skimbit%20Ltd._pcrid.449670_learning&clickid=
        V8yWcW3eTxyORoBwUx0Mo3ERUknSaeR1iz9G2M0&irgwc=1
            news sources, influencers, companies, schools, and groups.
            # TODO LATER while scraping if I find someone that has news source report it
            # TOTO LATER add "special report" to log

        Returns
        -------
        interests : a list of dict, with the following key values


            popup           : bool, , default None
                True    : if the profile has a popup
                False   : if the profile has no popup

            interests_list  : a list containing the interests
                if popup is True:
                    it will return the tabs present in the popup
                    e.g: ["companies", "schools"] for carolina
                if popup is False:
                    it will return a list of dictionaries with the following keys : values
                        name        : str, default None
                        followers   : int, default None
                        url         : str, default None
                if popup = None:
                    retruns an empty list, []

            companies       : a list of dictionaries with the following keys : values, default []
                name        : str, default None
                followers   : int, default None
                url         : str, default None

            schools         : a list of dictionaries with the following keys : values, default []
                name        : str, default None
                followers   : int, default None
                url         : str, default None

            groups          : a list of dictionaries with the following keys : values, default []
                name        : str, default None
                followers   : int, default None
                url         : str, default None

            influencers     : a list of dictionaries with the following keys : values, default []
                name        : str, default None
                headline    : str, default None
                followers   : int, default None
                url         : str, default None


        """
        logger_people_scraper.debug("session, started, interests")
        # Init

        self.interests = {
            "popup": None,  #
            "interests_list": [],  #
            "interests_list_no_popup": [],  #
            "influencers": [],  #
            "companies": [],  #
            "groups": [],  #
            "schools": []  #
        }

        if self.config["interest_popup"]:
            self.interests["popup"] = True
            # find popup button
            self.click_on(By.CSS_SELECTOR, '.pv-interests-section a[data-control-name="view_interest_details"]')

            tabs = self.select_elements(By.CSS_SELECTOR, 'nav.pv-profile-detail__nav ul > li')

            for tab in tabs:
                # tab index
                i = tab.get_attribute("class").replace("pv-interests-modal__following-", "")
                self.interests["interests_list"].append(i)

                list_info = self.get_popup(items.get_interest_info,
                                           tab,
                                           By.CSS_SELECTOR, 'a[data-control-name=following_' + i + ']',
                                           By.CSS_SELECTOR, '.entity-list-wrapper .entity-list-item',
                                           By.CSS_SELECTOR, '.pv-profile-detail__modal button[aria-label="Dismiss"]',
                                           limit_list=self.config["popup_limit"],
                                           dismiss=False
                                           )
                list_info = mt.flatten_list(list_info)
                self.interests.update({i: list_info})

            self.click_on(By.CSS_SELECTOR, '.pv-profile-detail__modal button[aria-label="Dismiss"]')

        elif not self.config["interest_popup"]:
            self.interests["popup"] = False
            self.interests["interests_list_no_popup"] = self.get_data_in_section(
                section="interests",
                method=By.CSS_SELECTOR,
                selector=".pv-interests-section",
                list_method=By.CSS_SELECTOR,
                list_selector=".pv-interests-section li.pv-interest-entity",
                get_info_function=items.get_interest_info)

        self.data.update({"interests": self.interests})
        logger_people_scraper.debug("session, done, interests")
        return self.interests

    def get_people_also_viewed(self):
        """
        Updates person_data with the people also viewed section

        Returns
        -------
        people_also_viewed : list of dict, default empty list
            Returns a list of dictionaries with the following {keys: values}

            full_name   : str, default None.
            headline    : str, default None.
            dist_value    : int, default None
            url  : str, default None.

        """
        self.people_also_viewed = []
        self.people_also_viewed = self.get_data_in_section(
            section="people_also_viewed",
            method=By.CSS_SELECTOR,
            selector="section .pv-browsemap-section",
            list_method=By.CSS_SELECTOR,
            list_selector="section .pv-browsemap-section .pv-browsemap-section__member-container",
            get_info_function=items.get_people_browser_info)
        self.data.update({"people_also_viewed": self.people_also_viewed})

        return self.people_also_viewed

    def scrape(self):
        """
        Returns
        -------
            data : dict
                see profile data structure for more details

        """

        # Define the status
        self.scraping_status = "running"

        # Load the profile
        self.load_page(By.ID, "profile-wrapper", By.CSS_SELECTOR, ".pv-profile-unavailable", "/in/")
        self.get_meta_data("start")
        # Section background
        self.get_pv_top_card_info()

        get_functions = [

            self.get_contact_info,

            # self.get_featured,

            # self.get_activity,

            # Get summary
            self.get_summary,

            # Get highlights
            # self.get_highlights,

            # Get experience
            self.get_experience,

            # Get education
            self.get_education,

            # Get Licenses and certifications
            self.get_licenses_and_certifications,

            # Get volunteering
            self.get_volunteering,

            # Get skills
            self.get_skills,

            # Get recommendations
            self.get_recommendations,

            # Get accomplishments
            self.get_accomplishments,

            # Get Interests
            self.get_interests,

            # Get people also viewed
            self.get_people_also_viewed]

        # Shuffle the functions to simulate a random behavior
        random.shuffle(get_functions)
        for fun in get_functions:
            fun()

        self.extra = {
            "name": None,
            "len": None,
            "tag": None,
            "added": None,
            "other": None
        }
        self.data.update({"extra": self.extra})
        self.save_html_and_png_full_page()
        logger_people_scraper.info("done , " +
                                 " , " +
                                   self.data["init"]["id"] +
                                 " , " +
                                   self.data["init"]["url"])

        self.get_meta_data("end")

        self.scraping_status = "completed"
