"""
Title     : TODO
Objective : TODO
Created by: Abdellah Janid
Created on: 6/1/20
"""

# Standard library imports
from enum import Enum
from typing import Optional, Union, List

from pydantic import BaseModel, Field
# Related third party imports
from pydantic import EmailStr

# Import local applications
from scraper.general_schemas import InitModel, ExtraModel


#############################################
# Top card
#############################################

# noinspection PyMissingOrEmptyDocstring
class PersonMembershipsModel(BaseModel):
    premium: Optional[bool] = Field(...)
    influencer: Optional[bool] = Field(...)
    jobSeeker: Optional[bool] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Membership'
        fields = {
            'premium': 'premium',
            'influencer': 'influencer',
            'jobSeeker': 'jobSeeker'

        }

        allow_population_by_field_name = True


# noinspection PyMissingOrEmptyDocstring
class PersonTopCardModel(BaseModel):
    full_name: str = Field(...,
                           title='Full name',
                           description='Full name of the person',
                           examples='Abdellah Janid',
                           min_length=2
                           )

    # it can be None: e.g rasmus
    dist_value: Optional[str] = Field(...,
                                      title='Distance',
                                      description='Distance to the person',
                                      examples='"1st", "2nd", "3rd", None'
                                      )

    headline: str = Field(...,
                          title='Headline',
                          description='MSc. Student of Applied Economics And Finance at CBS',
                          examples=''
                          )
    tags: Optional[str] = Field(...,
                          title='Tags',
                          description='Tags of topics the individuals talks about eg #research',
                          examples=''
                          )
    location: str = Field(...,
                          title="Location",
                          description='Location of the person',
                          examples='Copenhagen, Capital Region, Denmark'
                          )

    # it can be None: e.g rasmus
    connections: Optional[str] = Field(...,
                                       title="Connections",
                                       description='Connections',
                                       examples='500+ connections'
                                       )

    # it can be None: e.g rasmus
    dist_value_int: Optional[int] = Field(...,
                                          title="Distance",
                                          examples='1, 2, 3, None'
                                          )
    # it can be None: e.g rasmus
    connections_int: Optional[int] = Field(...,
                                           title="Version",
                                           examples='500,or None'
                                           )

    membership_info: PersonMembershipsModel = Field(...)

    image: bool = Field(...,
                        title='image',
                        description='True if the person has image, False otherwise'
                        )

    followersCount: int = Field(...,
                                title="Followers",
                                description='Number of followers',
                                examples='1441'
                                )
    extra_top_card : Optional[dict] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            'full_name': 'full_name',
            'dist_value': 'dist_value',
            'headline': 'headline',
            'tags': 'tags',
            'location': 'location',
            'connections': 'connections',
            'dist_value_int': 'dist_value_int',
            'connections_int': 'connections_int',
            'membership_info': 'membership_info',
            'image': 'image',
            'followersCount': 'followersCount',
            'extra_top_card': 'extra_top_card'

        }
        allow_population_by_field_name = True


class PersonTopCardSchema(BaseModel):
    """
    Top card information
    """
    top_card_info: PersonTopCardModel = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Top card info'
        fields = {
            'top_card_info': 'top_card_info'
        }
        allow_population_by_field_name = True


#############################################
# Contact info
#############################################

# noinspection PyMissingOrEmptyDocstring
class PersonContactInfoModel(BaseModel):
    address: Optional[str] = Field(...,
                                   title="Address",
                                   description='Address of the person',
                                   examples='Vestergade 13, 1456 Copenhagen, Denmark'
                                   )
    email: Optional[EmailStr] = Field(...,
                                      title="Email",
                                      description='Email of the person',
                                      examples='janid.abdellah@gmail.com'
                                      )
    birthday: Optional[str] = Field(...,
                                    title="birthday",
                                    description='birthday of the person, in the format: %B %d',
                                    examples='March 12'
                                    )
    phone: Optional[str] = Field(...,
                                 title="Phone number",
                                 description='Phone number',
                                 examples='+4529937119\n (Mobile)'
                                 )
    connected: Optional[str] = Field(...,
                                     title="Connedted date",
                                     description='Date in which the scraper has connected with the person in the format '
                                                 '"%B %d, %Y"',
                                     examples='October 4, 2018'
                                     )
    twitter: Optional[str] = Field(...,
                                   title="Twitter",
                                   description='Twitter account'
                                   )
    # I did put them as string until I verify how websites are acquired from a profile
    websites: List[str] = Field(...,
                                title="websites",
                                description='Websites',
                                examples='["http://aryze.io",]'
                                )
    im: List[str] = Field(...,
                          title="Instant messaging",
                          description='Instant messaging',
                          examples='[janid.a(Skype)]'
                          )

    contact_info_headers: List[str] = Field(...,
                                            title="Contact info header",
                                            description='A list of the info showed in the contact info'
                                            )
    first_name: str = Field(...,
                            title="First name",
                            description='Inferred First Name',
                            examples="Abdellah"
                            )
    last_name: str = Field(...,
                           title="Last name",
                           description='Inferred Last name',
                           examples="Janid"
                           )

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            'address': 'address',
            'email': 'email',
            'birthday': 'birthday',
            "phone": "phone",
            "connected": "connected",
            "twitter": "twitter",
            "websites": "websites",
            "im": "im",
            "contact_info_headers": "contact_info_headers",
            "first_name": "first_name",
            "last_name": "last_name"
        }

        allow_population_by_field_name = True


class PersonContactInfoSchema(BaseModel):
    """
    Contact info
    """
    contact_info: PersonContactInfoModel = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Contact info'
        fields = {
            'contact_info': 'contact_info'
        }
        allow_population_by_field_name = True


#############################################
# Highlights
#############################################

# noinspection PyMissingOrEmptyDocstring
class PersonHighlightsModel(BaseModel):
    mutual_connections: Optional[int] = Field(None,
                                              title="Last name",
                                              description='Inferred Last name',
                                              examples="52"
                                              )
    url_connections: Optional[str] = Field(None,
                                           title="Connections",
                                           description='Link to mutual connections and second degree connections'
                                           )

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            'mutual_connections': 'mutual_connections',
            'url_connections': 'url_connections'
        }

        allow_population_by_field_name = True


class PersonHighlightsSchema(BaseModel):
    """
    Highlights Mutual connections
    """
    highlights: PersonHighlightsModel = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Highlights'
        fields = {
            'highlights': 'highlights'
        }
        allow_population_by_field_name = True


#############################################
# Experiences
#############################################
class EmploymentTypeEnum(str, Enum):
    """
    Enum to check that the employment type is part of the set
    """
    full_time = 'Full-time'
    part_time = 'Part-time'
    self_employed = 'Self-employed'
    freelance = 'Freelance'
    contract = 'Contract'
    internship = 'Internship'
    apprenticeship = 'Apprenticeship'
    seasonal = 'Seasonal'
    permanent = 'Permanent'


# noinspection PyMissingOrEmptyDocstring
class PersonExperienceModel(BaseModel):
    title: str = Field(...,
                       title="Title",
                       description='Title of the position'
                       )
    employment_type: Optional[EmploymentTypeEnum] = Field(...,
                                                          title="Employment type",
                                                          description='Employment type: Full-time, Part-time, '
                                                                      'Self-employed, Freelance, Contract, '
                                                                      'Internship, Apprenticeship',
                                                          examples="Full-time"
                                                          )

    date_range: Optional[str] = Field(...,
                                      title="Employment type",
                                      description='date employed in the job position in the format: %b %Y ',
                                      examples="Sep 2015 \u2013 Jun 2016"
                                      )

    location: Optional[str] = Field(...,
                                    title="Location",
                                    description='location of of the job position',
                                    examples="Bologna Area, Italy"
                                    )
    description: Optional[str] = Field(...,
                                       title="Description",
                                       description='job description',

                                       )

    organization: str = Field(...,
                              title="Company",
                              description='Company name',
                              examples="Sep 2015 \u2013 Jun 2016"
                              )
    url: Optional[str] = Field(...,
                               title="Company webpage",
                               description='Company webpage',
                               examples="https://www.linkedin.com/company/capgemini/"
                               )

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            'title': 'title',
            'employment_type': 'employment_type',
            'date_range': 'date_range',
            'location': 'location',
            'description': 'description',
            'organization': 'organization',
            'url': 'url'
        }
        allow_population_by_field_name = True


class PersonExperienceSchema(BaseModel):
    """
    Experiences
    """
    experience: List[PersonExperienceModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Experiences'
        fields = {
            'experience': 'experience'
        }
        allow_population_by_field_name = True


#############################################
# Education
#############################################
# noinspection PyMissingOrEmptyDocstring
class PersonEducationModel(BaseModel):
    school: str = Field(...)
    degree: Optional[str] = Field(...)
    field_of_study: Optional[str] = Field(...)
    date_range: Optional[str] = Field(...)
    grades: Optional[str] = Field(...)
    activities: Optional[str] = Field(...)
    description: Optional[str] = Field(...)
    url: Optional[str] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "school": "school",
            "degree": "degree",
            "field_of_study": "field_of_study",
            "date_range": "date_range",
            "grades": "grades",
            "activities": "activities",
            "description": "description",
            "url": "url"
        }
        allow_population_by_field_name = True


class PersonEducationSchema(BaseModel):
    """
    Education
    """
    education: List[PersonEducationModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Education'
        fields = {
            'education': 'education'
        }
        allow_population_by_field_name = True


#############################################
# Licenses and certifications
#############################################
# noinspection PyMissingOrEmptyDocstring
class PersonLicensesModel(BaseModel):
    name: str = Field(...)
    issuing_organization: Optional[str] = Field(...)
    issue_date: Optional[str] = Field(...)
    expiration_date: Optional[str] = Field(...,
                                           examples="'No Expiration Date', 'Expires %b %Y' e.g 'Expires Dec 2021', "
                                                    "'Expired %b %Y' e.g 'Expired Mar 2016' "
                                           )
    credential_id: Optional[str] = Field(...)
    credential_url: Optional[str] = Field(...)
    url: Optional[str] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "name": "name",
            "issuing_organization": "issuing_organization",
            "issue_date": "issue_date",
            "expiration_date": "expiration_date",
            "credential_id": "credential_id",
            "credential_url": "credential_url",
            "url": "url"
        }
        allow_population_by_field_name = True


class PersonLicensesSchema(BaseModel):
    """
    Licenses
    """
    lic_and_cert: List[PersonLicensesModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Licenses and certifications'
        fields = {
            'lic_and_cert': 'certifications'
        }
        allow_population_by_field_name = True


#############################################
# Volunteering
#############################################

# noinspection PyMissingOrEmptyDocstring
class PersonVolunteeringModel(BaseModel):
    organization: str = Field(...)
    role: Optional[str] = Field(...)
    cause: Optional[str] = Field(...)
    date_range: Optional[str] = Field(...,
                                      examples='"Sep 2016", "Oct 2016 \u2013 Nov 2017, "2005 \u2013 2007", "Nov 2017 '
                                               '\u2013 Present"')
    description: Optional[str] = Field(...)
    url: Optional[str] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "organization": "organization",
            "role": "role",
            "cause": "cause",
            "date_range": "date_range",
            "description": "description",
            "url": "url"
        }
        allow_population_by_field_name = True


class PersonVolunteeringSchema(BaseModel):
    """
    Volunteering
    """
    volunteering: List[PersonVolunteeringModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Volunteering'
        fields = {
            'volunteering': 'volunteering'
        }
        allow_population_by_field_name = True


#############################################
# Person
#############################################

# TESTED carolina japas, she has no endorsments
# noinspection PyMissingOrEmptyDocstring
class PersonShortModel(BaseModel):
    full_name: str = Field(...)
    headline: Optional[str] = Field(..., examples="34")
    url: str = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "full_name": "full_name",
            "headline": "headline",
            "url": "url"
        }
        allow_population_by_field_name = True


#############################################
# Skills
#############################################

# noinspection PyMissingOrEmptyDocstring
class PersonSkillsModel(BaseModel):
    name: str = Field(..., examples="Project Management")
    endorsements: Optional[int] = Field(..., examples="34")
    verified: Optional[bool] = Field(..., examples="Passed: LinkedIn Assessment")
    endorsers: List[PersonShortModel] = Field(None)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "name": "name",
            "endorsements": "endorsements",
            "verified": "verified",
            "endorsers": "endorsers"
        }
        allow_population_by_field_name = True


class PersonSkillsSchema(BaseModel):
    """
    Skills
    """
    skills: List[PersonSkillsModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Skills'
        fields = {
            'skills': 'skills'
        }
        allow_population_by_field_name = True


#############################################
# Recommendations
#############################################

# noinspection PyMissingOrEmptyDocstring
class PersonRecommendationModel(BaseModel):
    full_name: Optional[str] = Field(...)
    headline: Optional[str] = Field(...)
    recommendation: Optional[str] = Field(...)
    relationship_info: Optional[str] = Field(...)
    date: Optional[str] = Field(..., examples='format "%B %d, %Y", e.g "October 18, 2018"')
    relationship: Optional[str] = Field(...)
    url: Optional[str] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "full_name": "full_name",
            "headline": "headline",
            "recommendation": "recommendation",
            "relationship_info": "relationship_info",
            "date": "date",
            "relationship": "relationship",
            "url": "url"
        }
        allow_population_by_field_name = True


# noinspection PyMissingOrEmptyDocstring
class PersonRecommendationsModel(BaseModel):
    received: List[PersonRecommendationModel] = Field(...)
    given: List[PersonRecommendationModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "received": "received",
            "given": "given"

        }
        allow_population_by_field_name = True


class PersonRecommendationsSchema(BaseModel):
    """
    Recommendations
    """
    recommendations: PersonRecommendationsModel = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Recommendations'
        fields = {
            'recommendations': 'recommendations'
        }
        allow_population_by_field_name = True


#############################################
# Accomplishments
#############################################
# Publications
# -------------
# noinspection PyMissingOrEmptyDocstring
class PublicationsModel(BaseModel):
    title: Optional[str] = Field(...)
    date: Optional[str] = Field(...)
    publisher: Optional[str] = Field(...)
    description: Optional[str] = Field(...)
    url: Optional[str] = Field(...)
    collaborators: List[PersonShortModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "title": "title",
            "date": "date",
            "publisher": "publisher",
            "description": "description",
            "url": "url",
            "collaborators": "collaborators"

        }
        allow_population_by_field_name = True


# Patents
# -------------
# noinspection PyMissingOrEmptyDocstring
class PatentsModel(BaseModel):
    title: Optional[str] = Field(...)
    issuer_and_nr: Optional[str] = Field(...)
    status_and_date: Optional[str] = Field(...)
    description: Optional[str] = Field(...)
    url: Optional[str] = Field(...)
    collaborators: List[PersonShortModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "title": "title",
            "issuer_and_nr": "issuer_and_nr",
            "status_and_date": "status_and_date",
            "description": "description",
            "url": "url",
            "collaborators": "collaborators"
        }
        allow_population_by_field_name = True


# courses
# -------------
# noinspection PyMissingOrEmptyDocstring
class CoursesModel(BaseModel):
    name: Optional[str] = Field(...)
    number: Optional[str] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "name": "name",
            "number": "number"
        }
        allow_population_by_field_name = True


# ProjectsModel
# -------------
# noinspection PyMissingOrEmptyDocstring
class ProjectsModel(BaseModel):
    name: str = Field(...)
    date: Optional[str] = Field(...)
    description: Optional[str] = Field(...)
    url: Optional[str] = Field(...)
    collaborators: List[PersonShortModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "name": "name",
            "date": "date",
            "description": "description",
            "url": "url",
            "collaborators": "collaborators"
        }
        allow_population_by_field_name = True


# Honors
# -------------
# noinspection PyMissingOrEmptyDocstring
class HonorsModel(BaseModel):
    title: Optional[str] = Field(...)
    issuer: Optional[str] = Field(...)
    date: Optional[str] = Field(...)
    description: Optional[str] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "title": "title",
            "issuer": "issuer",
            "date": "date",
            "description": "description"
        }
        allow_population_by_field_name = True


# LanguagesModel
# -------------
# noinspection PyMissingOrEmptyDocstring
class LanguagesModel(BaseModel):
    language: Optional[str] = Field(...)
    proficiency: Optional[str] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "language": "language",
            "proficiency": "proficiency"

        }
        allow_population_by_field_name = True


# OrganizationsModel
# -------------
# noinspection PyMissingOrEmptyDocstring
class OrganizationsModel(BaseModel):
    name: str = Field(...)
    position: Optional[str] = Field(...)
    date: Optional[str] = Field(...)
    description: Optional[str] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "name": "name",
            "position": "position",
            "date": "date",
            "description": "description"
        }
        allow_population_by_field_name = True


# Test_scoresModel
# -------------
# noinspection PyMissingOrEmptyDocstring
class ScoresModel(BaseModel):
    name: str = Field(...)
    date: Optional[str] = Field(...)
    score: Optional[str] = Field(...)
    description: Optional[str] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "name": "name",
            "date": "date",
            "score": "score",
            "description": "description"
        }
        allow_population_by_field_name = True


# noinspection PyMissingOrEmptyDocstring
class PersonAccomplishmentsModel(BaseModel):
    publications: List[PublicationsModel] = Field(...)
    patents: List[PatentsModel] = Field(...)
    courses: List[CoursesModel] = Field(...)
    projects: List[ProjectsModel] = Field(...)
    honors: List[HonorsModel] = Field(...)
    languages: List[LanguagesModel] = Field(...)
    organizations: List[OrganizationsModel] = Field(...)
    test_scores: List[ScoresModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "publications": "publications",
            "patents": "patents",
            "courses": "courses",
            "projects": "projects",
            "honors": "honors",
            "languages": "languages",
            "organizations": "organizations",
            "test_scores": "test_scores"
        }
        allow_population_by_field_name = True


class PersonAccomplishmentsSchema(BaseModel):
    """
    Accomplishments
    """
    accomplishments: PersonAccomplishmentsModel = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'accomplishments'
        fields = {
            'accomplishments': 'accomplishments'
        }
        allow_population_by_field_name = True


#############################################
# Interests
#############################################

# noinspection PyMissingOrEmptyDocstring
class PersonEntityInfoModel(BaseModel):
    name: str = Field(...)
    headline: Optional[str] = Field(...)
    followers: Optional[int] = Field(...)
    url: Optional[str] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "name": "name",
            "headline": "headline",
            "followers": "followers",
            "url": "url"
        }
        allow_population_by_field_name = True


# noinspection PyMissingOrEmptyDocstring
class PersonInterestsModel(BaseModel):
    popup: Optional[bool] = Field(...)
    interests_list: List[str] = Field(...)
    interests_list_no_popup: List[PersonEntityInfoModel] = Field(...)
    influencers: List[PersonEntityInfoModel] = Field(...)
    companies: List[PersonEntityInfoModel] = Field(...)
    groups: List[PersonEntityInfoModel] = Field(...)
    schools: List[PersonEntityInfoModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "popup": "popup",
            "interests_list": "interests_list",
            "interests_list_no_popup": "interests_list_no_popup",
            "influencers": "influencers",
            "companies": "companies",
            "groups": "groups",
            "schools": "schools"
        }
        allow_population_by_field_name = True


class PersonInterestsSchema(BaseModel):
    """
    Interests
    """
    interests: PersonInterestsModel = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Interests'
        fields = {
            'interests': 'interests'
        }
        allow_population_by_field_name = True


#############################################
# People also viewed
#############################################
# noinspection PyMissingOrEmptyDocstring
class PersonPeopleAlsoViewedModel(BaseModel):
    full_name: Optional[str] = Field(...)
    headline: Optional[str] = Field(...)
    dist_value: Optional[str] = Field(...)
    url: Optional[str] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        fields = {
            "full_name": "full_name",
            "headline": "headline",
            "dist_value": "dist_value",
            "url": "url"
        }
        allow_population_by_field_name = True


class PersonPeopleAlsoViewedSchema(BaseModel):
    """
    People also viewed panel in the right hand side
    """
    people_also_viewed: List[PersonPeopleAlsoViewedModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'People also viewed'
        fields = {
            'people_also_viewed': 'people_also_viewed'
        }
        allow_population_by_field_name = True


#############################################
# Person info All
#############################################

class LiPersonAllModel(BaseModel):
    """
    A dictionary containing info scraped from Linkedin profile of a given person
    """
    # init json schema validation
    # variable : type  = Field/schema
    init: InitModel = Field(...)
    extra: ExtraModel = Field(...)
    top_card_info: PersonTopCardModel = Field(...)
    contact_info: PersonContactInfoModel = Field(...)
    summary: Optional[str] = Field(...)
    highlights: Union[dict, PersonHighlightsModel] = Field(...)
    experience: List[PersonExperienceModel] = Field(...)
    education: List[PersonEducationModel] = Field(...)
    lic_and_cert: List[PersonLicensesModel] = Field(...)
    volunteering: List[PersonVolunteeringModel] = Field(...)
    skills: List[PersonSkillsModel] = Field(...)
    recommendations: PersonRecommendationsModel = Field(...)
    accomplishments: PersonAccomplishmentsModel = Field(...)
    interests: PersonInterestsModel = Field(...)
    people_also_viewed: List[PersonPeopleAlsoViewedModel] = Field(...)

    # noinspection PyMissingOrEmptyDocstring
    class Config:
        title = 'Person info'
        fields = {
            'init': 'init',
            'extra': 'extra',
            'top_card_info': 'top_card_info',
            'contact_info': 'contact_info',
            'summary': 'summary',
            'highlights': 'highlights',
            'experience': 'experience',
            'education': 'education',
            'lic_and_cert': 'certifications',
            'volunteering': 'volunteering',
            'skills': 'skills',
            'recommendations': 'recommendations',
            'accomplishments': 'accomplishments',
            'interests': 'interests',
            'people_also_viewed': 'people_also_viewed'
        }
        extra = 'allow'
        allow_population_by_field_name = True
