"""
Title     : Schools
Objective : Provide tools to scrape data from a list of Schools
Created by: Abdellah Janid
Created on: 6/2/20
"""
# Standard library imports
import logging
import random
import re
import urllib.request
from pathlib import Path
from urllib.parse import urljoin, urlparse
# Import local applications
from scraper.linkedin.schools import items
from scraper.session import ScraperSession
from scraper import scraping_tools as mt

# Related third party imports
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By


# Create a module logger
logger_schools_scraper = logging.getLogger('schools_scraper')
# Set level
logger_schools_scraper.setLevel(logging.DEBUG)


class Schools(ScraperSession):
    """Scrape the list of employees of a given company"""

    def get_top_card_info(self):
        """
        Get the info of top card
        Returns
        -------
        dict

        """
        self.top_card_info = {}
        top_card = self.scroll_select(By.CSS_SELECTOR, '.org-top-card',
                                      return_type="bs_el",
                                      html_name="org_top_card",
                                      html_path=self.html_path_profile,
                                      screenshot_name="org_top_card",
                                      screenshot_path=self.screenshots_path_profile,
                                      )
        self.top_card_info = items.get_top_card(top_card)

        self.data.update({"top_card_info": self.top_card_info})
        logger_schools_scraper.debug("top_card_info, done")
        return self.top_card_info

    def get_logo(self):
        """ Get the logo"""
        logo_element = self.select_element(By.CSS_SELECTOR,
                                           '.org-top-card-primary-content__logo-container img',
                                           return_type="bs_el",
                                           html=False,
                                           screenshot_name=self.file_name + "_logo",
                                           screenshot_path=self.screenshots_path_profile,
                                           )

        if logo_element:
            # select the attribute src in the img tag
            urllib.request.urlretrieve(logo_element.img["src"],
                                       Path(self.media_path,
                                            self.file_name_ts + ".png"))
        logger_schools_scraper.debug("getting logo, done")

    def extract_keys(self, keys_list, meta_section):
        """
        Returns a dictionary from the list of string
        Parameters
        ----------
        keys_list : List[str]
            list of desired keys to extract
        meta_section : dict
            the key under which look for the keys
        Returns
        -------
            Updates data with the listed keys
        """
        for k in keys_list:
            # noinspection PyBroadException
            try:
                if k in ["description"]:
                    info = list(mt.find_keys(meta_section, k))
                else:
                    info = list(mt.find_keys(meta_section, k))[0]
                mt.scrub(info, "$type")
                self.data.update({k: info})
            except:
                logger_schools_scraper.debug("key not found " + k)

    def get_grouped_locations(self):
        """
        Get grouped locations form metadata
        """
        meta_data_grouped_locations = mt.desired_dict("groupedLocations", self.meta_data)
        elements = meta_data_grouped_locations["data"]["elements"]

        # get groupedLocations
        for i in elements:
            if "groupedLocations" in i.keys():
                self.groupedLocations = i["groupedLocations"]

        # Clean the dictionary by removing the key $type
        mt.scrub(self.groupedLocations, bad_key="$type")
        self.data.update({"groupedLocations": self.groupedLocations})
        logger_schools_scraper.debug("groupedLocations, done")

    def get_extra_info(self):
        """
        Get extra data from the meta data
        """

        # Alumni
        meta = mt.desired_dict("totalStudentsAndAlumni", self.meta_data)
        keys = ["totalStudentsAndAlumni"]
        self.extract_keys(keys, meta["included"])

        meta = mt.desired_dict("fundingData", self.meta_data)
        keys = ["description", "staffCount", "staffCountRange", "phone", "tagline",
                "associatedHashtags", "companyPageUrl", "specialities", "fundingData", "foundedOn", "companyType"]
        self.extract_keys(keys, meta["included"])
        logger_schools_scraper.debug("Getting extra info, done")

    def scrape(self):
        """
        Returns
        -------
            data : dict
                see profile data structure for more details

        """
        # Define the status
        self.scraping_status = "running"
        if "legacySchoolId" in self.url:
            self.driver.get(self.url)
            self.url = self.driver.current_url

        if not self.url.endswith("about/"):
            o = urlparse(self.url)
            o = o._replace(query='')
            o = urljoin(o.geturl(), 'about/')
            self.url = o
            logger_schools_scraper.debug('Adding "about" to the url : ' + self.url)
        # Load the profile
        self.load_page(By.CSS_SELECTOR, ".scaffold-layout-container",
                       By.CSS_SELECTOR, ".artdeco-empty-state__headline",
                       "/school/")

        # Get info from the top banner
        self.get_top_card_info()
        self.get_logo()
        self.get_meta_data()
        self.get_grouped_locations()
        self.get_extra_info()

        # Add extra dictionary info
        self.extra = {
            "name": None,
            "len": None,
            "tag": None,
            "added": None,
            "other": None
        }
        self.data.update({"extra": self.extra})
        # -----------------------------------------------------------------------
        # Save html of tabs for future inspections
        # -----------------------------------------------------------------------
        # -----------------------------------------------------------------------
        # Save html of tabs for future inspections
        # -----------------------------------------------------------------------
        tabs = self.select_elements(By.CSS_SELECTOR, 'nav.org-page-navigation ul > li')
        random.shuffle(tabs)
        for tab in tabs:
            tab.click()
            page = self.select_element(method=By.CSS_SELECTOR, selector='a.active', web_element=tab)
            print("getting tab : " + tab.text)
            if page:
                mt.save_html(self.driver.page_source, self.html_path_profile, tab.text)
                self.get_meta_data(tab.text.lower())


            mt.random_rest(self.config["driver"]["click_pause"]["min"],
                           self.config["driver"]["click_pause"]["max"])

        self.scraping_status = "completed"
