"""
Title     : Tools to add links to list
Created by: Abdellah Janid
Created on: 6/3/20
"""
# Standard library imports
import json
import logging
from datetime import datetime
from pathlib import Path

# Related third party imports
import pandas as pd
import numpy as np

import scraper.exceptions as exceptions
# Import local applications
from scraper import init_configuration
# Create a module logger
from scraper.scraping_tools import save_json
from scraper import scraping_tools as mt

logger_list = logging.getLogger('logger_list')
# Set level
logger_list.setLevel(logging.DEBUG)


class ListManagement(object):
    """Class for managing the links list"""

    def __init__(self, source, add, target):
        """ Contains methods to run the scraper and manage the list of links to scrape and run tests
        Parameters
        ----------
        source : tuple
            source from with add the data, eg source and folder: companies data
                path where the data of the session will be saved.
                "watchdog" : is the folder meant for saving data validation, routine check and tests
                "data : is the main folder where the data is saved
        add : tuple
            method : str
                'all' or 'selected'
            data
            target to add: ['people','followers','following','connections','companies','employees',
            'groups', 'schools', ''jobs_list]
        """
        # New
        self.source, self.source_folder = source
        self.method, self.data_to_add = add
        self.target, self.target_folder = target

        # path variable
        self.wd_path = None  # working directory path
        self.source_list_path = None
        self.profiles_path = None
        self.target_list_path = None
        # dataframe variable
        self.source_list_df = None
        self.target_list_df = None

        # get configuration
        self.config = init_configuration(self.source)
        # set paths
        self.set_paths()

    def set_paths(self):
        """ Set paths
        """
        self.wd_path = Path(Path.home(), "brobygger_li")
        # List from which links are going to be added
        self.source_list_path = Path(self.wd_path,
                                     self.source,
                                     self.source_folder,
                                     "links_list",
                                     self.config["links_list"][self.source])

        self.profiles_path = Path(self.wd_path,
                                  self.source,
                                  self.source_folder,
                                  "profiles")

        # List to which links are going to be added
        self.target_list_path = Path(self.wd_path,
                                     self.target,
                                     self.target_folder,
                                     "links_list",
                                     self.config["links_list"][self.target])

        logger_list.debug("source list path :" + str(self.source_list_path))
        logger_list.debug("profiles path :" + str(self.profiles_path))
        logger_list.debug("target list path :" + str(self.target_list_path))

    def load_list(self, list_path):
        """Load the list
        Parameters
        ----------
        list_path : str
            Path to the links list to load
        Returns
        -------
        list_df : pandas.core.frame.DataFrame
            The list with links list
        Raises
        ------
        LinksListError
            Invalid list, missing columns
        """
        logger_list.info("loading the list: " + str(list_path))
        # Load the list
        list_df = pd.read_csv(list_path, sep=";")  # import list
        # strip the list from white spaces, to avoid error
        list_df = list_df.rename(columns=lambda x: x.strip())

        # Validate the list
        list_validations_rules = ["status", "priority", "link", "id", "info", "file_name_source", "source_description",
                                  "timestamp_start",
                                  "timestamp_end", "command", "error_msg", "tag", "added", "select"]
        if all(name in list(list_df.columns) for name in list_validations_rules):
            logger_list.info("list validated")
        else:
            logger_list.critical(
                "Invalid list, missing columns: " + str(set(list_validations_rules).difference(list(list_df.columns))))
            raise exceptions.LinksListError("Invalid list, missing columns:",
                                            set(list_validations_rules).difference(list(list_df.columns)))
        return list_df

    def add(self, method):
        """ Add links to the appropriate list based on the folder and target used to initiate the instance
        Parameters
        ----------
        method : str
            all or selected. If all, it will set the column operation to add

        """

        self.source_list_df = self.load_list(self.source_list_path)
        self.target_list_df = self.load_list(self.target_list_path)
        # set the columns as object type, to add list to a cell

        self.source_list_df['added'] = self.source_list_df['added'].astype('object')
        self.target_list_df['added'] = self.target_list_df['added'].astype('object')

        if self.method == "all":
            df_of_interest = self.source_list_df[self.source_list_df.status.eq("completed")]['added']
        else:
            df_of_interest = self.source_list_df[self.source_list_df.status.eq("completed") &
                                                 self.source_list_df.select.eq("y")]['added']

        for i, o in list(df_of_interest.items()):
            # if the cell is nan fill with an empty list
            if pd.isnull(self.source_list_df.at[i, 'added']):
                self.source_list_df.at[i, 'added'] = []
            # convert to a python list, specify engine to avoind error @https://stackoverflow.com/questions/54759936/extension-dtypes-in-pandas-appear-to-have-a-bug-with-query
            added = pd.eval(self.source_list_df.at[i, 'added'], engine='python')

            if self.data_to_add not in added:
                # get the name of the file in profiles
                id = self.source_list_df.iloc[i, self.source_list_df.columns.get_loc('id')]
                timestamp = self.source_list_df.iloc[
                    i, self.source_list_df.columns.get_loc('timestamp_start')]

                timestamp_obj = datetime.strptime(timestamp, "%d/%m/%y %H:%M:%S")
                timestamp = datetime.strftime(timestamp_obj, "%d_%m_%y_(%H_%M_%S)")
                file_name = id + "_" + timestamp + ".json"

                # open the json file
                file = Path(self.profiles_path, file_name)
                with file.open() as fp:
                    data = json.load(fp)

                data_to_add_map ={}
                collaborators = []
                highlights = []
                # mapping the right class for scraping
                if self.source == "companies":
                    data_to_add_map = {
                        "employees": [data["top_card_info"]["url_employees"]],
                        "connections_in_org": [data["top_card_info"]["url_connections_in_org"]],
                        "similar_companies": [d["url"] for d in data["similar_companies"] if d["url"]]
                    }
                elif self.source == "schools":
                    data_to_add_map = {
                        "employees": [data["top_card_info"]["url_employees"]],
                        "connections_in_org": [data["top_card_info"]["url_connections_in_org"]]
                    }
                elif self.source == "employees":
                    data_to_add_map = {
                        "employees": [d["url"] for d in data["employees"] if (d["url"] and ("/in/" in d["url"]))],
                        "mutual_connections_url": [d["mutual_connections_url"] for d in data["employees"] if
                                                   d["mutual_connections_url"]],
                        # "mutual_connections": mt.flatten_list(
                        #     [d["mutual_connections"] for d in data["employees"] if d["mutual_connections"]])
                    }
                elif self.source == "people":
                    if data["highlights"]:
                        highlights = [data["highlights"] ["url_connections"]]
                    else:
                        highlights =[]

                    if self.data_to_add in ['publications_collaborators', 'patents_collaborators',
                                            'projects_collaborators']:
                        collaborators = []
                        for p in data["accomplishments"][self.data_to_add.split('_')[0]]:
                            for c in p["collaborators"]:
                                collaborators.append(c['url'])

                    data_to_add_map = {
                        # mutual connections
                        "mutual_connections_url": highlights,
                        # Companies
                        "experience": [d["url"] for d in data["experience"] if d["url"]],
                        "education": [d["url"] for d in data["education"] if d["url"]],
                        "certifications": [d["url"] for d in data["certifications"] if d["url"]],
                        "volunteering": [d["url"] for d in data["volunteering"] if d["url"]],

                        # People
                        "received_recommendations": [d["url"] for d in data["recommendations"]["received"] if d["url"]],
                        "given_recommendations": [d["url"] for d in data["recommendations"]["given"] if d["url"]],

                        "publications_collaborators": collaborators,
                        "patents_collaborators": collaborators,
                        "projects_collaborators": collaborators,

                        "people_also_viewed": [d["url"] for d in data["people_also_viewed"] if d["url"]]

                    }
                elif self.source == "my_followers":
                    data_to_add_map = {
                        "my_followers": [d["url"] for d in data["my_followers"] if d["url"]]
                    }
                elif self.source == "my_connections":
                    data_to_add_map = {
                        "my_connections": [d["url"] for d in data["my_connections"] if d["url"]]
                    }
                elif self.source == "connections_in_org":
                    data_to_add_map = {
                        "connections_in_org": [d["url"] for d in data["connections_in_org"] if d["url"]],
                        "mutual_connections_url": [d["mutual_connections_url"] for d in data["connections_in_org"] if
                                                   d["mutual_connections_url"]],
                        # "connections_in_org": mt.flatten_list(
                        #     [d["url"] for d in data["connections_in_org"] if d["url"]])
                    }
                elif self.source == "mutual_connections":
                    data_to_add_map = {
                        "mutual_connections": [d["url"] for d in data["mutual_connections"] if d["url"]],
                        "mutual_connections_url": [d["mutual_connections_url"] for d in data["mutual_connections"] if
                                                   d["mutual_connections_url"]],
                        # "mutual_connections": mt.flatten_list(
                        #     [d["mutual_connections"] for d in data["mutual_connections"] if d["mutual_connections"]])
                    }
                elif self.source == "group_members":
                    data_to_add_map = {
                        "group_members": [d["url"] for d in data["group_members"] if d["url"]]
                    }
                elif self.source == "jobs_list":
                    data_to_add_map = {
                        "jobs": [d["url_job"] for d in data["jobs_list"] if d["url_job"]],
                        "companies": [d["url_company"] for d in data["jobs_list"] if d["url_company"]],
                        "connections_in_org": [d["connections_in_org"] for d in data["jobs_list"] if d["connections_in_org"]]
                    }
                elif self.source == "jobs":
                    data_to_add_map = {
                        "jobs_poster": [data["jobs_poster"]["url"]],
                        "companies": [data["info"]["url_company"]],
                        "similar_jobs": [d["url"] for d in data["similar_jobs"] if d["url"]]
                    }


                logger_list.debug("from {} {} {}:".format(self.source, self.source_folder, data["init"]["url"]))
                logger_list.debug("adding {}: {} ".format(self.data_to_add,
                                                          data_to_add_map[self.data_to_add]))

                if data_to_add_map[self.data_to_add]:
                    for j, d in enumerate(data_to_add_map[self.data_to_add]):
                        self.target_list_df = self.target_list_df.append({
                            "status": "hold",
                            "priority": 1,
                            "link": d,
                            "file_name_source": file_name,
                            "source_description": "[from]{}:{},".format(self.source, data["init"]["url"]) + str(
                                self.source_list_df.iloc[i, self.source_list_df.columns.get_loc('source_description')]),
                            "tag": self.source_list_df.iloc[i, self.source_list_df.columns.get_loc('tag')],
                            "info": self.source_list_df.iloc[i, self.source_list_df.columns.get_loc('info')]
                        }, ignore_index=True)

                added.append(self.data_to_add)
                data["extra"]["added"] = added
                self.source_list_df.at[i, 'added'] = added
                # fix to add in added the data that has been added
                if self.source_list_path == self.target_list_path:
                    logger_list.debug("Checking" +str(added))
                    self.target_list_df.at[i, 'added'] = added
                # Update links_list target
                self.source_list_df.to_csv(self.source_list_path, index=False, sep=";")
                # Update links_list to add to
                self.target_list_df.to_csv(self.target_list_path, index=False, sep=";")
                # Save the data---------
                save_json(data, self.profiles_path, file_name)

        # for i, o in list(df):
        #     if pd.isnull(o):
        #         self.source_list_df.at[i, 'operation'] = []
        #
        #     to_add = pd.eval(self.source_list_df.at[i, 'operation'])
        #     if self.data_to_add not in to_add:
        #         to_add.append(self.data_to_add)
        #
        #     self.source_list_df.at[i, 'operation'] = to_add
        #
        #
        # operation_add_df = completed_df[completed_df.operation.eq("add")]
        # for i, s in list(operation_add_df["situation"].items()):
        #     if s != "added":
        #         # get the name of the file in profiles
        #         id = list_df.iloc[i, list_df.columns.get_loc('id')]
        #         timestamp = list_df.iloc[i, list_df.columns.get_loc('timestamp_start')]
        #         timestamp_obj = datetime.strptime(timestamp, "%d/%m/%y %H:%M:%S")
        #         timestamp = datetime.strftime(timestamp_obj, "%d_%m_%y_(%H_%M_%S)")
        #         file_name = id + "_" + timestamp + ".json"
        #
        #         # open the json file
        #         file = Path(self.profiles_path, file_name)
        #         with file.open() as fp:
        #             data = json.load(fp)
        #
        #         if self.target == "companies" or self.target == "schools":
        #             logger_list.debug(
        #                 "adding {} : {}".format(data["top_card_info"]["url_employees"], data["init"]["url"]))
        #             list_to_add_to_df = list_to_add_to_df.append({
        #                 "status": "hold",
        #                 "priority": 1,
        #                 "link": data["top_card_info"]["url_employees"],
        #                 "file_name_source": file_name,
        #                 "source_description": "[from]{}:{},".format(self.target, list_df.iloc[
        #                     i, list_df.columns.get_loc('link')]) + str(
        #                     list_df.iloc[i, list_df.columns.get_loc('source_description')]),
        #                 "tag": list_df.iloc[i, list_df.columns.get_loc('tag')],
        #                 "info": list_df.iloc[i, list_df.columns.get_loc('info')]
        #
        #             }, ignore_index=True)
        #             data["extra"]["operation"] = "add"
        #             data["extra"]["situation"] = "added"
        #             list_df.iloc[i, list_df.columns.get_loc('situation')] = "added"
        #             # Update links_list target
        #             list_df.to_csv(self.list_path, index=False, sep=";")
        #             # Update links_list to add to
        #             list_to_add_to_df.to_csv(self.list_to_add_to_path, index=False, sep=";")
        #             # Save the data---------
        #             save_json(data,
        #                       self.profiles_path,
        #                       file_name)
        #
        #         elif self.target == "jobs_list":
        #
        #             for d in data[self.target]:
        #                 if d["url_job"] and ("/jobs/" in d["url_job"]):
        #                     logger_list.debug("adding {} ".format(d["title"]))
        #                     list_to_add_to_df = list_to_add_to_df.append({
        #                         "status": "hold",
        #                         "priority": 1,
        #                         "link": d["url_job"],
        #                         "file_name_source": file_name + "," + str(list_df.iloc[
        #                                                                       i, list_df.columns.get_loc(
        #                                                                           'file_name_source')]),
        #                         "source_description": "[from]{}:{},".format(self.target, list_df.iloc[
        #                             i, list_df.columns.get_loc('link')]) + str(
        #                             list_df.iloc[i, list_df.columns.get_loc('source_description')]),
        #                         "tag": list_df.iloc[i, list_df.columns.get_loc('tag')],
        #                         "info": None
        #                     }, ignore_index=True)
        #
        #             data["extra"]["operation"] = "add"
        #             data["extra"]["situation"] = "added"
        #             list_df.iloc[i, list_df.columns.get_loc('situation')] = "added"
        #             # Update links_list target
        #             list_df.to_csv(self.list_path, index=False, sep=";")
        #             # Update links_list to add to
        #             list_to_add_to_df.to_csv(self.list_to_add_to_path, index=False, sep=";")
        #             # Save the data---------
        #             save_json(data,
        #                       self.profiles_path,
        #                       file_name)
        #
        #         elif self.target == "jobs":
        #             source = data["init"]["url"]
        #
        #             for d in data["similar_jobs"]:
        #                 if d["url"] and ("/jobs/" in d["url"]):
        #                     logger_list.debug("adding {} ".format(d["title"]))
        #                     list_to_add_to_df = list_to_add_to_df.append({
        #                         "status": "hold",
        #                         "priority": 1,
        #                         "link": d["url"],
        #                         "file_name_source": file_name + "," + list_df.iloc[
        #                             i, list_df.columns.get_loc('file_name_source')],
        #                         "source_description": "[from]{}:{},".format(self.target, list_df.iloc[
        #                             i, list_df.columns.get_loc('link')]) + str(
        #                             list_df.iloc[i, list_df.columns.get_loc('source_description')])
        #
        #                     }, ignore_index=True)
        #
        #             data["extra"]["name"] = source
        #             data["extra"]["operation"] = "add"
        #             data["extra"]["situation"] = "added"
        #             list_df.iloc[i, list_df.columns.get_loc('situation')] = "added"
        #             # Update links_list target
        #             list_df.to_csv(self.list_path, index=False, sep=";")
        #             # Update links_list to add to
        #             list_to_add_to_df.to_csv(self.list_to_add_to_path, index=False, sep=";")
        #             # Save the data---------
        #             save_json(data,
        #                       self.profiles_path,
        #                       file_name)
        #         else:
        #
        #             source = "[from]{}:{},".format(self.target, list_df.iloc[
        #                 i, list_df.columns.get_loc('link')]) + str(
        #                 list_df.iloc[i, list_df.columns.get_loc('source_description')])
        #
        #             for d in data[self.target]:
        #                 if d["url"] and ("/in/" in d["url"]):
        #                     logger_list.debug("adding {} : {}".format(d["full_name"], d["url"]))
        #                     list_to_add_to_df = list_to_add_to_df.append({
        #                         "status": "hold",
        #                         "priority": 1,
        #                         "link": d["url"],
        #                         "file_name_source": file_name + "," + str(list_df.iloc[
        #                                                                       i, list_df.columns.get_loc(
        #                                                                           'file_name_source')]),
        #                         "source_description": "[from]{}:{},".format(self.target, list_df.iloc[
        #                             i, list_df.columns.get_loc('link')]) + str(
        #                             list_df.iloc[i, list_df.columns.get_loc('source_description')]),
        #                         "tag": list_df.iloc[i, list_df.columns.get_loc('tag')],
        #                         "info": None
        #                     }, ignore_index=True)
        #
        #             data["extra"]["name"] = source
        #             data["extra"]["operation"] = "add"
        #             data["extra"]["situation"] = "added"
        #             list_df.iloc[i, list_df.columns.get_loc('situation')] = "added"
        #             # Update links_list target
        #             list_df.to_csv(self.list_path, index=False, sep=";")
        #             # Update links_list to add to
        #             list_to_add_to_df.to_csv(self.list_to_add_to_path, index=False, sep=";")
        #             # Save the data---------
        #             save_json(data,
        #                       self.profiles_path,
        #                       file_name)
