"""
Title     : Scheduler
Objective : Init configuration, validate the list of links and schedule jobs
Created by: Abdellah Janid
Created on: 5/12/20
"""

# Standard library imports
import json
import logging
import random
from datetime import datetime
from pathlib import Path
from urllib.parse import urljoin, urlparse
# Related third party imports
import pandas as pd
import numpy as np
# Import local applications
import scraper.exceptions as exceptions
from scraper import init_configuration
from scraper import scraping_tools as mt
# -----------------------------------------------------------------
from scraper.linkedin.connections_in_org.connections_in_org import ConnectionsInOrg
from scraper.linkedin.connections_in_org.schemas import ConnectionsInOrgSchema
# -----------------------------------------------------------------
from scraper.linkedin.mutual_connections.mutual_connections import MutualConnections
from scraper.linkedin.mutual_connections.schemas import MutualConnectionsSchema
# -----------------------------------------------------------------
from scraper.linkedin.jobs.jobs import Jobs
from scraper.linkedin.jobs.schemas import JobsSchema
# -----------------------------------------------------------------
from scraper.linkedin.jobs_list.jobs_list import JobsList
from scraper.linkedin.jobs_list.schemas import JobsListSchema
# -----------------------------------------------------------------
from scraper.linkedin.schools.schools import Schools
from scraper.linkedin.schools.schemas import SchoolsSchema
# -----------------------------------------------------------------
from scraper.linkedin.companies.companies import Companies
from scraper.linkedin.companies.schemas import CompaniesSchema
# -----------------------------------------------------------------
from scraper.linkedin.employees.employees import Employees
from scraper.linkedin.employees.schemas import EmployeesSchema
# -----------------------------------------------------------------
from scraper.linkedin.group_members.group_members import GroupMembers
from scraper.linkedin.group_members.schemas import GroupMembersSchema
# -----------------------------------------------------------------
from scraper.linkedin.my_connections import MyConnections
from scraper.linkedin.my_connections.schemas import LiMyConnectionsSchema
# -----------------------------------------------------------------
from scraper.linkedin.my_followers.followers import MyFollowers
from scraper.linkedin.my_followers.schemas import LiMyFollowersSchema
# -----------------------------------------------------------------
from scraper.linkedin.people.people import People
from scraper.linkedin.people.schemas import LiPersonAllModel
# -----------------------------------------------------------------
from scraper.scraping_tools import save_json
from scraper.validation_tools import validate_schema

logger_go_scraping = logging.getLogger('go_scraping')
# Set level; change to DEBUG to log debug messages to file
logger_go_scraping.setLevel(logging.DEBUG)


class GoScraping(object):
    """Go scraping data class"""

    def __init__(self,
                 target,
                 folder
                 ):
        """ Contains methods to run the scraper and manage the list of links to scrape and run tests
        Parameters
        ----------
        target : str
            target to scrape: ['people','followers','following','connections','companies','employees','groups']
        folder : str
            path where the data of the session will be saved.
            "watchdog" : is the folder meant for saving data validation, routine check and tests
            "data : is the main folder where the data is saved
        """
        self.config = init_configuration(target)
        self.target = target
        self.folder = folder
        self.wd_path = None  # working directory path
        self.list_path = None
        self.command = None  # command to stop jbo with ctrl+C through KeyboardInterrupt

    def set_paths(self):
        """ Set paths
        """
        logger_go_scraping.debug("setting paths for scraping")
        self.wd_path = Path(Path.home(), "brobygger_li")
        self.list_path = Path(self.wd_path,
                              self.target,
                              self.folder,
                              "links_list",
                              self.config["links_list"][self.target])
        logger_go_scraping.debug("in :" + str(self.list_path))

    def load_list(self, list_path):
        """Load and validate the list

        Parameters
        ----------
        list_path : str
            Path to the links list to load
        Returns
        -------
        list_df : pandas.core.frame.DataFrame
            The list with links list
        Raises
        ------
        LinksListError
            Invalid list, missing columns
        """
        logger_go_scraping.info("loading the list: " + str(list_path))
        # Load the list
        list_df = pd.read_csv(list_path, sep=";")  # import list
        # strip the list from white spaces, to avoid error
        list_df = list_df.rename(columns=lambda x: x.strip())

        # Validate the list
        list_validations_rules = ["status", "priority", "link", "id", "info", "file_name_source", "source_description",
                                  "timestamp_start",
                                  "timestamp_end", "command", "error_msg", "tag", "added", "select"]
        if all(name in list(list_df.columns) for name in list_validations_rules):
            logger_go_scraping.info("list validated")
        else:
            logger_go_scraping.critical(
                "Invalid list, missing columns: " + str(set(list_validations_rules).difference(list(list_df.columns))))
            raise exceptions.LinksListError("Invalid list, missing columns:",
                                            set(list_validations_rules).difference(list(list_df.columns)))

        # Drop duplicates
        logger_go_scraping.info("Dropping duplicated , " + str(sum(list_df.duplicated())))
        list_df = list_df.drop_duplicates()

        # If automatic queuing is set select random links and set them to queued
        if self.config["hold_to_queued"]:
            tot_hold = sum(list_df.status.eq("hold"))
            if tot_hold < 13:
                to_queued = list_df[list_df.status.eq("hold")].index
                list_df.loc[to_queued, "status"] = 'queued'
            else:
                # A random number between 13 and 30
                nr_to_queue = random.randint(13, -max(0, 30 - tot_hold) + 30)
                # Select random rows and set to queued
                to_queued = list_df[list_df.status.eq("hold")].sample(nr_to_queue).index
                list_df.loc[to_queued, "status"] = 'queued'

        list_df = list_df.sort_values(by=["status", "priority"], ignore_index=True)
        list_df.to_csv(list_path, index=False, sep=";")
        return list_df

    def scraping(self):
        """ Execute scraping jobs
        """

        # Load the list
        self.set_paths()
        list_df = self.load_list(self.list_path)
        # Create report with frequency table of session results
        if self.target == "my_followers":
            list_df = list_df.append({
                "status": "queued",
                "priority": 1,
                "link": "https://www.linkedin.com/feed/followers/"
            }, ignore_index=True)
        if self.target == "my_connections":
            list_df = list_df.append({
                "status": "queued",
                "priority": 1,
                "link": "https://www.linkedin.com/mynetwork/invite-connect/connections/"
            }, ignore_index=True)
        start_report_df = pd.crosstab(index=list_df["status"], columns="status_counter")
        queued_check = list_df["status"].str.contains("queued").any()
        if not queued_check:
            logger_go_scraping.info("No queued links in the list")
            raise ValueError("No queued links in the list")

        logger_go_scraping.info("Start report: " + str(start_report_df.to_dict()))
        tot_queued = start_report_df.at["queued", "status_counter"]
        first_index = (list_df.status.eq("queued")).idxmax()

        # reset counter
        status_counter = {"failed": 0,
                          "cancelled": 0,
                          "duplicate": 0,
                          "completed": 0
                          }
        # reset the go home counter
        go_home_counter = 0
        # rest watchdog check
        watchdog_check = 0

        # mapping the right class for scraping
        target_fun = {
            "people": People,
            "my_followers": MyFollowers,
            "my_connections": MyConnections,
            "employees": Employees,
            "group_members": GroupMembers,
            "companies": Companies,
            "schools": Schools,
            "jobs_list": JobsList,
            "jobs": Jobs,
            "mutual_connections": MutualConnections,
            "connections_in_org": ConnectionsInOrg
        }
        validation_model = {
            "people": LiPersonAllModel,
            "my_followers": LiMyFollowersSchema,
            "my_connections": LiMyConnectionsSchema,
            "employees": EmployeesSchema,
            "group_members": GroupMembersSchema,
            "companies": CompaniesSchema,
            "schools": SchoolsSchema,
            "jobs_list": JobsListSchema,
            "jobs": JobsSchema,
            "mutual_connections": MutualConnectionsSchema,
            "connections_in_org": ConnectionsInOrgSchema
        }
        # -----------------------------
        # Start
        # -----------------------------
        # initiate the scraper
        scraping_instance = target_fun[self.target](config=self.config, target=self.target, folder=self.folder)
        scraping_instance.setup(login_settings=scraping_instance.config["login_settings"],
                              html_name="setup",
                              html_path=scraping_instance.html_path,
                              screenshot_name="setup",
                              screenshot_path=scraping_instance.screenshots_path
                              )

        for i, l in list(list_df[list_df.status.eq("queued")]["link"].items()):
            logger_go_scraping.info("#******************************************************************************#")
            logger_go_scraping.info("#******************************************************************************#")
            try:
                # Validate link
                if self.target in ["people", "companies", "schools"]:
                    l = mt.validate_link(l)
                    list_df.iloc[i, list_df.columns.get_loc('link')] = l

                # scrape profile
                # If link is in the completed list, set to duplicate
                if (l in list_df[list_df.status.eq("completed")]["link"].values) and (
                        self.target not in ["my_followers",
                                            "my_connections"]) and (
                        list_df.iloc[
                            i, list_df.columns.get_loc('priority')] != 0):  # If priority is 0 scrape the link again
                    list_df.iloc[i, list_df.columns.get_loc('status')] = "duplicate"

                    timestamp = datetime.now().strftime("%d/%m/%y %H:%M:%S")
                    list_df.iloc[i, list_df.columns.get_loc('timestamp_start')] = timestamp
                    list_df.iloc[i, list_df.columns.get_loc('timestamp_end')] = timestamp

                    logger_go_scraping.info(
                        "found duplicate , index:" + str(i + 1) + " , ratio :" + str(i + 1 - first_index) + "/" + str(
                            start_report_df.loc["queued", "status_counter"]) + " , " + str(l))

                    # Increment the counter
                    status_counter["duplicate"] += 1
                    # Update list
                    list_df.to_csv(self.list_path, index=False, sep=";")
                else:
                    # noinspection PyBroadException
                    try:
                        logger_go_scraping.info(
                            "scraping , index:" + str(i + 1) + " , ratio :" + str(i + 1 - first_index) + "/" + str(
                                tot_queued) + " , " + str(l))
                        timestamp = datetime.now()
                        scraping_instance.timestamp_start = timestamp.strftime("%d_%m_%y_(%H_%M_%S)")
                        list_df.iloc[i, list_df.columns.get_loc('timestamp_start')] = timestamp.strftime("%d/%m/%y %H:%M:%S")

                        # command ----------
                        if pd.isnull(list_df.iloc[i, list_df.columns.get_loc('command')]):
                            scraping_instance.csv_command = None
                        else:
                            scraping_instance.csv_command = list_df.iloc[i, list_df.columns.get_loc('command')]
                        # source info ----------
                        if pd.isnull(list_df.iloc[i, list_df.columns.get_loc('file_name_source')]):
                            scraping_instance.file_name_source = None
                        else:
                            scraping_instance.file_name_source = list_df.iloc[i, list_df.columns.get_loc('file_name_source')]

                        if pd.isnull(list_df.iloc[i, list_df.columns.get_loc('source_description')]):
                            scraping_instance.source_description = None
                        else:
                            scraping_instance.source_description = list_df.iloc[i, list_df.columns.get_loc('source_description')]

                        if pd.isnull(list_df.iloc[i, list_df.columns.get_loc('info')]):
                            scraping_instance.csv_info = None
                        else:
                            scraping_instance.csv_info = str(list_df.iloc[i, list_df.columns.get_loc('info')])

                        if pd.isnull(list_df.iloc[i, list_df.columns.get_loc('tag')]):
                            scraping_instance.tag = None
                        else:
                            scraping_instance.tag = list_df.iloc[i, list_df.columns.get_loc('tag')]
                        # Get the data----------

                        scraping_instance.url = l
                        scraping_instance.scrape()
                        # ----------------------
                        # Validate the data-----
                        timestamp = datetime.now()
                        scraping_instance.timestamp_end = datetime.now().strftime("%d_%m_%y_(%H_%M_%S)")
                        scraping_instance.data["init"]["timestamp_end"] = scraping_instance.timestamp_end
                        data = validate_schema(validation_model[self.target],
                                               scraping_instance.data,
                                               scraping_instance.failed_json_path, scraping_instance.file_name_ts)
                        # ----------------------
                        # Save the data---------
                        save_json(json.loads(data.json(by_alias=True)),
                                  scraping_instance.profiles_path,
                                  scraping_instance.file_name_ts)
                        # Save the meta_data---------

                        meta_data = {}
                        meta_data.update({"init":scraping_instance.data["init"]})
                        self.extra = {
                            "name": None,
                            "len": None,
                            "tag": None,
                            "to_add": None,
                            "added": None,
                            "other": None
                        }
                        meta_data.update({"extra": self.extra})
                        meta_data.update({"meta_data_container": scraping_instance.meta_data_container})
                        save_json(meta_data,
                                  scraping_instance.meta_data_path,
                                  scraping_instance.file_name_ts)
                        # ---------------------
                        # Update the list-------
                        list_df.iloc[i, list_df.columns.get_loc('status')] = "completed"
                        list_df.iloc[i, list_df.columns.get_loc('timestamp_end')] = timestamp.strftime("%d/%m/%y %H:%M:%S")
                        list_df.iloc[i, list_df.columns.get_loc('id')] = scraping_instance.init["id"]
                        if scraping_instance.extra_info:
                            list_df.iloc[i, list_df.columns.get_loc('info')] = scraping_instance.extra_info
                        # ----------------------

                        status_counter["completed"] += 1
                        # Update list
                        list_df.to_csv(self.list_path, index=False, sep=";")

                        # increment the counter to go home after a certain number of profiles
                        go_home_counter += 1
                        # increment the watchdog_check
                        watchdog_check += 1
                        # for security reset data containers
                        scraping_instance.reset_data_containers()

                        if 4 <= go_home_counter <= 6:
                            go_home = bool(random.getrandbits(1))
                            if go_home:
                                logger_go_scraping.debug(
                                    "tot url mined {}, go home after {}".format(status_counter["completed"],
                                                                                go_home_counter))
                                scraping_instance.go_home()
                                go_home_counter = 0
                        if go_home_counter > 7:
                            logger_go_scraping.info(
                                "Force go home , urls mined {}, go home after {}".format(
                                    status_counter["completed"], go_home_counter))
                            scraping_instance.go_home()
                            go_home_counter = 0

                    except Exception as e:
                        logger_go_scraping.exception("Exception on main holder: ")
                        list_df.iloc[i, list_df.columns.get_loc('status')] = "failed"
                        timestamp = datetime.now().strftime("%d/%m/%y %H:%M:%S")
                        list_df.iloc[i, list_df.columns.get_loc('timestamp_end')] = timestamp
                        list_df.iloc[i, list_df.columns.get_loc('error_msg')] = e

                        status_counter["failed"] += 1
                        scraping_instance.reset_data_containers()
                        list_df.to_csv(self.list_path, index=False, sep=";")
                        # If failed more than 3 stop and close session
                        if status_counter["failed"] > self.config["failed_limit"]:
                            logger_go_scraping.info("failed more than {} times, finishing the scraping".format(self.config["failed_limit"]))
                            break

            except KeyboardInterrupt:
                try:
                    # TODO: stop selenium without quiting the browser
                    print('\nPausing...')
                    print("#---------------------------------------------------------------------")
                    print("Enter one of the following commands:")
                    print("#---------------------------------------------------------------------")
                    print('quit    : to stop the scraping and exit')
                    print("#---------------------------------------------------------------------")
                    while True:
                        self.command = input("Insert command: ")
                        if self.command in ["quit"]:
                            break
                    if self.command == "quit":
                        break
                except KeyboardInterrupt:
                    print('bye...')
                    break
        # -----------------------------
        # Close
        # -----------------------------
        end_report_df = pd.crosstab(index=list_df["status"], columns="count")
        logger_go_scraping.info("End report session: " + str(status_counter))
        logger_go_scraping.info("End report list status: " + str(end_report_df.to_dict()))

        # close the logger
        logging.shutdown()
        # Close the browser
        scraping_instance.driver.quit()
