"""
Title     : scraping tools
Objective : scraping tools to be used by all spyder
Created by: Abdellah Janid
"""

# Standard library imports
import json
import logging
import random
import time
from datetime import datetime
from pathlib import Path
from urllib.parse import urljoin, urlparse
# Related third party imports
# Import local applications

# Set level; change to DEBUG to log debug messages to file
logger_scraping_tools = logging.getLogger('scraping_tools')
logger_scraping_tools.setLevel(logging.DEBUG)


def get_attribute_value(element, selector, attribute, default=None):
    """Get the attribute value from a beautifulsoup element

    Parameters
    ----------
    element : bs4.BeautifulSoup
        bs4 object
    selector : str
        CSS selector
    attribute : str
        attribute's value to get
    default: Any
        default value if the selector is not found

    Returns
    -------
    attribute_value : str or default
        attribute value or default
    """
    # noinspection PyBroadException
    try:
        return element.select_one(selector)[attribute]
    except:
        return default


def get_info(element, mapping, default=None):
    """Turn bs4 element and key->selector dict into a key->value dict

    Parameters
    ----------
    element : bs4.BeautifulSoup
        bs4 object
    mapping : dict
        a dictionary with key selector mapping
    default: Any
        default value if the selector is not found

    Returns
    -------
    info : dict
        a dictionary with "key":"value"

    """
    info = {key: get_text(element, mapping[key], default=default) for key in mapping}

    if "description" in info.keys():
        if info["description"]:
            info["description"] = strip_string_from_end(info["description"], 'see less').strip()

    if "recommendation" in info.keys():
        if info["recommendation"]:
            info["recommendation"] = strip_string_from_end(info["recommendation"], 'See less').strip()
            info["recommendation"] = strip_string_from_end(info["recommendation"], '... See more').strip()

    return info


def strip_string_from_end(string_to_strip, suffix):
    """
    Parameters
    ----------
    string_to_strip : str
        string to strip
    suffix : str
        the end of the string to strip

    Returns
    -------
    string_to_strip : str
        string stripped if the condition is satisfied
    """
    if suffix and string_to_strip.endswith(suffix):
        return string_to_strip[:-len(suffix)]
    return string_to_strip


def get_text(element, selector, default=None):
    """Get text from a bs4 element given the selector

    Parameters
    ----------
    element : BeautifulSoup
        bs4 object
    selector : str
        CSS selector
    default: Any
        default value if the selector is not found

    Returns
    -------
    text : str or default
        Text under the CSS selector
    """
    # noinspection PyBroadException
    try:
        return element.select_one(selector).get_text().strip()
    except:
        return default


def select_all(soup, selector, default=None):
    """Get all matching elements for a css selector within an element
    Parameters
    ----------
    soup : bs4.BeautifulSoup
        bs4 object
    selector : str
        CSS selector
    default: Any
        default value if the selector is not found
    """

    if default is None:
        default = []
    # noinspection PyBroadException
    try:
        elements = soup.select(selector)
        if len(elements) == 0:
            return default
        return soup.select(selector)
    except:
        return default


def flatten_list(lists):
    """Flattens a nested list

    Parameters
    ----------
    lists : list
        list to flatten
    Returns
    -------
    list : list
    """
    return [item for sublist in lists for item in sublist]


def save_json(file, path, name, add_timestamp=None, **kwargs):
    """save a json file
     Parameters
     ----------
    file : dict

    path : str
        path where to save
    name : str
        name
    kwargs : dict
        args for json.dump
    """
    if add_timestamp:
        if name[-5::] == ".json":
            name = name[:-5] + add_timestamp + ".json"
        else:
            name += add_timestamp + ".json"
    else:
        if name[-5::] == ".json":
            name = name[:-5] + ".json"
        else:
            name += ".json"

    file_path = Path(path, name)
    logger_scraping_tools.info("JSON saved in: " + str(file_path))
    with file_path.open("w", encoding='utf-8') as fp:
        json.dump(file, fp, **kwargs)


def save_html(file, path, name):
    """save a json file
     Parameters
     ----------
    file : dict

    path : str
        path where to save
    name : str
        name
     """
    timestamp = datetime.now().strftime("_%d_%m_%y_(%H_%M_%S)")
    if name[-5::] == ".html":
        name = name[:-5] + timestamp + ".html"
    else:
        name += (timestamp + ".html")

    file_path = Path(path, name)
    logger_scraping_tools.info("HTML saved in: " + str(file_path))
    with file_path.open('w', encoding='utf-8') as fp:
        fp.write(str(file))


def random_rest(min_seconds, max_seconds):
    """sleep a random amount of time
    Parameters
    ----------
    min_seconds : float
        minimum seconds to go sleeping
    max_seconds : float
        max seconds
    """
    # sleep 2 seconds because it takes very less time
    # so that you can see the actual difference
    sleeping = random.uniform(min_seconds, max_seconds)
    logger_scraping_tools.debug("Sleeping for: " + str(sleeping))
    time.sleep(sleeping)
    return sleeping


def desired_dict(string, d_list):
    """
    Returns a dictionary from the list of string
    Parameters
    ----------
    string : str
        string to look for
    d_list : list
        list of dictionaries

    Returns
    -------
    data : dict

    """
    for d in d_list:
        if string in d:
            logger_scraping_tools.debug("found key:" + string)
            # noinspection PyBroadException
            try:
                meta = json.loads(d)
            except:
                meta = None
            return meta


def scrub(obj, bad_key="_this_is_bad"):
    """
    Remove a given key from the dictionary
    Parameters
    ----------
    obj : dict
        dictionary to clean
    bad_key : str
        key to remove from the dictionary

    Returns
    -------

    """
    if isinstance(obj, dict):
        for key in list(obj.keys()):
            if key == bad_key:
                del obj[key]
            else:
                scrub(obj[key], bad_key)
    elif isinstance(obj, list):
        for i in reversed(range(len(obj))):
            if obj[i] == bad_key:
                del obj[i]
            else:
                scrub(obj[i], bad_key)
    else:
        # neither a dict nor a list, do nothing
        pass


def find_keys(node, kv):
    """
    return a list of values
    Parameters
    ----------
    node : Any
        object to search
    kv : str
        key name

    Returns
    -------
    value of the key desired
    """
    if isinstance(node, list):
        for i in node:
            for x in find_keys(i, kv):
                yield x
    elif isinstance(node, dict):
        if kv in node:
            yield node[kv]
        for j in node.values():
            for x in find_keys(j, kv):
                yield x


def convert_timestamp(timestamp):
    if "_" in timestamp:
        timestamp_obj = datetime.strptime(timestamp, "%d_%m_%y_(%H_%M_%S)")
        timestamp = datetime.strftime(timestamp_obj, "%d/%m/%y %H:%M:%S")

    else:
        timestamp_obj = datetime.strptime(timestamp, "%d/%m/%y %H:%M:%S")
        timestamp = datetime.strftime(timestamp_obj, "%d_%m_%y_(%H_%M_%S)")
    return timestamp

def validate_link(link):
    # validate link
    # lst = ["/in/", "/company/", "/school/"]
    # if not any(s in link for s in lst):
    #     raise ValueError("link not valid")
    # deconstruct
    path = urlparse(link).path
    start, end = 0, 3
    url = None
    # construct
    if '/in/' in link:
        url = "https://www.linkedin.com" + '/'.join(path.split('/')[start:end]) + '/'
    elif any(s in link for s in ["/company/", "/school/"]):
        url = "https://www.linkedin.com"+ '/'.join(path.split('/')[start:end]) + '/about/'
    return url

def validate_link_one(link):
    # construct
    url = None
    if '/in/' in link:
        path = urlparse(link).path
        start, end = 0, 3
        url = "https://www.linkedin.com" + '/'.join(path.split('/')[start:end]) + '/'
    return url


