"""
Title     : Scraper Session
Objective : Initialize a session and provide general tools for web scraping
Created by: Janid
Created on: 08/05/20

Notes:
    - I didn't use **kwargs in some functions to pass them to other functions, in order to have documentation every time
    the function is used.
    copying functions signature for better introspection is still an open issue in PEP
    Ref -->  https://github.com/python/typing/issues/270
    - TODO: when the parameters and functions needed are clear, remap functions signature and docstrings

"""

# Standard library imports
import hashlib
import json
import logging
import re
import time
from datetime import datetime
from pathlib import Path
# Related third party imports
from typing import List

import bs4
# noinspection PyPackageRequirements
from Screenshot import Screenshot_Clipping
from selenium import webdriver
from selenium.common.exceptions import TimeoutException, NoSuchElementException
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.ui import WebDriverWait

# Import local applications
import scraper.scraping_tools as mt
from scraper import exceptions, __about__

# Create a module logger
logger_scraper_session = logging.getLogger('scraper_session')
# Set level; change to DEBUG to log debug messages to file
logger_scraper_session.setLevel(logging.DEBUG)


class ScraperSession(object):
    """Scraper Session"""

    def __init__(self,
                 config, target, folder
                 ):
        """ Class object of type ScraperSession contains tools to interact with a browser using selenium and extract
         bs4 elements
        Attributes
        ----------
        config: dict
            scraper session configuration
        target : str
            target to scrape: ['people','followers','following','connections','companies','employees','groups']
        folder : str
            path where the data of the session will be saved.
            "watchdog" : is the folder meant for saving data validation, routine check and tests
            "data : is the main folder where the data is saved

        """
        logger_scraper_session.debug("Init ScraperSession")
        # Load the configuration
        self.config = config
        self.target = target
        self.folder = folder

        # Test set it to watchdog
        # Init scraping containers
        self.url = None
        self.file_name = None  # file name to be used to save html, screenshots, media, json
        self.file_name_ts = None  # file name to be used to save html, screenshots, media, json
        self.timestamp_start = None
        self.timestamp_end = None
        self.file_name_source = None
        self.source_description = None
        self.tag = None
        self.csv_info = None
        self.csv_command = None
        # define paths
        self.failed_json_path = None  # path to save failed validation
        self.profiles_path = None
        self.html_path = None
        self.html_path_profile = None
        self.media_path = None
        self.media_path_profile = None
        self.screenshots_path = None
        self.screenshots_path_profile = None
        # set the path to data
        self.set_data_path(self.target, self.folder)

        # General data container
        self.full_page = None
        self.scraping_status = None
        self.data = {}
        self.init = {}
        self.meta_data = None
        self.meta_data_container = {}

        # A general container for all sections
        self.info = []

        # Linkedin container
        self.top_card_info = {}
        self.contact_info = {}
        self.summary = None
        self.highlights = {}
        self.experience = []
        self.education = []
        self.lic_and_cert = []
        self.volunteering = []
        self.skills = []
        self.recommendations = {
            "received": [],
            "given": []
        }
        self.accomplishments = {
            "publications": [],  # Done
            "patents": [],  # Done
            "courses": [],  # Done
            "projects": [],  # Done
            "honors": [],  # Done
            "languages": [],  # Done
            "organizations": [],  # Done
            "test-scores": []  # Done
        }

        self.interests = {
            "popup": None,  #
            "interests_list": [],  #
            "interests_list_no_popup": [],  #
            "influencers": [],  #
            "companies": [],  #
            "groups": [],  #
            "schools": []  #
        }
        self.people_also_viewed = []

        # For employees package
        self.employees = []
        # Group members
        self.extra = {}
        self.group_members = []
        # extra info, extra info to put in the links_list under info column
        self.extra_info = None

        # Companies
        self.groupedLocations = None
        self.similar_companies = []
        # Connections in organization
        self.connections_in_org = []

        # Jobs
        self.jobs_list = []

        # Job
        self.jobs_poster = {}
        self.description_details = {}
        self.seniority_level = []
        self.location = []
        self.similar_jobs = []

        # Learn
        self.quiz = {}
        # Apply all options anti bot options
        options = webdriver.ChromeOptions()
        options.add_argument('--no-sandbox')
        options.add_argument('--disable-notifications')
        options.add_argument('--profile-directory=Default')
        options.add_argument("--disable-blink-features=AutomationControlled")
        options.add_argument("--start-maximized")
        options.add_argument("user-data-dir={}/browser".format(Path(Path.home(), "brobygger_li")))
        options.add_experimental_option("excludeSwitches", ["enable-automation"])
        options.add_experimental_option('useAutomationExtension', False)
        options.add_experimental_option('prefs', {
            'credentials_enable_service': False,
            'profile': {
                'password_manager_enabled': False
            }
        })
        self.driver = webdriver.Chrome(options=options)
        logger_scraper_session.debug("Initiated webdriver")

    def reset_data_containers(self):
        """
        Reset data containers
        """
        # General data container
        self.full_page = None
        self.scraping_status = None
        self.data = {}
        self.init = {}

        # A general container for all sections
        self.info = []

        # Linkedin container
        self.top_card_info = {}
        self.contact_info = {}
        self.summary = None
        self.highlights = {}
        self.experience = []
        self.education = []
        self.lic_and_cert = []
        self.volunteering = []
        self.skills = []
        self.recommendations = {
            "received": [],
            "given": []
        }
        self.accomplishments = {
            "publications": [],  # Done
            "patents": [],  # Done
            "courses": [],  # Done
            "projects": [],  # Done
            "honors": [],  # Done
            "languages": [],  # Done
            "organizations": [],  # Done
            "test-scores": []  # Done
        }

        self.interests = {
            "popup": None,  #
            "interests_list": [],  #
            "interests_list_no_popup": [],  #
            "influencers": [],  #
            "companies": [],  #
            "groups": [],  #
            "schools": []  #
        }
        self.people_also_viewed = []
        # For employees package
        self.employees = []
        # Group members
        self.extra = {}
        self.group_members = []
        # extra info, extra info to put in the links_list under info column
        self.extra_info = None

        # Companies
        self.meta_data = None
        self.groupedLocations = None

        # Jobs
        self.jobs_list = []

        # Job
        self.jobs_poster = {}
        self.description_details = {}
        self.seniority_level = []
        self.location = []
        self.similar_jobs = []

    def select_element(self,
                       method,
                       selector,
                       web_element=None,
                       return_type="sel_el",
                       timeout=5,
                       default=None,
                       html=True,
                       html_name="",
                       html_path="",
                       screenshot=True,
                       screenshot_name="",
                       screenshot_path=""
                       ):
        """Returns a WebElement or bs4 element given the method and selector

        Parameters
        ----------
        method : str
            By selector type selenium.webdriver.common.by.By
        selector : str
            selector to look for
        web_element : WebElement
            selenium WebElement
        return_type : str
            "sel_el" for selenium WebElement or "bs_el" for bs4 element
        timeout: int
            timeout to wait for the element to be selected
        default : Any or None
            default return value
        html : bool
            save html
        html_name: str
            path to save of the html file
        html_path: str
            path to save of the html file
        screenshot : bool
            take screenshot
        screenshot_name: str
            name to save screenshot
        screenshot_path: str
            path to save screenshot
        Returns
        -------
        el : WebElement or bs4.BeautifulSoup or None
            if return_type = "sel_el" it return selenium WebElement otherwise None
            if return_type = "bs_el" it return bs4 otherwise None
        """

        logger_scraper_session.debug("Trying to select element: {} , {}".format(method, selector))
        try:
            if web_element is None:
                web_element = self.driver
            # Wait for the given conditions until element_present to occur before proceeding further with execution
            el = WebDriverWait(web_element, timeout).until(
                expected_conditions.presence_of_element_located((method, selector)))

            logger_scraper_session.debug("element is ready , {} , {}".format(method, selector))

            if screenshot and screenshot_name and screenshot_path:
                logger_scraper_session.debug(
                    "saving screenshot select_element: path = {} , name = {}".format(screenshot_path,
                                                                                     screenshot_name))
                self.save_screenshot(screenshot_path, screenshot_name)

            if return_type == "bs_el":
                el = bs4.BeautifulSoup(el.get_attribute('outerHTML'), 'html.parser')
                if html and html_path and html_name:
                    logger_scraper_session.debug(
                        "saving html from select element: path = {} , name = {}".format(html_path, html_name))
                    mt.save_html(el, html_path, html_name)
                return el
            else:
                return el
        except TimeoutException:
            logger_scraper_session.warning(
                "Timeout , {} , {}, url = {}".format(method, selector, self.driver.current_url))
            self.save_failed_element(html_path, html_name, screenshot_name, screenshot_path)
            return default

    def select_elements(self, method, selector, web_element=None, timeout=5, default=None, return_type="sel_el"):
        """Returns a list of WebElement or bs4 elements given the method and selector

        Parameters
        ----------
        method : str
            By selector type selenium.webdriver.common.by.By
        selector : str
            selector to look for
        web_element : WebElement or WebDriver
            selenium WebElement
        timeout: int
            timeout to wait for the element to be selected
        default : Any or None
            default return value

        return_type : str
            "sel_el" for selenium web element or "bs_el" for bs4 element

        Returns
        -------
        el : list[WebElement] or list[bs4.BeautifulSoup] or None
            if return_type = "sel_el" it returns a list of selenium WebElement otherwise None
            if return_type = "bs_el" it returns a list of bs4 otherwise None

        """
        logger_scraper_session.debug("Trying to select elements")
        try:
            if web_element is None:
                web_element = self.driver
            # Wait for the given conditions until element_present to occur before proceeding further with execution
            sel_el = WebDriverWait(web_element, timeout).until(
                expected_conditions.presence_of_all_elements_located((method, selector)))

            logger_scraper_session.debug(
                "elements are ready , {} , {}, nr elements: {}".format(method, selector, len(sel_el)))

            if return_type == "bs_el":
                logger_scraper_session.debug("getting elements bs4 , {} , {}".format(method, selector))
                return [bs4.BeautifulSoup(i.get_attribute('outerHTML'), 'html.parser') for i in sel_el]
            else:
                return sel_el

        except TimeoutException:
            logger_scraper_session.warning(
                "Timeout , {} , {}, url = {}".format(method, selector, self.driver.current_url))
            return default

    def scroll_to_element(self, method, selector, web_element=None):
        """Tries to find an element and scroll to it to put it into view

        Parameters
        ----------
        method : str
            selector type selenium.webdriver.common.by.By
        selector : str
            value to look for
        web_element: WebElement or None
            use web_element to scroll to a child web element

        Returns
        -------
        element : WebElement or None
            selenium WebElement, None if not found
        """
        logger_scraper_session.debug("scrolling to element, {} , {}".format(method, selector))
        element = None
        try:
            if web_element:
                element = web_element.find_element(method, selector)
            else:
                element = self.driver.find_element(method, selector)

            # Scroll to element
            self.driver.execute_script("return arguments[0].scrollIntoView(true);", element)
            # put element into center view
            self.driver.execute_script("window.scrollBy(0, -(window.innerHeight / 2));")

        except NoSuchElementException:
            logger_scraper_session.debug("element to scroll to not found , {} ,{}".format(method, selector))
            pass

        return element

    def scroll_select(self,
                      method,
                      selector,
                      web_element=None,
                      return_type="sel_el",
                      timeout=5,
                      default=None,
                      html=True,
                      html_name="",
                      html_path="",
                      screenshot=True,
                      screenshot_name="",
                      screenshot_path=""
                      ):
        """Scroll to the element and gets it

        Parameters
        ----------
        method : str
            By selector type selenium.webdriver.common.by.By
        selector : str
            selector to look for
        web_element: WebElement or None
            use web_element to scroll to a child web element
        return_type : str
            "sel_el" for selenium WebElement or "bs_el" for bs4 element
        timeout: int
            timeout to wait for the element to be selected
        default : Any or None
            default return value
        html : bool
            save html
        html_name: str
            path to save of the html file
        html_path: str
            path to save of the html file
        screenshot : bool
            save screenshot
        screenshot_name: str
            name to save screenshot
        screenshot_path: str
            path to save screenshot
        Returns
        -------
        el : WebElement or bs4.BeautifulSoup or None
            if return_type = "sel_el" it return selenium WebElement otherwise None
            if return_type = "bs_el" it return bs4 otherwise None
        """
        el = self.select_element(method,
                                 selector,
                                 web_element,
                                 return_type,
                                 timeout,
                                 default,
                                 html,
                                 html_name,
                                 html_path,
                                 screenshot,
                                 screenshot_name + "_before_scroll",
                                 screenshot_path)
        if el is not None:
            self.scroll_to_element(method, selector, web_element)
            if screenshot and screenshot_name:
                self.save_screenshot(screenshot_path, screenshot_name + "_after_scroll")

        return el

    def scroll_to_bottom(self, scroll_pause_min=1, scroll_pause_max=3, scroll_increment=300):
        """Scroll to the bottom of the page to load lazy elements

        Parameters
        ----------
        scroll_pause_min : float
            pause between scrolls in seconds
        scroll_pause_max : float
            pause between scrolls in seconds
        scroll_increment : int
            scroll increment in seconds
        Returns
        -------
        bool
            returns True when finished scrolling
        """
        logger_scraper_session.debug("session, started, scroll_to_bottom")
        current_height = 0
        while True:
            # Scroll down to bottom
            new_height = self.driver.execute_script(
                "return Math.min({}, document.body.scrollHeight)".format(current_height + scroll_increment))

            if new_height == current_height:
                break
            self.driver.execute_script(
                "window.scrollTo(0, Math.min({}, document.body.scrollHeight));".format(new_height))
            current_height = new_height
            # Wait to load page
            mt.random_rest(scroll_pause_min,
                           scroll_pause_max)
        logger_scraper_session.debug("session, done, scroll_to_bottom")
        return True

    def login(self, login_settings,
              html_name="",
              html_path="",
              screenshot_name="",
              screenshot_path=""
              ):
        """Login to the page

        Parameters
        ----------
        login_settings : `dict`
            login settings dictionary
                login_page : str
                    website url.
                user : str
                    User name for login.
                css_user : str
                    selector User CSS Selector.
                password : str
                    Password.
                css_password : str
                    Password CSS Selector.
                login_check : str
                    A string used as condition to check that the login has been successful.
                css_login_check : str
                    Login Check CSS Selector.
        html_name: str
            path to save of the html file
        html_path: str
            path to save of the html file
        screenshot_name: str
            name to save screenshot
        screenshot_path: str
            path to save screenshot
        Raises
        ------
        UserBoxNotFound
            User box not found
        PasswordBoxNotFound
            Password box not found
        Returns
        -------
        bool
            returns True when login is submitted
        """
        logger_scraper_session.debug("Attempting login:")
        # Login
        user_box = self.select_element(By.CSS_SELECTOR, login_settings["css_user"],
                                       html_name=html_name,
                                       html_path=html_path,
                                       screenshot_name=screenshot_name,
                                       screenshot_path=screenshot_path
                                       )
        if user_box is None:
            raise exceptions.UserBoxNotFound(By.CSS_SELECTOR, login_settings["css_user"], self.driver.current_url)
        user_box.send_keys(login_settings["user"])
        user_box.submit()
        password_box = self.select_element(By.CSS_SELECTOR, login_settings["css_password"],
                                           html_name=html_name,
                                           html_path=html_path,
                                           screenshot_name=screenshot_name,
                                           screenshot_path=screenshot_path
                                           )
        if password_box is None:
            logger_scraper_session.critical("Password box not found")
            raise exceptions.PasswordBoxNotFound(By.CSS_SELECTOR, login_settings["css_password"],
                                                 self.driver.current_url)
        password_box.send_keys(login_settings["password"])
        password_box.submit()
        logger_scraper_session.info("Credentials submitted")
        return True

    def check_login(self, login_settings,
                    timeout=15,
                    html_name="",
                    html_path="",
                    screenshot_name="",
                    screenshot_path=""
                    ):
        """Check that the page is loaded

        Parameters
        ----------
        login_settings : dict
            dictionary containing logins settings
        timeout: int
            timeout to wait for the condition
        html_name: str
            path to save of the html file
        html_path: str
            path to save of the html file
        screenshot_name: str
            name to save screenshot
        screenshot_path: str
            path to save screenshot
        Returns
        -------
        bool
            returns True if logged in is logged in otherwise False
        Raises
        ------
        FailedCheckLogin
            Failed login check
        """
        logger_scraper_session.info("Checking that the profile is logged in")

        # noinspection PyBroadException
        try:
            check = self.select_element(By.CSS_SELECTOR, login_settings["css_login_check"], return_type="bs_el",
                                        timeout=timeout)

            if check:
                if login_settings["login_check_type"] == "attribute":
                    check = mt.get_attribute_value(check, login_settings["css_login_tag"],
                                                   login_settings["css_login_attribute"])
                else:
                    check = check.get_text().strip()
                if check == login_settings["login_check"]:
                    logger_scraper_session.info("logged in checked at 1st attempt: OK")
                    return True
                else:
                    raise exceptions.FailedCheckLogin(By.CSS_SELECTOR, login_settings["css_login_check"],
                                                      self.driver.current_url, login_settings["login_check"])
            else:
                raise exceptions.FailedCheckLogin(By.CSS_SELECTOR, login_settings["css_login_check"],
                                                  self.driver.current_url, login_settings["login_check"])
        except:
            logger_scraper_session.exception("Failed first login check attempt :")
            logger_scraper_session.debug("method : {}, selector : {}, condition : {}".format(
                By.CSS_SELECTOR, login_settings["css_login_check"], login_settings["login_check"]))
            logger_scraper_session.debug("Trying to see if the condition to check login in the source page")
            check_again = self.driver.page_source.__contains__(login_settings["login_check"])
            if check_again:
                logger_scraper_session.debug("condition to check login in found the source page")
                return True
            else:
                if html_path and html_name:
                    logger_scraper_session.debug(
                        "saving html from failed login check error: path = {} , name = {}".format(html_path, html_name))
                    html_name = "error_" + html_name
                    mt.save_html(self.driver.page_source, html_path, html_name)

                if screenshot_name and screenshot_path:
                    logger_scraper_session.warning(
                        "saving screenshot from failed login check error: path = {} , name = {}".format(html_path,
                                                                                                        html_name))
                    screenshot_name = "error_" + screenshot_name
                    self.save_screenshot(path=screenshot_path, name=screenshot_name)

                raise exceptions.FailedCheckLogin(By.CSS_SELECTOR, login_settings["css_login_check"],
                                                  self.driver.current_url, login_settings["login_check"])

    def setup(self, login_settings,
              html_name="",
              html_path="",
              screenshot_name="",
              screenshot_path=""
              ):
        """
        Parameters
        ----------
        login_settings : dict
            login settings
        html_name: str
            path to save of the html file
        html_path: str
            path to save of the html file
        screenshot_name: str
            name to save screenshot
        screenshot_path: str
            path to save screenshot
        Returns
        -------
        bool
            True if setup is successful
        """
        logger_scraper_session.info("Getting login page: " + login_settings["login_page"])
        self.driver.get(login_settings["login_page"])

        # noinspection PyBroadException
        try:
            self.check_login(
                login_settings,
                html_name=html_name,
                html_path=html_path,
                screenshot_name=screenshot_name,
                screenshot_path=screenshot_path
            )
            logger_scraper_session.info("Setup OK without need to login")
            return 1
        except:
            logger_scraper_session.exception("setup exception: ")
            self.login(login_settings,
                       html_name,
                       html_path,
                       screenshot_name,
                       screenshot_path
                       )
            time.sleep(5)
            # attempts = 3
            # while attempts < 3:
            #     try:
            #         logger_scraper_session.info("Attempt check login number: ", attempts)
            self.check_login(
                login_settings,
                html_name=html_name,
                html_path=html_path,
                screenshot_name=screenshot_name,
                screenshot_path=screenshot_path
            )
            # except Exception as e:
            #     logger_scraper_session.exception(e)
        logger_scraper_session.info("Setup OK with need to login")
        return 2

    def save_screenshot(self, path, name):
        """Flattens a nested list

         Parameters
         ----------
         path : str
             path where to save
         name : str
             name
         """
        timestamp = datetime.now().strftime("_%d_%m_%y_(%H_%M_%S)")
        if name[-4::] == ".png":
            name = name[:-4] + timestamp + ".png"
        else:
            name += timestamp + ".png"

        file_path = Path(path, name)
        self.driver.save_screenshot(str(file_path))
        logger_scraper_session.debug("screenshot saved in : " + str(file_path))

    def init_data(self):
        """info about the scraping task
        Returns
        -------
        init : dict
            A dictionary with the following key : values

                url      : str
                    A string representing the url being scraped
                id              : hash md5
                    hex A unique id that could help connecting info to other sources
                scraper  : str
                    the user to scrape the profile
                timestamp       : str
                    timestamp in which the profile has been scraped in the format "%d_%m_%Y_(%H_%M_%S)"
                version         : str
                    version of the package used for scraping
                browser_version : str
                    version of the browser used for scraping
                driver_version : str
                    version of the webdriver

        """

        # Init data dictionary
        logger_scraper_session.debug("session, started, init_data")
        self.init = {}
        self.data = {}
        # add url being scraped
        self.init.update({"url": self.url})

        # get hash of the url
        id_profile = hashlib.md5(self.url.encode())
        self.init.update({"id": id_profile.hexdigest()})

        # Add scraper
        self.init.update({"miner": self.config["miner"]})

        # Add the timestamp of the scraping session
        timestamp = datetime.now().strftime("%d_%m_%y_(%H_%M_%S)")
        self.init.update({"timestamp_start": self.timestamp_start})
        self.init.update({"timestamp_end": None})
        self.init.update({"file_name_source": self.file_name_source})
        self.init.update({"source_description": self.source_description})
        self.init.update({"tag": self.tag})
        self.init.update({"info": self.csv_info})
        # Add the package version
        self.init.update({"version": __about__.__version__})

        # Add browser version
        self.init.update({"browser_version": self.driver.capabilities['browserVersion']})
        # Add webdriver version
        self.init.update({"driver_version": self.driver.capabilities['chrome']['chromedriverVersion']})

        # Update person data with section info
        self.data.update({"init": self.init})
        logger_scraper_session.debug("session, done, init_data")
        return self.data

    def set_data_path(self, target, folder):
        """

        Parameters
        ----------
        target : str
            target to scrape: ['people','followers','following','connections','companies','employees','groups']
        folder : str
            path where the data of the session will be saved.
            "watchdog" : is the folder meant for saving data validation, routine check and tests
            "data : is the main folder where the data is saved
        """
        self.failed_json_path = Path(Path.home(), "brobygger_li", target, folder, "failed_json_validations")
        self.profiles_path = Path(Path.home(), "brobygger_li", target, folder, "profiles")
        self.html_path = Path(Path.home(), "brobygger_li", target, folder, "html")
        self.media_path = Path(Path.home(), "brobygger_li", target, folder, "media")
        self.screenshots_path = Path(Path.home(), "brobygger_li", target, folder, "screenshots")
        self.meta_data_path = Path(Path.home(), "brobygger_li", target, folder, "meta_data")

    def load_page(self, ava_method, ava_selector, not_ava_method, not_ava_selector, url_condition=""):
        """
        Parameters
        ----------
        ava_method : str
            selector type selenium.webdriver.common.by.By
        ava_selector : str
            value to look for
        not_ava_method : str
            selector type selenium.webdriver.common.by.By
        not_ava_selector : str
            value to look for
        url_condition : str
            a security to check that the url contains a certain pattern if not raises InvalidUrlError

        Check if the url is valid
        Raises
        ------
        InvalidUrlError
            An invalid url

        """
        self.reset_data_containers()
        logger_scraper_session.debug("session, started, load_page started " + self.url)
        # test
        if url_condition:
            if url_condition not in self.url:
                # test
                raise exceptions.InvalidUrlError("Invalid url: ", self.url)
        self.driver.get(self.url)
        # If the profile is available init the person_data otherwise raise error if it is found to be unavailable
        if self.config["login_settings"]["login_check_flag"]:
            self.check_login(
                self.config["login_settings"],
                html_name="check",
                html_path=self.html_path,
                screenshot_name="check",
                screenshot_path=self.screenshots_path
            )
        # check if the profile is available
        sel_el = self.select_element(method=ava_method,
                                     selector=ava_selector,
                                     timeout=15,
                                     screenshot=False,  # takes screenshot only when it fails
                                     screenshot_name="availability_check.png",
                                     screenshot_path=self.screenshots_path,
                                     html=False,
                                     html_name="availability_check",
                                     html_path=self.html_path
                                     )
        if sel_el:
            self.init_data()
            # TODO make external to leave session unreleated to linkedin
            if "/jobs/view/" in url_condition:
                self.click_on(By.CSS_SELECTOR, '.jobs-description footer button[aria-label="Click to see more description"]')
            # Scroll to the bottom of the page incrementally to load any lazy-loaded content
            if "/jobs/search/" not in url_condition:
                self.scroll_to_bottom(scroll_pause_min=self.config["driver"]["scroll_pause"]["min"],
                                      scroll_pause_max=self.config["driver"]["scroll_pause"]["max"],
                                      scroll_increment=self.config["driver"]["scroll_increment"])
            # Set the file name
            self.file_name = self.data["init"]["id"]
            if not self.data["init"]["timestamp_start"]:
                self.data["init"]["timestamp_start"] = datetime.now().strftime("%d_%m_%y_(%H_%M_%S)")
            self.file_name_ts = self.data["init"]["id"] + "_" + self.data["init"]["timestamp_start"]
            profile_folder = self.data["init"]["id"] + "_" + self.data["init"]["timestamp_start"]
            # Create a folder where the html with be saved
            # test
            Path(self.html_path, profile_folder).mkdir()
            # test
            Path(self.screenshots_path, profile_folder).mkdir()

            # Update the profile paths
            self.html_path_profile = Path(self.html_path, profile_folder)
            self.screenshots_path_profile = Path(self.screenshots_path, profile_folder)

            # save the source page loaded at the start
            # test
            mt.save_html(self.driver.page_source, self.html_path_profile, "start")

        elif sel_el is None:
            # check if the profile is not available
            sel_el = self.select_element(method=not_ava_method,
                                         selector=not_ava_selector,
                                         timeout=15,
                                         )
            # If I find the the condition that the profile is not available raise an error
            if sel_el:
                # test
                raise exceptions.UrlNotAvailable("url not available: ", self.url)
            else:
                raise ValueError("Error loading page")

    def save_html_and_png_full_page(self):
        """save the html and full page screenshot"""
        mt.save_html(self.driver.page_source, self.html_path_profile, "end")
        # Save full page screenshot
        ob = Screenshot_Clipping.Screenshot()
        timestamp = datetime.now().strftime("_%d_%m_%y_(%H_%M_%S)")
        ob.full_Screenshot(self.driver, save_path=str(self.screenshots_path_profile),
                           image_name='end' + timestamp + ".png")

    def get_keys(self, key_string, node_keys):
        """Returns a dictionary of the keys present in the node found through re
        Parameters
        ----------
        key_string : str
            a regular expression to search in the full page

        node_keys : list
            list of key values to return

        Returns
        -------
        node_data: dict , default None
            value of the key or none if not found

        """
        logger_scraper_session.debug("session, started, get_keys")
        self.full_page = self.select_element(
            method=By.CSS_SELECTOR,
            selector="html.artdeco",
            return_type="bs_el",
            html=False,
            screenshot=False
        )
        # Returns the node containing the text
        node_text = self.full_page.find_all(text=re.compile(key_string))
        # Convert to a dictionary
        node_data = json.loads(node_text[0])

        # Init
        node_info = {}

        for key in node_keys:
            # noinspection PyBroadException
            try:
                node_info.update({key: node_data["data"][key]})
            except:
                logger_scraper_session.exception("Exception to get node: ")
                node_info.update({key: None})
                pass

        logger_scraper_session.debug("session, done, get_keys")
        return node_info

    def click_on(self,
                 method,
                 selector,
                 web_element=None,
                 timeout=5,
                 html_name="",
                 html_path="",
                 screenshot_name="",
                 screenshot_path=""
                 ):
        """Returns a WebElement or bs4 element given the method and selector

        Parameters
        ----------
        method : str
            By selector type selenium.webdriver.common.by.By
        selector : str
            selector to look for
        timeout: int
            timeout to wait for the element to be selected
        web_element: WebElement
            use WebElement containing the click
        html_name: str
            path to save of the html file
        html_path: str
            path to save of the html file
        screenshot_name: str
            name to save screenshot
        screenshot_path: str
            path to save screenshot
        Returns
        -------
        bool
            True if click is successful otherwise False
        """
        logger_scraper_session.debug("Trying to click element: {} , {}".format(method, selector))
        # noinspection PyBroadException
        try:
            # Wait for the given conditions until element_present to occur before proceeding further with execution
            if web_element is None:
                web_element = self.driver
            self.scroll_to_element(method, selector, web_element)
            el = WebDriverWait(web_element, timeout).until(
                expected_conditions.element_to_be_clickable((method, selector)))

            logger_scraper_session.debug("element is ready , {} , {}".format(method, selector))
            el.click()
            logger_scraper_session.debug("click on element: {} , {}".format(method, selector))
            # Randomly sleep to scatter clicks
            mt.random_rest(self.config["driver"]["click_pause"]["min"],
                           self.config["driver"]["click_pause"]["max"])
            return True

        except:
            logger_scraper_session.exception("Click exception :")
            logger_scraper_session.warning(
                "failed click , {} , {}, url = {}".format(method, selector, self.driver.current_url))
            self.save_failed_element(html_path, html_name, screenshot_name, screenshot_path)
            raise

    def save_failed_element(self, html_path, html_name, screenshot_name, screenshot_path):
        """

        Parameters
        ----------
        html_name: str
            path to save of the html file
        html_path: str
            path to save of the html file
        screenshot_name: str
            name to save screenshot
        screenshot_path: str
            path to save screenshot

        """
        if html_path and html_name:
            logger_scraper_session.debug(
                "saving html from select element error: path = {} , name = {}".format(html_path, html_name))
            html_name = "error_" + html_name
            mt.save_html(self.driver.page_source, html_path, html_name)

        if screenshot_name and screenshot_path:
            logger_scraper_session.warning(
                "saving screenshot from select element error: path = {} , name = {}".format(html_path, html_name))
            screenshot_name = "error_" + screenshot_name
            self.save_screenshot(path=screenshot_path, name=screenshot_name)

    def expand_buttons(self, button_method, button_selector, list_method, list_selector, main_web_element=None):
        """Expands the buttons of see more"""
        while True:
            # noinspection PyBroadException
            try:
                # Wait for the  given conditions until element_present to occur before proceeding further with execution
                if main_web_element is None:
                    main_web_element = self.driver

                elements = self.select_elements(list_method, list_selector, main_web_element)
                buttons = self.select_elements(button_method, button_selector, main_web_element)
                if buttons is None:
                    break
                if elements:
                    logger_scraper_session.debug(
                        "elements where possible buttons could be, found :" + str(len(elements)))
                    logger_scraper_session.debug("buttons found so far :" + str(len(buttons)))

                    for i, element in enumerate(elements):
                        # noinspection PyBroadException
                        try:
                            self.click_on(button_method, button_selector, web_element=element)
                            logger_scraper_session.debug("Clicked on " + str(i))
                        except:
                            logger_scraper_session.debug(
                                "elements {} has no buttons or button failed to click".format(i))
                            pass
                else:
                    logger_scraper_session.debug(
                        "No more buttons to expand , " + button_method + " , " + button_selector)
                    break
            except Exception:
                logger_scraper_session.exception(
                    "Failed expandable buttons , " + button_method + " , " + button_selector)
                break

    def go_home(self):
        """Go to the home page and scroll a little"""
        self.driver.get("https://www.linkedin.com/")
        mt.random_rest(self.config["driver"]["home_pause"]["min"],
                       self.config["driver"]["home_pause"]["max"])

        # Scroll a little
        self.driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
        mt.random_rest(self.config["driver"]["home_pause"]["min"],
                       self.config["driver"]["home_pause"]["max"])

    def get_data_from_section(self,
                              section,
                              method, selector,
                              list_method, list_selector,
                              get_info_function,
                              ):
        """
        Parameters
        ----------
        section: str
            section name
        method : str
            By selector type selenium.webdriver.common.by.By
        selector : str
            selector to look for
        list_method : str
            The list to extract to which the function to get info will me mapped
        list_selector : str
            selector to look for
        get_info_function : Any
            function to map to the list

        Returns
        -------
        info : list of dict
        """
        logger_scraper_session.debug("session, started, " + section)
        # Init Experience list
        self.info = []

        # Scroll to Experiences section
        element = self.select_element(method, selector)
        # if the element is found
        if element:

            # save the html
            self.select_element(method, selector,
                                return_type="bs_el",
                                html_name=section,
                                html_path=self.html_path_profile,
                                screenshot_name=section,
                                screenshot_path=self.screenshots_path_profile,
                                )

            section_bs = self.select_elements(list_method, list_selector, return_type="bs_el")

            if section_bs:
                self.info = list(map(get_info_function, section_bs))
                self.info = mt.flatten_list(self.info)
            else:
                logger_scraper_session.debug("section bs, not found, " + section)
        else:
            logger_scraper_session.debug("section, not found, " + section)

        logger_scraper_session.debug("session, done, " + section)
        return self.info

    def get_data_from_pagination(self,
                                 section,
                                 method, selector,
                                 list_method, list_selector,
                                 get_info_function,
                                 page_nr_method, page_nr_selector,
                                 page_nr_attribute,
                                 tot_pages_method, tot_pages_selector,
                                 next_method, next_selector,
                                 info: List
                                 ):
        """
        Parameters
        ----------
        section : str
            name of the section being scraped eg "employees"
        method : str
            By selector type selenium.webdriver.common.by.By of the WebElement containing the info
        selector : str
            selector to look for of the WebElement containing the info
        list_method : str
            The list to extract to which the function to get info will me mapped
        list_selector : str
            selector to look for
        next_method : str
            By selector type selenium.webdriver.common.by.By for the clicking on the next page
        next_selector : str
            selector to look for the clicking on the next page
        get_info_function : Any
            function to map to the list
        page_nr_method : str
            By selector type selenium.webdriver.common.by.By to get page number
        page_nr_selector : str
            selector to look for to get page number
        page_nr_attribute : str
            html attribute indicating the number of the page
        tot_pages_method : str
            By selector type selenium.webdriver.common.by.By
        tot_pages_selector : str
            selector
        info : list
            data container


        Returns
        -------
        info : list of dict
        """
        # noinspection PyBroadException
        # page number
        self.scroll_to_bottom(scroll_pause_min=self.config["driver"]["scroll_pause"]["min"],
                              scroll_pause_max=self.config["driver"]["scroll_pause"]["max"],
                              scroll_increment=self.config["driver"]["scroll_increment"])
        try:
            page_nr = self.select_element(page_nr_method, page_nr_selector, return_type="sel_el").get_attribute(
                page_nr_attribute)
            logger_scraper_session.debug("getting page nr : " + page_nr )
            if not page_nr.isdigit():
                logger_scraper_session.debug("Page number is not a digit, set page_nr to 0")
                page_nr = '0'
        except:
            page_nr = '0'


        # tot pages
        try:
            tot_pages = self.select_elements(tot_pages_method, tot_pages_selector, return_type="sel_el")
            tot_pages = tot_pages[-1].get_attribute(page_nr_attribute)
            logger_scraper_session.debug("getting tot pages : "+  tot_pages)
            if not tot_pages.isdigit():
                logger_scraper_session.debug("Page number is not a digit, tot_page to 0")
                tot_pages = '0'
        except:
            tot_pages = '0'

        logger_scraper_session.debug("getting page nr : " + page_nr + "/" + tot_pages)
        self.get_meta_data(section+ "_" + str(page_nr))
        mt.save_html(self.driver.page_source, self.html_path_profile, "page_"+str(page_nr))
        info.extend(
            self.get_data_from_section(
                section=section + "_" + page_nr,
                method=method, selector=selector,
                list_method=list_method, list_selector=list_selector,
                get_info_function=get_info_function
            )
        )
        if tot_pages == '0' or page_nr == tot_pages:
            logger_scraper_session.debug("getting data from pagination done")
        else:
            # noinspection PyBroadException
            try:
                self.click_on(next_method, next_selector)
                self.get_data_from_pagination(
                    section,
                    method, selector,
                    list_method, list_selector,
                    get_info_function,
                    page_nr_method, page_nr_selector,
                    page_nr_attribute,
                    tot_pages_method, tot_pages_selector,
                    next_method, next_selector,
                    info
                )

            except:
                logger_scraper_session.debug("failed following pagination")

    def get_meta_data(self, key = None):
        """
        Parameters
        ----------
        key : str
            name of the key to add meta data
        """
        # get the full page source
        self.full_page = bs4.BeautifulSoup(self.driver.page_source, features="html.parser")
        # get the data
        self.meta_data = self.full_page.find_all(text=re.compile('{"data":'))

        if self.meta_data:
            logger_scraper_session.debug("found meta data")
            if key:
                self.meta_data_container.update({key:[json.loads(d) for d in self.meta_data]})

        else:
            logger_scraper_session.debug("failed getting meta data")


