"""
Title     : Validation tools
Objective :
Created by: Abdellah Janid
Created on: 6/1/20
"""
# Standard library imports
import json
import logging
from typing import Type, Union

# Related third party imports
from pydantic import ValidationError

# Import local applications
from scraper.configuration_schemas import LinkedinConfigSchema
from scraper.general_schemas import InitSchema
from scraper.linkedin.my_followers.schemas import LiMyFollowersSchema
from scraper.linkedin.people.schemas import LiPersonAllModel
from scraper.scraping_tools import save_json

logger_validation_tools = logging.getLogger('validation_tools')
logger_validation_tools.setLevel(logging.DEBUG)


def validate_schema(
        model: Type[Union[
            LinkedinConfigSchema,
            InitSchema, LiPersonAllModel, LiMyFollowersSchema]],
        data,
        failed_path,
        file_name) -> Union[LinkedinConfigSchema, InitSchema, LiPersonAllModel, LiMyFollowersSchema, bool]:
    """Validate a json schema

    Parameters
    ----------
    model : pydantic.main.BaseModel
        pydantic Schema Model
    data : dict
        data to validate
    failed_path: str
        path where to save failed validations
    file_name : str
        file name to save the failed validation in the failed_json_validations directory
    Returns
    -------
    data_results : pydantic.main.BaseModel
        returns a pydantic.main.ModelMetaclass or False in case the validation fails
    """
    try:
        data_results = model(**data)
        logger_validation_tools.info("json schema validated , {}".format(model.Config.title))
        return data_results

    except ValidationError as e:
        logger_validation_tools.error("json schema validation failed , {}".format(model.Config.title))
        # Save the data json
        save_json(data, failed_path, file_name, indent=4, separators=(',', ': '))
        # Save the data json of the error
        save_json(json.loads(e.json()), failed_path, "error_" + file_name, indent=4, separators=(',', ': '))
        raise
